@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import os
 import datetime
@@ -25,17 +23,99 @@ import logging
 
 logger = logging.getLogger(__name__)
 
+from stalker.log import logging_level
+logger.setLevel(logging_level)
 
-class Config(object):
+
+class ConfigBase(object):
     """Config abstraction
 
     Idea is coming from Sphinx config.
+    """
+    default_config_values = {}
 
-    Holds system wide configuration variables. See
+    def __init__(self):
+        self.config_values = self.default_config_values.copy()
+        self.user_config = {}
+
+        # the priority order is
+        # stalker.config
+        # config.py under .stalker_rc directory
+        # config.py under $STALKER_PATH
+
+        self._parse_settings()
+
+    def _parse_settings(self):
+        # for now just use $STALKER_PATH
+
+        # try to get the environment variable
+        if self.env_key not in os.environ:
+            # don't do anything
+            logger.debug("no environment key found for user settings")
+        else:
+            logger.debug("environment key found")
+
+            resolved_path = os.path.expanduser(
+                os.path.join(
+                    os.environ[self.env_key],
+                    "config.py"
+                )
+            )
+
+            # using `while` is not safe to expand variables
+            # so expand vars for 100 times which already is ridiculously
+            # complex
+            max_recursion = 100
+            i = 0
+            while '$' in resolved_path and i < max_recursion:
+                resolved_path = os.path.expandvars(
+                    resolved_path
+                )
+                i += 1
+
+            try:
+                logger.debug("importing user config")
+                with open(resolved_path) as f:
+                    exec(f.read(), self.user_config)
+            except IOError:
+                logger.warning("The $STALKER_PATH: %s doesn't exists! "
+                               "skipping user config" % resolved_path)
+            except SyntaxError as e:
+                raise RuntimeError(
+                    "There is a syntax error in your configuration file: %s" %
+                    str(e)
+                )
+            finally:
+                # append the data to the current settings
+                logger.debug("updating system config")
+                for key in self.user_config:
+                    #if key in self.config_values:
+                    self.config_values[key] = self.user_config[key]
+
+    def __getattr__(self, name):
+        return self.config_values[name]
+
+    def __getitem__(self, name):
+        return getattr(self, name)
+
+    def __setitem__(self, name, value):
+        return setattr(self, name, value)
+
+    def __delitem__(self, name):
+        # delattr(self, name)
+        self.config_values.pop(name)
+
+    def __contains__(self, name):
+        return name in self.config_values
+
+
+class Config(ConfigBase):
+    """Holds system wide configuration variables. See
     `configuring stalker`_ for more detail.
 
     .. _configuring stalker: ../configure.html
     """
+    env_key = "STALKER_PATH"
 
     default_config_values = dict(
 
@@ -44,7 +124,7 @@ class Config(object):
         # for possible parameters
         #
         database_engine_settings={
-            "sqlalchemy.url": "sqlite:///:memory:",
+            "sqlalchemy.url": "sqlite://",
             "sqlalchemy.echo": False,
         },
 
@@ -56,6 +136,9 @@ class Config(object):
         # Storage for uploaded files
         server_side_storage_path=os.path.expanduser('~/Stalker_Storage'),
 
+        repo_env_var_template='REPO%(code)s',
+        repo_env_var_template_old='REPO%(id)s',
+
         #
         # Tells Stalker to create an admin by default
         #
@@ -217,6 +300,18 @@ class Config(object):
             'STOP',
             'CMPL'
         ],
+    
+        project_status_names=[
+            'Ready To Start',
+            'Work In Progress',
+            'Completed'
+        ],
+
+        project_status_codes=[
+            'RTS',
+            'WIP',
+            'CMPL'
+        ],
 
         review_status_names=[
             'New',
@@ -266,7 +361,7 @@ class Config(object):
         tjp_studio_template="""
 project {{ studio.tjp_id }} "{{ studio.tjp_id }}" {{ studio.start.date() }} - {{ studio.end.date() }} {
     timingresolution {{ '%i'|format((studio.timing_resolution.days * 86400 + studio.timing_resolution.seconds)//60|int) }}min
-    now {{ studio.now.strftime('%Y-%m-%d-%H:%M') }}
+    now {{ studio.now.astimezone(utc).strftime('%Y-%m-%d-%H:%M') }}
     dailyworkinghours {{ studio.daily_working_hours }}
     weekstartsmonday
 {{ studio.working_hours.to_tjp }}
@@ -307,10 +402,10 @@ task {{task.tjp_id}} "{{task.tjp_id}}" {
 
             {% if task.schedule_constraint %}
                 {% if task.schedule_constraint == 1 or task.schedule_constraint == 3 %}
-                    start {{ task.start.strftime('%Y-%m-%d-%H:%M') }}
+                    start {{ task.start.astimezone(utc).strftime('%Y-%m-%d-%H:%M') }}
                 {% endif %}
                 {% if task.schedule_constraint == 2 or task.schedule_constraint == 3 -%}
-                    end {{ task.end.strftime('%Y-%m-%d-%H:%M') }}
+                    end {{ task.end.astimezone(utc).strftime('%Y-%m-%d-%H:%M') }}
                 {% endif %}
             {% endif %}
 
@@ -334,7 +429,7 @@ task {{task.tjp_id}} "{{task.tjp_id}}" {
             {%- endfor %}
         {%- endif %}
         {% for time_log in task.time_logs %}
-            booking {{time_log.resource.tjp_id}} {{time_log.start.strftime('%Y-%m-%d-%H:%M:%S')}} - {{time_log.end.strftime('%Y-%m-%d-%H:%M:%S')}} { overtime 2 }
+            booking {{time_log.resource.tjp_id}} {{time_log.start.astimezone(utc).strftime('%Y-%m-%d-%H:%M:%S')}} - {{time_log.end.astimezone(utc).strftime('%Y-%m-%d-%H:%M:%S')}} { overtime 2 }
         {% endfor %}
     {% endif %}
 
@@ -351,7 +446,7 @@ resource {{department.tjp_id}} "{{department.tjp_id}}" {
 }
 """,
 
-        tjp_vacation_template='''vacation {{ vacation.start.strftime('%Y-%m-%d-%H:%M:%S') }} - {{ vacation.end.strftime('%Y-%m-%d-%H:%M:%S') }}''',
+        tjp_vacation_template='''vacation {{ vacation.start.astimezone(utc).strftime('%Y-%m-%d-%H:%M:%S') }} - {{ vacation.end.astimezone(utc).strftime('%Y-%m-%d-%H:%M:%S') }}''',
 
         tjp_user_template='''resource {{user.tjp_id}} "{{user.tjp_id}}" {
     efficiency {{user.efficiency}}
@@ -380,7 +475,7 @@ resource {{department.tjp_id}} "{{department.tjp_id}}" {
         {% endfor %}
 
         # reports
-        taskreport breakdown "{{csv_file_full_path}}"{
+        taskreport breakdown "{{csv_file_name}}"{
             formats csv
             timeformat "%Y-%m-%d-%H:%M"
             columns id, start, end {%- if compute_resources %}, resources{% endif %}
@@ -404,13 +499,13 @@ resource {{department.tjp_id}} "{{department.tjp_id}}" {
 {{ tasks_buffer }}
 
 # reports
-taskreport breakdown "{{csv_file_full_path}}"{
+taskreport breakdown "{{csv_file_name}}"{
     formats csv
     timeformat "%Y-%m-%d-%H:%M"
     columns id, start, end {%- if compute_resources %}, resources{% endif %}
 }""",
 
-        tj_command='/usr/local/bin/tj3',
+        tj_command='tj3' if os.name == 'nt' else '/usr/local/bin/tj3',
 
         path_template='{{project.code}}/{%- for parent_task in parent_tasks -%}{{parent_task.nice_name}}/{%- endfor -%}',
         filename_template='{{task.entity_type}}_{{task.id}}_{{version.take_name}}_v{{"%03d"|format(version.version_number)}}',
@@ -467,84 +562,3 @@ taskreport breakdown "{{csv_file_full_path}}"{
         thumbnail_quality=70,
         thumbnail_size=[320, 180],
     )
-
-    def __init__(self):
-        self.config_values = Config.default_config_values.copy()
-        self.user_config = {}
-
-        # the priority order is
-        # stalker.config
-        # config.py under .stalker_rc directory
-        # config.py under $STALKER_PATH
-
-        self._parse_settings()
-
-    def _parse_settings(self):
-        # for now just use $STALKER_PATH
-        env_key = "STALKER_PATH"
-
-        # try to get the environment variable
-        if env_key not in os.environ:
-            # don't do anything
-            logger.debug("no environment key found for user settings")
-        else:
-            logger.debug("environment key found")
-
-            resolved_path = os.path.expanduser(
-                os.path.join(
-                    os.environ[env_key],
-                    "config.py"
-                )
-            )
-
-            # using `while` is not safe to expand variables
-            # do the expansion for 5 times which is complex enough
-            # and I don't (hopefully) expect anybody to use
-            # more than 5 level deep environment variables
-            resolved_path = os.path.expandvars(
-                os.path.expandvars(
-                    os.path.expandvars(
-                        os.path.expandvars(
-                            resolved_path
-                        )
-                    )
-                )
-            )
-
-            try:
-                logger.debug("importing user config")
-                with open(resolved_path) as f:
-                    exec(f.read(), self.user_config)
-            except IOError:
-                logger.warning("The $STALKER_PATH: %s doesn't exists! "
-                               "skipping user config" % resolved_path)
-            except SyntaxError as e:
-                raise RuntimeError(
-                    "There is a syntax error in your configuration file: %s" %
-                    str(e)
-                )
-            finally:
-                # append the data to the current settings
-                logger.debug("updating system config")
-                for key in self.user_config:
-                    #if key in self.config_values:
-                    self.config_values[key] = self.user_config[key]
-
-    def __getattr__(self, name):
-        return self.config_values[name]
-
-    def __getitem__(self, name):
-        return getattr(self, name)
-
-    def __setitem__(self, name, value):
-        return setattr(self, name, value)
-
-    def __delitem__(self, name):
-        delattr(self, name)
-
-    def __contains__(self, name):
-        return name in self.config_values
-
-
-# use this instance
-defaults = Config()
