@@ -1,41 +1,36 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 """Database module of Stalker.
 
 Whenever stalker.db or something under it imported, the
 :func:`stalker.db.setup` becomes available to let one setup the database.
 """
 
-import logging
 
-from sqlalchemy import engine_from_config
-from sqlalchemy.exc import IntegrityError
-
-from stalker import defaults
-from stalker.db.declarative import Base
-from stalker.db.session import DBSession
 from stalker.log import logging_level
 
+import logging
 logger = logging.getLogger(__name__)
 logger.setLevel(logging_level)
 
+# TODO: Try to get it from the API (it was not working inside a package before)
+alembic_version = 'bf67e6a234b4'
+
 
 def setup(settings=None):
     """Utility function that helps to connect the system to the given database.
@@ -50,38 +45,52 @@ def setup(settings=None):
     """
 
     if settings is None:
+        from stalker import defaults
         settings = defaults.database_engine_settings
-        logger.debug('no settings given, using the default: %s' % settings)
+        logger.debug('no settings given, using the default setting')
 
-    logger.debug("settings: %s" % settings)
+    # logger.debug("settings: %s" % settings)
     # create engine
+    from sqlalchemy import engine_from_config
     engine = engine_from_config(settings, 'sqlalchemy.')
 
     logger.debug('engine: %s' % engine)
 
     # create the Session class
+    from stalker.db.session import DBSession
     DBSession.remove()
     DBSession.configure(
         bind=engine,
         extension=None
     )
 
+    # check alembic versions of the database
+    # and raise an error if it is not matching with the system
+    check_alembic_version()
+
     # create the database
     logger.debug("creating the tables")
+    from stalker.db.declarative import Base
     Base.metadata.create_all(engine)
+    DBSession.commit()
 
     # update defaults
     update_defaults_with_studio()
 
+    # create repo env variables
+    create_repo_vars()
+
 
 def update_defaults_with_studio():
     """updates the default values from Studio instance if a database and a
     Studio instance is present
     """
+    from stalker.db.session import DBSession
     if DBSession:
         with DBSession.no_autoflush:
             from stalker.models.studio import Studio
-            studio = Studio.query.first()
+            # studio = Studio.query.first()
+            studio = DBSession.query(Studio).first()
             if studio:
                 logger.debug('found a studio, updating defaults')
                 studio.update_defaults()
@@ -94,13 +103,13 @@ def init():
 
     # register all Actions available for all SOM classes
     class_names = [
-        'Asset', 'Budget', 'BudgetEntry', 'Client', 'Daily', 'Department',
-        'Entity', 'FilenameTemplate', 'Good', 'Group', 'ImageFormat', 'Link',
-        'Message', 'Note', 'Page', 'Permission', 'PriceList', 'Project',
-        'Repository', 'Review', 'Role', 'Scene', 'Sequence', 'Shot',
-        'SimpleEntity', 'Status', 'StatusList', 'Structure', 'Studio', 'Tag',
-        'Task', 'Ticket', 'TicketLog', 'TimeLog', 'Type', 'User', 'Vacation',
-        'Version'
+        'Asset', 'AuthenticationLog', 'Budget', 'BudgetEntry', 'Client',
+        'Daily', 'Department', 'Entity', 'EntityGroup', 'FilenameTemplate',
+        'Good', 'Group', 'ImageFormat', 'Invoice', 'Link', 'Message', 'Note',
+        'Page', 'Payment', 'Permission', 'PriceList', 'Project', 'Repository',
+        'Review', 'Role', 'Scene', 'Sequence', 'Shot', 'SimpleEntity',
+        'Status', 'StatusList', 'Structure', 'Studio', 'Tag', 'Task', 'Ticket',
+        'TicketLog', 'TimeLog', 'Type', 'User', 'Vacation', 'Version'
     ]
 
     for class_name in class_names:
@@ -115,8 +124,10 @@ def init():
         register(class_)
 
     # create the admin if needed
+    admin = None
+    from stalker import defaults
     if defaults.auto_create_admin:
-        __create_admin__()
+        admin = __create_admin__()
 
     # create statuses
     create_ticket_statuses()
@@ -125,32 +136,44 @@ def init():
     create_entity_statuses(
         entity_type='Daily',
         status_names=defaults.daily_status_names,
-        status_codes=defaults.daily_status_codes
+        status_codes=defaults.daily_status_codes,
+        user=admin
+    )
+    create_entity_statuses(
+        entity_type='Project',
+        status_names=defaults.project_status_names,
+        status_codes=defaults.project_status_codes,
+        user=admin
     )
     create_entity_statuses(
         entity_type='Task',
         status_names=defaults.task_status_names,
-        status_codes=defaults.task_status_codes
+        status_codes=defaults.task_status_codes,
+        user=admin
     )
     create_entity_statuses(
         entity_type='Asset',
         status_names=defaults.task_status_names,
-        status_codes=defaults.task_status_codes
+        status_codes=defaults.task_status_codes,
+        user=admin
     )
     create_entity_statuses(
         entity_type='Shot',
         status_names=defaults.task_status_names,
-        status_codes=defaults.task_status_codes
+        status_codes=defaults.task_status_codes,
+        user=admin
     )
     create_entity_statuses(
         entity_type='Sequence',
         status_names=defaults.task_status_names,
-        status_codes=defaults.task_status_codes
+        status_codes=defaults.task_status_codes,
+        user=admin
     )
     create_entity_statuses(
         entity_type='Review',
         status_names=defaults.review_status_names,
-        status_codes=defaults.review_status_codes
+        status_codes=defaults.review_status_codes,
+        user=admin
     )
 
     # create alembic revision table
@@ -159,12 +182,66 @@ def init():
     logger.debug('finished initializing the database')
 
 
+def create_repo_vars():
+    """creates environment variables for all of the repositories in the current
+    database
+    """
+    # get all the repositories
+    import os
+    from stalker import defaults, Repository
+    all_repos = Repository.query.all()
+    for repo in all_repos:
+        os.environ[repo.env_var] = repo.path
+
+        # TODO: Remove this in upcoming versions.
+        # This is added for backwards compatibility
+        os.environ[
+            defaults.repo_env_var_template_old % {'id': repo.id}] = \
+            repo.path
+
+
+def get_alembic_version():
+    """returns the alembic version of the database
+    """
+    # try to query the version value
+    from stalker.db.session import DBSession
+    conn = DBSession.connection()
+    engine = conn.engine
+    if engine.dialect.has_table(conn, 'alembic_version'):
+        sql_query = 'select version_num from alembic_version'
+        from sqlalchemy.exc import OperationalError, ProgrammingError
+        try:
+            return DBSession.connection().execute(sql_query).fetchone()[0]
+        except (OperationalError, ProgrammingError, TypeError):
+            DBSession.rollback()
+            return None
+    else:
+        return None
+
+
+def check_alembic_version():
+    """checks the alembic version of the database and raise a ValueError if it
+    is not matching with this version of Stalker
+    """
+    current_alembic_version = get_alembic_version()
+    logger.debug('current_alembic_version: %s' % current_alembic_version)
+    if current_alembic_version and current_alembic_version != alembic_version:
+        # invalidate the connection
+        from stalker.db.session import DBSession
+        DBSession.connection().invalidate()
+
+        # and raise a ValueError (which I'm not sure is the correct exception)
+        raise ValueError(
+            'Please update the database to version: %s' % alembic_version
+        )
+
+
 def create_alembic_table():
     """creates the default alembic_version table and creates the data so that
     any new database will be considered as the latest version
     """
-    # Now, this is not the correct way of doing, there is a proper way of doing
-    # it and it is explained nicely in the Alembic library documentation.
+    # Now, this is not the correct way of doing this, there is a proper way of
+    # doing it and it is explained nicely in the Alembic library documentation.
     #
     # But it is simply not working when Stalker is installed as a package.
     #
@@ -172,16 +249,18 @@ def create_alembic_table():
     # don't forget to update the version_num (and the corresponding test
     # whenever a new alembic revision is created)
 
-    version_num = '856e70016b2'
+    version_num = alembic_version
 
     from sqlalchemy import Table, Column, Text
 
     table_name = 'alembic_version'
 
+    from stalker.db.session import DBSession
     conn = DBSession.connection()
     engine = conn.engine
 
     # check if the table is already there
+    from stalker.db.declarative import Base
     table = Table(
         table_name, Base.metadata,
         Column('version_num', Text),
@@ -215,16 +294,10 @@ def create_alembic_table():
 def __create_admin__():
     """creates the admin
     """
+    from stalker import defaults
     from stalker.models.auth import User
     from stalker.models.department import Department
 
-    # check if there is already an admin in the database
-    admin = User.query.filter_by(name=defaults.admin_name).first()
-    if admin:
-        #there should be an admin user do nothing
-        logger.debug("there is an admin already")
-        return
-
     logger.debug("creating the default administrator user")
 
     # create the admin department
@@ -234,6 +307,7 @@ def __create_admin__():
 
     if not admin_department:
         admin_department = Department(name=defaults.admin_department_name)
+        from stalker.db.session import DBSession
         DBSession.add(admin_department)
         # create the admins group
 
@@ -247,39 +321,42 @@ def __create_admin__():
         admins_group = Group(name=defaults.admin_group_name)
         DBSession.add(admins_group)
 
-    # # create the admin user
-    # admin = User.query \
-    #     .filter_by(name=defaults.admin_name) \
-    #     .first()
+    # check if there is already an admin in the database
+    admin = User.query.filter_by(name=defaults.admin_name).first()
+    if admin:
+        # there should be an admin user do nothing
+        logger.debug("there is an admin already")
+        return admin
+    else:
+        admin = User(
+            name=defaults.admin_name,
+            login=defaults.admin_login,
+            password=defaults.admin_password,
+            email=defaults.admin_email,
+            departments=[admin_department],
+            groups=[admins_group]
+        )
 
-    # if not admin:
-    admin = User(
-        name=defaults.admin_name,
-        login=defaults.admin_login,
-        password=defaults.admin_password,
-        email=defaults.admin_email,
-        departments=[admin_department],
-        groups=[admins_group]
-    )
+        admin.created_by = admin
+        admin.updated_by = admin
 
-    admin.created_by = admin
-    admin.updated_by = admin
+        # update the department as created and updated by admin user
+        admin_department.created_by = admin
+        admin_department.updated_by = admin
 
-    # update the department as created and updated by admin user
-    admin_department.created_by = admin
-    admin_department.updated_by = admin
+        admins_group.created_by = admin
+        admins_group.updated_by = admin
 
-    admins_group.created_by = admin
-    admins_group.updated_by = admin
+        DBSession.add(admin)
+        DBSession.commit()
 
-    DBSession.add(admin)
-    DBSession.commit()
+    return admin
 
 
 def create_ticket_statuses():
     """creates the default ticket statuses
     """
-    from stalker import User
+    from stalker import defaults, User
 
     # create as admin
     admin = User.query.filter(User.login == defaults.admin_name).first()
@@ -299,6 +376,7 @@ def create_ticket_statuses():
 
     # create Ticket Types
     logger.debug("Creating Ticket Types")
+    from stalker.db.session import DBSession
     if 'Defect' not in t_names:
         ticket_type_1 = Type(
             name='Defect',
@@ -318,6 +396,8 @@ def create_ticket_statuses():
             updated_by=admin
         )
         DBSession.add(ticket_type_2)
+
+    from sqlalchemy.exc import IntegrityError
     try:
         DBSession.commit()
     except IntegrityError:
@@ -352,6 +432,7 @@ def create_entity_statuses(entity_type='', status_names=None,
     status_names_in_db = list(map(lambda x: x.name, statuses))
     logger.debug('statuses_names_in_db: %s' % status_names_in_db)
 
+    from stalker.db.session import DBSession
     for name, code in zip(status_names, status_codes):
         if name not in status_names_in_db:
             logger.debug('Creating Status: %s (%s)' % (name, code))
@@ -388,6 +469,7 @@ def create_entity_statuses(entity_type='', status_names=None,
     status_list.statuses = statuses
     DBSession.add(status_list)
 
+    from sqlalchemy.exc import IntegrityError
     try:
         DBSession.commit()
     except IntegrityError as e:
@@ -401,8 +483,8 @@ def create_entity_statuses(entity_type='', status_names=None,
 def register(class_):
     """Registers the given class to the database.
 
-    It is mainly used to create the :class:`.Action`\ s needed for the
-    :class:`.User`\ s and :class:`.Group`\ s to be able to interact with the
+    It is mainly used to create the :class:`.Action` s needed for the
+    :class:`.User` s and :class:`.Group` s to be able to interact with the
     given class. Whatever class you have created needs to be registered.
 
     Example, lets say that you have a data class which is specific to your
@@ -428,16 +510,16 @@ def register(class_):
 
     Now because Stalker is using Pyramid authorization mechanism it needs to be
     able to have an :class:`.Permission` about your new class, so you can
-    assign this :class;`.Permission` to your :class:`.User`\ s or
-    :class:`.Group`\ s. So you ned to register your new class with
+    assign this :class;`.Permission` to your :class:`.User` s or
+    :class:`.Group` s. So you ned to register your new class with
     :func:`stalker.db.register` like shown below::
 
       from stalker import db
       db.register(MyDataClass)
 
     This will create the necessary Actions in the 'Actions' table on your
-    database, then you can create :class:`.Permission`\ s and assign these to
-    your :class:`.User`\ s and :class:`.Group`\ s so they are Allowed or Denied
+    database, then you can create :class:`.Permission` s and assign these to
+    your :class:`.User` s and :class:`.Group` s so they are Allowed or Denied
     to do the specified Action.
 
     :param class_: The class itself that needs to be registered.
@@ -456,6 +538,7 @@ def register(class_):
 
     class_name = class_.__name__
 
+    from stalker.db.session import DBSession
     if not EntityType.query.filter_by(name=class_name).first():
         new_entity_type = EntityType(class_name)
         # update attributes
@@ -470,15 +553,15 @@ def register(class_):
 
         DBSession.add(new_entity_type)
 
+    from stalker import defaults
     for action in defaults.actions:
         for access in ['Allow', 'Deny']:
             permission_obj = Permission(access, action, class_name)
             if permission_obj not in permissions_db:
                 DBSession.add(permission_obj)
 
+    from sqlalchemy.exc import IntegrityError
     try:
         DBSession.commit()
     except IntegrityError:
         DBSession.rollback()
-    # else:
-    #     DBSession.flush()
