@@ -1,25 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-import calendar
-import datetime
-from stalker.exceptions import CircularDependencyError
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 
 def make_plural(name):
@@ -74,14 +69,15 @@ def check_circular_dependency(entity, other_entity, attr_name):
     """
     for e in walk_hierarchy(entity, attr_name):
         if e is other_entity:
+            from stalker.exceptions import CircularDependencyError
             raise CircularDependencyError(
                 '%(entity_name)s (%(entity_class)s) and '
                 '%(other_entity_name)s (%(other_entity_class)s) creates a '
-                'circular dependency in their %(attr_name)s attribute' %
+                'circular dependency in their "%(attr_name)s" attribute' %
                 {
-                    'entity_name': entity.name.__str__,
+                    'entity_name': entity,
                     'entity_class': entity.__class__.__name__,
-                    'other_entity_name': other_entity.__str__,
+                    'other_entity_name': other_entity,
                     'other_entity_class': other_entity.__class__.__name__,
                     'attr_name': attr_name
                 }
@@ -95,6 +91,8 @@ def utc_to_local(utc_dt):
     http://stackoverflow.com/questions/4563272/how-to-convert-a-python-utc-datetime-to-a-local-datetime-using-only-python-stand/13287083#13287083
     """
     # get integer timestamp to avoid precision lost
+    import calendar
+    import datetime
     timestamp = calendar.timegm(utc_dt.timetuple())
     local_dt = datetime.datetime.fromtimestamp(timestamp)
     return local_dt.replace(microsecond=utc_dt.microsecond)
