@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 from sqlalchemy import Column, Integer, ForeignKey
 from stalker.models.task import Task
@@ -32,13 +30,13 @@ logger.setLevel(logging_level)
 class Asset(Task, CodeMixin):
     """The Asset class is the whole idea behind Stalker.
 
-    *Assets* are containers of :class:`.Task`\ s. And :class:`.Task`\ s are the
+    *Assets* are containers of :class:`.Task` s. And :class:`.Task` s are the
     smallest meaningful part that should be accomplished to complete the
     :class:`.Project`.
 
     An example could be given as follows; you can create an asset for one of
     the characters in your project. Than you can divide this character asset in
-    to :class:`.Task`\ s. These :class:`.Task`\ s can be defined by the type of
+    to :class:`.Task` s. These :class:`.Task` s can be defined by the type of
     the :class:`.Asset`, which is a :class:`.Type` object created specifically
     for :class:`.Asset` (ie. has its :attr:`.Type.target_entity_type` set to
     "Asset"),
@@ -81,4 +79,3 @@ class Asset(Task, CodeMixin):
         """the overridden __hash__ method
         """
         return super(Asset, self).__hash__()
-
