@@ -1,39 +1,39 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
+import sys
 import os
 import json
 import re
 import base64
 import datetime
+import pytz
 
-from sqlalchemy import (Table, Column, Integer, ForeignKey, String, DateTime,
-                        Enum, Float)
+from sqlalchemy import (Table, Column, Integer, ForeignKey, String, Enum,
+                        Float)
 from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.orm import relationship, synonym, validates
 from sqlalchemy.schema import UniqueConstraint
 
-from stalker import defaults
 from stalker.db.declarative import Base
+from stalker.db.types import GenericDateTime
 from stalker.models.mixins import ACLMixin
-from stalker.models.entity import Entity
+from stalker.models.entity import Entity, SimpleEntity
 from stalker.log import logging_level
 
 import logging
@@ -41,6 +41,10 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging_level)
 
 
+LOGIN = 'login'
+LOGOUT = 'logout'
+
+
 class Permission(Base):
     """A class to hold permissions.
 
@@ -98,9 +102,9 @@ class Permission(Base):
 
       # get the permissions for the Project class
       project_permissions = Permission.query\
-          .filter(Actions.access='Allow')\
-          .filter(Actions.action='Create')\
-          .filter(Actions.class_name='Project')\
+          .filter(Permission.access='Allow')\
+          .filter(Permission.action='Create')\
+          .filter(Permission.class_name='Project')\
           .first()
 
       # now we have the permission specifying the allowance of creating a
@@ -114,6 +118,10 @@ class Permission(Base):
       DBSession.add(group)
       DBSession.commit()
     """
+    from stalker import defaults  # I know it is a weird place to import but we
+                                  # need to do it in that way to limit the
+                                  # import statement with a scope, because the
+                                  # default can be updated
     __tablename__ = 'Permissions'
     __table_args__ = (
         UniqueConstraint('access', 'action', 'class_name'),
@@ -123,7 +131,7 @@ class Permission(Base):
     id = Column(Integer, primary_key=True)
     _access = Column('access', Enum('Allow', 'Deny', name='AccessNames'))
     _action = Column('action',
-                     Enum(*defaults.actions, name='ActionNames'))
+                     Enum(*defaults.actions, name='AuthenticationActions'))
     _class_name = Column('class_name', String(32))
 
     def __init__(self, access, action, class_name):
@@ -131,6 +139,11 @@ class Permission(Base):
         self._action = self._validate_action(action)
         self._class_name = self._validate_class_name(class_name)
 
+    def __hash__(self):
+        """returns the hash value for this instance
+        """
+        return hash(self.access + self.action + self.class_name)
+
     def _validate_access(self, access):
         """validates the given access value
         """
@@ -190,9 +203,10 @@ class Permission(Base):
                 (self.__class__.__name__, action.__class__.__name__)
             )
 
+        from stalker import defaults
         if action not in defaults.actions:
             raise ValueError(
-                '%s.action should be one of the values of %s not %s' %
+                "%s.action should be one of the values of %s not '%s'" %
                 (self.__class__.__name__, defaults.actions, action)
             )
 
@@ -217,7 +231,7 @@ class Permission(Base):
 class Group(Entity, ACLMixin):
     """Creates groups for users to be used in authorization system.
 
-    A Group instance is nothing more than a list of :class:`.User`\ s created
+    A Group instance is nothing more than a list of :class:`.User` s created
     to be able to assign permissions in a group level.
 
     The Group class, as with the :class:`.User` class, is mixed with the
@@ -246,13 +260,18 @@ class Group(Entity, ACLMixin):
         """
     )
 
-    def __init__(self, name='', users=None, **kwargs):
+    def __init__(self, name='', users=None, permissions=None, **kwargs):
         if users is None:
             users = []
+
+        if permissions is None:
+            permissions = []
+
         kwargs.update({'name': name})
         super(Group, self).__init__(**kwargs)
 
         self.users = users
+        self.permissions = permissions
 
     @validates('users')
     def _validate_users(self, key, user):
@@ -346,7 +365,7 @@ class User(Entity, ACLMixin):
       should be a list of Department objects. One user can be listed in
       multiple departments.
 
-    :type departments: list of :class:`.Department`\ s
+    :type departments: list of :class:`.Department` s
 
     :param password: it is the password of the user, can contain any character.
       Stalker doesn't store the raw passwords of the users. To check a stored
@@ -363,7 +382,7 @@ class User(Entity, ACLMixin):
     :param tasks: it is a list of Task objects which holds the tasks that this
       user has been assigned to
 
-    :type tasks: list of :class:`.Task`\ s
+    :type tasks: list of :class:`.Task` s
 
     :param last_login: it is a datetime.datetime object holds the last login
       date of the user (not implemented yet)
@@ -374,8 +393,12 @@ class User(Entity, ACLMixin):
     __tablename__ = "Users"
     __mapper_args__ = {"polymorphic_identity": "User"}
 
-    user_id = Column("id", Integer, ForeignKey("Entities.id"),
-                     primary_key=True)
+    user_id = Column(
+        "id",
+        Integer,
+        ForeignKey("Entities.id"),
+        primary_key=True
+    )
 
     departments = association_proxy(
         'department_role',
@@ -388,7 +411,7 @@ class User(Entity, ACLMixin):
         back_populates='user',
         cascade='all, delete-orphan',
         primaryjoin='Users.c.id==Department_Users.c.uid',
-        doc="""A list of :class:`.Department`\ s that
+        doc="""A list of :class:`.Department` s that
         this user is a part of"""
     )
 
@@ -401,8 +424,9 @@ class User(Entity, ACLMixin):
     company_role = relationship(
         "ClientUser",
         back_populates="user",
+        cascade='all, delete-orphan',
         primaryjoin="Users.c.id==Client_Users.c.uid",
-        doc="""A list of :class:`.Client`\ s that this user is a part of."""
+        doc="""A list of :class:`.Client` s that this user is a part of."""
     )
 
     email = Column(
@@ -421,13 +445,6 @@ class User(Entity, ACLMixin):
         """
     )
 
-    last_login = Column(
-        DateTime,
-        doc="""The last login time of this user.
-
-        It is an instance of datetime.datetime class."""
-    )
-
     login = Column(
         String(256),
         nullable=False,
@@ -438,6 +455,16 @@ class User(Entity, ACLMixin):
         """
     )
 
+    authentication_logs = relationship(
+        "AuthenticationLog",
+        primaryjoin="AuthenticationLogs.c.uid==Users.c.id",
+        back_populates="user",
+        cascade='all, delete-orphan',
+        doc="""A list of :class:`.AuthenticationLog` instances which holds the
+        login/logout info for this :class:`.User`.
+        """
+    )
+
     groups = relationship(
         'Group',
         secondary='Group_Users',
@@ -465,7 +492,7 @@ class User(Entity, ACLMixin):
         "Task",
         secondary="Task_Resources",
         back_populates="resources",
-        doc=""":class:`.Task`\ s assigned to this user.
+        doc=""":class:`.Task` s assigned to this user.
 
         It is a list of :class:`.Task` instances.
         """
@@ -475,7 +502,7 @@ class User(Entity, ACLMixin):
         'Task',
         secondary='Task_Watchers',
         back_populates='watchers',
-        doc=''':class:`.Tasks`\ s that this user is
+        doc=''':class:`.Tasks` s that this user is
         assigned as a watcher.
 
         It is a list of :class:`.Task` instances.
@@ -560,7 +587,6 @@ class User(Entity, ACLMixin):
 
         self.tasks = []
 
-        self.last_login = None
         self.efficiency = efficiency
         self.rate = rate
 
@@ -634,37 +660,24 @@ class User(Entity, ACLMixin):
 
         if len_splits < 2:
             raise ValueError(
-                "check the formatting %s.email, there are no @ sign" %
+                "check the formatting of %s.email, there is no @ sign" %
                 self.__class__.__name__
             )
 
         if splits[0] == "":
             raise ValueError(
-                "check the formatting %s.email, the name part is missing" %
+                "check the formatting of %s.email, the name part is missing" %
                 self.__class__.__name__
             )
 
         if splits[1] == "":
             raise ValueError(
-                "check the %s.email formatting, the domain part is missing" %
+                "check the formatting %s.email, the domain part is missing" %
                 self.__class__.__name__
             )
 
         return email_in
 
-    @validates("last_login")
-    def _validate_last_login(self, key, last_login_in):
-        """validates the given last_login argument
-        """
-        if not isinstance(last_login_in, datetime.datetime) \
-           and last_login_in is not None:
-            raise TypeError(
-                "%s.last_login should be an instance of datetime.datetime or "
-                "None not %s" %
-                (self.__class__.__name__, last_login_in.__class__.__name__)
-            )
-        return last_login_in
-
     @classmethod
     def _format_login(cls, login):
         """formats the given login value
@@ -689,6 +702,11 @@ class User(Entity, ACLMixin):
     @validates("password")
     def _validate_password(self, key, password_in):
         """validates the given password
+
+        Note:
+            This function was updated to support both Python 2.7 and 3.5.
+            It will now explicitly convert the base64 bytes object into
+            a string object.
         """
         if password_in is None:
             raise TypeError(
@@ -696,22 +714,46 @@ class User(Entity, ACLMixin):
             )
 
         if password_in == "":
-            raise ValueError("raw_password can not be empty string")
+            raise ValueError(
+                "%s.password can not be an empty string" %
+                self.__class__.__name__
+            )
 
         # mangle the password
-        return base64.b64encode(password_in.encode('utf-8'))
+        mangled_password_bytes = base64.b64encode(password_in.encode('utf-8'))
+
+        if sys.version_info[0] == 2:
+            mangled_password_str = str(mangled_password_bytes)
+        else:
+            # Assuming Python >= 3.5
+            mangled_password_str = \
+                str(mangled_password_bytes.decode('utf-8'))
+
+        return mangled_password_str
 
     def check_password(self, raw_password):
         """Checks the given raw_password.
 
-        Checks the given raw_password with the current Users objects encrypted
-        password.
+        Checks the given raw_password with the current User object's mangled
+        password. Handles the encryption process behind the scene.
 
-        Checks the given raw password with the given encrypted password.
-        Handles the encryption process behind the scene.
+        Note:
+            This function was updated to support both Python 2.7 and 3.5.
+            It will now compare the string (str) versions of the given
+            raw_password and the current Users object encrypted password.
         """
-        return self.password == \
-            base64.b64encode(bytes(raw_password.encode('utf-8')))
+        mangled_password_str = str(self.password)
+        raw_password_bytes = base64.b64encode(
+            bytes(raw_password.encode('utf-8')))
+
+        if sys.version_info[0] == 2:
+            raw_password_encrypted_str = str(raw_password_bytes)
+        else:
+            # Assuming Python >= 3.5
+            raw_password_encrypted_str = \
+                str(raw_password_bytes.decode('utf-8'))
+
+        return mangled_password_str == raw_password_encrypted_str
 
     @validates("groups")
     def _validate_groups(self, key, group):
@@ -719,7 +761,7 @@ class User(Entity, ACLMixin):
         """
         if not isinstance(group, Group):
             raise TypeError(
-                "Any group in %s.groups should be an instance of"
+                "Any group in %s.groups should be an instance of "
                 "stalker.models.auth.Group not %s" %
                 (self.__class__.__name__, group.__class__.__name__)
             )
@@ -754,20 +796,6 @@ class User(Entity, ACLMixin):
             )
         return task
 
-    @validates('projects')
-    def _validate_projects(self, key, project):
-        """validates the given project instance
-        """
-        from stalker import Project
-
-        if not isinstance(project, Project):
-            raise TypeError(
-                '%s.projects should a list of stalker.models.project.Project '
-                'instances, not %s' %
-                (self.__class__.__name__, project.__class__.__name__)
-            )
-        return project
-
     @validates('vacations')
     def _validate_vacations(self, key, vacation):
         """validates the given vacation value
@@ -776,8 +804,8 @@ class User(Entity, ACLMixin):
 
         if not isinstance(vacation, Vacation):
             raise TypeError(
-                "All of the members of %s.vacations should be an instance of "
-                "stalker.models.studio.Vacation, not %s" %
+                "All of the elements in %s.vacations should be a "
+                "stalker.models.studio.Vacation instance, not %s" %
                 (self.__class__.__name__, vacation.__class__.__name__)
             )
         return vacation
@@ -838,7 +866,7 @@ class User(Entity, ACLMixin):
 
     @property
     def tickets(self):
-        """The list of :class:`.Ticket`\ s that this user has.
+        """The list of :class:`.Ticket` s that this user has.
 
         returns a list of :class:`.Ticket` instances
         which this user is the owner of.
@@ -852,7 +880,7 @@ class User(Entity, ACLMixin):
 
     @property
     def open_tickets(self):
-        """The list of open :class:`.Ticket`\ s that this user has.
+        """The list of open :class:`.Ticket` s that this user has.
 
         returns a list of :class:`.Ticket` instances which has a status of
         `Open` that this user is assigned as the owner.
@@ -869,7 +897,7 @@ class User(Entity, ACLMixin):
         """outputs a TaskJuggler formatted string
         """
         from jinja2 import Template
-
+        from stalker import defaults
         temp = Template(defaults.tjp_user_template, trim_blocks=True)
         return temp.render({'user': self})
 
@@ -926,7 +954,7 @@ class LocalSession(object):
         :param int millis: an int value showing the millis from unix EPOCH
         :return:
         """
-        epoch = datetime.datetime(1970, 1, 1)
+        epoch = datetime.datetime(1970, 1, 1, tzinfo=pytz.utc)
         return epoch + datetime.timedelta(milliseconds=millis)
 
     def load(self):
@@ -937,7 +965,7 @@ class LocalSession(object):
                 # try:
                 json_object = json.load(s)
                 valid_to = self.millis_to_datetime(json_object.get('valid_to'))
-                if valid_to > datetime.datetime.now():
+                if valid_to > datetime.datetime.now(pytz.utc):
                     # fill __dict__ with the loaded one
                     self.valid_to = valid_to
                     self.logged_in_user_id = \
@@ -962,7 +990,8 @@ class LocalSession(object):
     def save(self):
         """remembers the data in user local file system
         """
-        self.valid_to = datetime.datetime.now() + datetime.timedelta(days=10)
+        self.valid_to = datetime.datetime.now(pytz.utc) + \
+                        datetime.timedelta(days=10)
         # serialize self
         dumped_data = json.dumps({
             'valid_to': self.valid_to,
@@ -984,6 +1013,7 @@ class LocalSession(object):
         """
         :return str: the session file full path
         """
+        from stalker import defaults
         return os.path.normpath(os.path.join(
             defaults.local_storage_path,
             defaults.local_session_data_file_name
@@ -1014,7 +1044,7 @@ class Role(Entity):
 
     .. versionadded 0.2.11: Roles
 
-    When :class:`.User`\ s are assigned to a
+    When :class:`.User` s are assigned to a
     :class:`.Client`/:class:`.Department`, they also can be assigned to a role
     for that client/department.
 
@@ -1068,3 +1098,109 @@ Group_Users = Table(
     Column("uid", Integer, ForeignKey("Users.id"), primary_key=True),
     Column("gid", Integer, ForeignKey("Groups.id"), primary_key=True)
 )
+
+
+class AuthenticationLog(SimpleEntity):
+    """Keeps track of login/logout dates and the action (login or logout).
+    """
+
+    __auto_name__ = True
+    __tablename__ = "AuthenticationLogs"
+    __mapper_args__ = {"polymorphic_identity": "AuthenticationLog"}
+
+    log_id = Column(
+        'id',
+        Integer,
+        ForeignKey('SimpleEntities.id'),
+        primary_key=True
+    )
+
+    user_id = Column(
+        'uid',
+        Integer,
+        ForeignKey('Users.id'),
+        nullable=False
+    )
+
+    user = relationship(
+        'User',
+        primaryjoin='AuthenticationLogs.c.uid==Users.c.id',
+        uselist=False,
+        back_populates='authentication_logs',
+        doc="The :class:`.User` instance that this AuthenticationLog is "
+        "created for"
+    )
+
+    action = Column(
+        'action',
+        Enum(LOGIN, LOGOUT, name='ActionNames'),
+        nullable=False
+    )
+
+    date = Column(
+        GenericDateTime,
+        nullable=False
+    )
+
+    def __init__(self, user=None, date=None, action=LOGIN, **kwargs):
+        super(AuthenticationLog, self).__init__(**kwargs)
+        self.user = user
+        self.date = date
+        self.action = action
+
+    def __lt__(self, other):
+        """make this orderable
+        """
+        return (
+            '%s %s %s' % (self.date, self.action, self.user.name),
+            '%s %s %s' % (other.date, other.action, other.user.name)
+        )
+
+    @validates('user')
+    def __validate_user__(self, key, user):
+        """validates the given user argument value
+        """
+        if not isinstance(user, User):
+            raise TypeError(
+                '%s.user should be a User instance, not %s' % (
+                    self.__class__.__name__,
+                    user.__class__.__name__
+                )
+            )
+
+        return user
+
+    @validates('action')
+    def __validate_action__(self, key, action):
+        """validates the given action argument value
+        """
+        if action is None:
+            import copy
+            action = copy.copy(LOGIN)
+
+        if action not in [LOGIN, LOGOUT]:
+            raise ValueError(
+                '%s.action should be one of "login" or "logout", not "%s"' % (
+                    self.__class__.__name__,
+                    action
+                )
+            )
+
+        return action
+
+    @validates('date')
+    def __validate_date__(self, key, date):
+        """validates the given date value
+        """
+        if date is None:
+            date = datetime.datetime.now(pytz.utc)
+
+        if not isinstance(date, datetime.datetime):
+            raise TypeError(
+                '%s.date should be a "datetime.datetime" instance, not %s' % (
+                    self.__class__.__name__,
+                    date.__class__.__name__
+                )
+            )
+
+        return date
