@@ -1,34 +1,35 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import datetime
 import re
 import uuid
-from sqlalchemy import (Table, Column, Integer, String, Text, ForeignKey,
-                        DateTime)
+
+import pytz
+import functools
+
+from sqlalchemy import Table, Column, Integer, String, Text, ForeignKey
 from sqlalchemy.orm import relationship, validates
 
-import stalker
 from stalker.db.declarative import Base
-
+from stalker.db.types import GenericDateTime
 from stalker.log import logging_level
+
 import logging
 
 logger = logging.getLogger(__name__)
@@ -183,15 +184,15 @@ class SimpleEntity(Base):
     )
 
     date_created = Column(
-        DateTime,
-        default=datetime.datetime.now(),
+        GenericDateTime,
+        default=functools.partial(datetime.datetime.now, pytz.utc),
         doc="""A :class:`datetime.datetime` instance showing the creation date
         and time of this object."""
     )
 
     date_updated = Column(
-        DateTime,
-        default=datetime.datetime.now(),
+        GenericDateTime,
+        default=functools.partial(datetime.datetime.now, pytz.utc),
         doc="""A :class:`datetime.datetime` instance showing the update date
         and time of this object."""
         ,
@@ -213,7 +214,7 @@ class SimpleEntity(Base):
         post_update=True,
         doc="""The type of the object.
 
-        It is an instance of :class:`.Type` with a proper
+        It is a :class:`.Type` instance with a proper
         :attr:`.Type.target_entity_type`.
         """
     )
@@ -278,7 +279,7 @@ class SimpleEntity(Base):
         self.created_by = created_by
         self.updated_by = updated_by
         if date_created is None:
-            date_created = datetime.datetime.now()
+            date_created = datetime.datetime.now(pytz.utc)
         if date_updated is None:
             date_updated = date_created
 
@@ -289,6 +290,8 @@ class SimpleEntity(Base):
         self.generic_text = generic_text
         self.html_style = html_style
         self.html_class = html_class
+
+        import stalker
         self.__stalker_version__ = stalker.__version__
 
     def __repr__(self):
@@ -322,7 +325,7 @@ class SimpleEntity(Base):
         from stalker import __string_types__
         if not isinstance(description, __string_types__):
             raise TypeError(
-                '%s.description should be an instance of string, not %s' %
+                '%s.description should be a string, not %s' %
                 (self.__class__.__name__, description.__class__.__name__)
             )
         return description
@@ -337,7 +340,7 @@ class SimpleEntity(Base):
         from stalker import __string_types__
         if not isinstance(generic_text, __string_types__):
             raise TypeError(
-                '%s.generic_text should be an instance of string, not %s' %
+                '%s.generic_text should be a string, not %s' %
                 (self.__class__.__name__, generic_text.__class__.__name__)
             )
         return generic_text
@@ -361,7 +364,7 @@ class SimpleEntity(Base):
         from stalker import __string_types__
         if not isinstance(name, __string_types__):
             raise TypeError(
-                "%s.name should be an instance of string not %s" %
+                "%s.name should be a string not %s" %
                 (self.__class__.__name__, name.__class__.__name__)
             )
 
@@ -406,11 +409,11 @@ class SimpleEntity(Base):
         # remove multiple spaces
         nice_name_in = re.sub(r'[\s]+', ' ', nice_name_in)
 
-        ## replace camel case letters
+        # # replace camel case letters
         # nice_name_in = re.sub(r"(.+?[a-z]+)([A-Z])", r"\1_\2", nice_name_in)
 
         # replace white spaces and dashes with under score
-        nice_name_in = re.sub("([\s\-])+", r"_", nice_name_in)
+        nice_name_in = re.sub("([ -])+", r"_", nice_name_in)
 
         # remove multiple underscores
         nice_name_in = re.sub(r"([_]+)", r"_", nice_name_in)
@@ -427,7 +430,7 @@ class SimpleEntity(Base):
         it is always lower case.
         """
         # also set the nice_name
-        #if self._nice_name is None or self._nice_name == "":
+        # if self._nice_name is None or self._nice_name == "":
         self._nice_name = self._format_nice_name(self.name)
         return self._nice_name
 
@@ -440,8 +443,9 @@ class SimpleEntity(Base):
         if created_by_in is not None:
             if not isinstance(created_by_in, User):
                 raise TypeError(
-                    "%s.created_by should be an instance of "
-                    "stalker.models.auth.User" % self.__class__.__name__
+                    "%s.created_by should be a stalker.models.auth.User "
+                    "instance, not %s" % (self.__class__.__name__,
+                                          created_by_in.__class__.__name__)
                 )
         return created_by_in
 
@@ -458,8 +462,9 @@ class SimpleEntity(Base):
         if updated_by_in is not None:
             if not isinstance(updated_by_in, User):
                 raise TypeError(
-                    "%s.updated_by should be an instance of "
-                    "stalker.models.auth.User" % self.__class__.__name__
+                    "%s.updated_by should be a stalker.models.auth.User "
+                    "instance, not %s" % (self.__class__.__name__,
+                                          updated_by_in.__class__.__name__)
                 )
         return updated_by_in
 
@@ -474,7 +479,7 @@ class SimpleEntity(Base):
 
         if not isinstance(date_created_in, datetime.datetime):
             raise TypeError(
-                "%s.date_created should be an instance of datetime.datetime" %
+                "%s.date_created should be a datetime.datetime instance" %
                 self.__class__.__name__
             )
 
@@ -490,10 +495,10 @@ class SimpleEntity(Base):
                 "%s.date_updated can not be None" % self.__class__.__name__
             )
 
-        # it is not an instance of datetime.datetime
+        # it is not a datetime.datetime instance
         if not isinstance(date_updated_in, datetime.datetime):
             raise TypeError(
-                "%s.date_updated should be an instance of datetime.datetime" %
+                "%s.date_updated should be a datetime.datetime instance" %
                 self.__class__.__name__
             )
 
@@ -501,7 +506,8 @@ class SimpleEntity(Base):
         if date_updated_in < self.date_created:
             raise ValueError(
                 "%(class)s.date_updated could not be set to a date before "
-                "%(class)s.date_created, try setting the 'date_created' first."
+                "%(class)s.date_created, try setting the ``date_created`` "
+                "first."
                 % {'class': self.__class__.__name__}
             )
         return date_updated_in
@@ -518,9 +524,8 @@ class SimpleEntity(Base):
             from stalker.models.type import Type
             if not isinstance(type_in, Type):
                 raise TypeError(
-                    "%s.type must be an instance of "
-                    "stalker.models.type.Type not %s" %
-                    (self.__class__.__name__, type_in))
+                    "%s.type must be a stalker.models.type.Type instance, not "
+                    "%s" % (self.__class__.__name__, type_in))
         return type_in
 
     @validates('thumbnail')
@@ -563,7 +568,7 @@ class SimpleEntity(Base):
         from stalker import __string_types__
         if not isinstance(html_style, __string_types__):
             raise TypeError(
-                '%s.html_style should be an instance of basestring, not %s' %
+                '%s.html_style should be a basestring instance, not %s' %
                 (self.__class__.__name__, html_style.__class__.__name__)
             )
         return html_style
@@ -578,7 +583,7 @@ class SimpleEntity(Base):
         from stalker import __string_types__
         if not isinstance(html_class, __string_types__):
             raise TypeError(
-                '%s.html_class should be an instance of basestring, not %s' %
+                '%s.html_class should be a basestring instance, not %s' %
                 (self.__class__.__name__, html_class.__class__.__name__)
             )
         return html_class
@@ -621,8 +626,7 @@ class Entity(SimpleEntity):
         "Note",
         secondary="Entity_Notes",
         backref="entities",
-        #cascade='all',
-        doc="""All the :class:`.Notes`\ s attached to this entity.
+        doc="""All the :class:`.Notes` s attached to this entity.
 
         It is a list of :class:`.Note` instances or an
         empty list, setting it to None will raise a TypeError.
@@ -651,9 +655,8 @@ class Entity(SimpleEntity):
         from stalker.models.note import Note
         if not isinstance(note, Note):
             raise TypeError(
-                "%s.note should be an instance of stalker.models.note.Note "
-                "not %s" %
-                (self.__class__.__name__, note.__class__.__name__)
+                "%s.note should be a stalker.models.note.Note instance, not "
+                "%s" % (self.__class__.__name__, note.__class__.__name__)
             )
         return note
 
@@ -664,8 +667,7 @@ class Entity(SimpleEntity):
         from stalker.models.tag import Tag
         if not isinstance(tag, Tag):
             raise TypeError(
-                "%s.tag should be an instance of stalker.models.tag.Tag not "
-                "%s" %
+                "%s.tag should be a stalker.models.tag.Tag instance, not %s" %
                 (self.__class__.__name__, tag.__class__.__name__)
             )
         return tag
@@ -681,6 +683,67 @@ class Entity(SimpleEntity):
         """
         return super(Entity, self).__hash__()
 
+
+class EntityGroup(Entity):
+    """Groups a wide variety of objects together to let one easily reach them.
+
+    :class:`.EntityGroup` helps grouping different types of entities together
+    to let one easily reach to them.
+    """
+
+    __auto_name__ = True
+    __tablename__ = "EntityGroups"
+    __mapper_args__ = {"polymorphic_identity": "EntityGroup"}
+    entity_group_id = Column("id", Integer, ForeignKey("Entities.id"),
+                             primary_key=True)
+
+    entities = relationship(
+        "SimpleEntity",
+        secondary="EntityGroup_Entities",
+        # primaryjoin='EntityGroups.c.id=='
+        #             'EntityGroup_Entities.c.entity_group_id',
+        # secondaryjoin='EntityGroup_Entities.c.other_entity_id=='
+        #               'SimpleEntities.c.id',
+        post_update=True,
+        backref="entity_groups",
+        doc="""All the :class:`.SimpleEntity`s grouped in this EntityGroup.
+        """
+    )
+
+    def __init__(self, entities=None, **kwargs):
+        super(Entity, self).__init__(**kwargs)
+
+        if entities is None:
+            entities = []
+
+        self.entities = entities
+
+    @validates('entities')
+    def _validate_entities(self, key, entity):
+        """validates the given entity value
+        """
+        if not isinstance(entity, SimpleEntity):
+            raise TypeError(
+                '%s.entities should be a list of SimpleEntities, not %s' % (
+                    self.__class__.__name__,
+                    entity.__class__.__name__
+                )
+            )
+
+        return entity
+
+    def __eq__(self, other):
+        """the equality operator
+        """
+        return super(SimpleEntity, self).__eq__(other) \
+            and isinstance(other, EntityGroup)
+
+    def __hash__(self):
+        """the overridden __hash__ method
+        """
+        return super(SimpleEntity, self).__hash__()
+
+
 # Entity Tags
 Entity_Tags = Table(
     "Entity_Tags", Base.metadata,
@@ -733,3 +796,20 @@ SimpleEntity_GenericData = Table(
         primary_key=True
     )
 )
+
+# EntityGroup Entities
+EntityGroup_Entities = Table(
+    'EntityGroup_Entities', Base.metadata,
+    Column(
+        'entity_group_id',
+        Integer,
+        ForeignKey('EntityGroups.id'),
+        primary_key=True
+    ),
+    Column(
+        'other_entity_id',
+        Integer,
+        ForeignKey('SimpleEntities.id'),
+        primary_key=True
+    )
+)
