@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 from sqlalchemy import Column, Integer, ForeignKey, Float
 from sqlalchemy.orm import validates
@@ -31,7 +29,7 @@ logger.setLevel(logging_level)
 
 
 class ImageFormat(Entity):
-    """Common image formats for the :class:`.Project`\ s.
+    """Common image formats for the :class:`.Project` s.
 
     :param width: The width of the format, it cannot be zero or negative, if a
       float number is given it will be converted to integer
@@ -40,11 +38,11 @@ class ImageFormat(Entity):
       a float number is given it will be converted to integer
 
     :param pixel_aspect: The pixel aspect ratio of the current ImageFormat
-      object, it can not be zero or negative, and if given as an integer it
+      object, it cannot be zero or negative, and if given as an integer it
       will be converted to a float, the default value is 1.0
 
     :param print_resolution: The print resolution of the ImageFormat given as
-      DPI (dot-per-inch). It can not be zero or negative
+      DPI (dot-per-inch). It cannot be zero or negative
     """
     __auto_name__ = False
     __tablename__ = "ImageFormats"
@@ -119,13 +117,13 @@ class ImageFormat(Entity):
         """
         if not isinstance(width, (int, float)):
             raise TypeError(
-                "%s.width should be an instance of int or float not %s" %
+                '%s.width should be an instance of int or float not %s' %
                 (self.__class__.__name__, width.__class__.__name__)
             )
 
         if width <= 0:
             raise ValueError(
-                "%s.width shouldn't be zero or negative" %
+                '%s.width cannot be zero or negative' %
                 self.__class__.__name__
             )
 
@@ -137,13 +135,13 @@ class ImageFormat(Entity):
         """
         if not isinstance(height, (int, float)):
             raise TypeError(
-                "%s.height should be an instance of int or float not %s" %
+                '%s.height should be an instance of int or float not %s' %
                 (self.__class__.__name__, height.__class__.__name__)
             )
 
         if height <= 0:
             raise ValueError(
-                "%s.height shouldn't be zero or negative" %
+                '%s.height cannot be zero or negative' %
                 self.__class__.__name__
             )
 
@@ -156,13 +154,13 @@ class ImageFormat(Entity):
 
         if not isinstance(pixel_aspect, (int, float)):
             raise TypeError(
-                "%s.pixel_aspect should be an instance of int or float not %s"
+                '%s.pixel_aspect should be an instance of int or float not %s'
                 % (self.__class__.__name__, pixel_aspect.__class__.__name__)
             )
 
         if pixel_aspect <= 0:
             raise ValueError(
-                "%s.pixel_aspect can not be zero or a negative value" %
+                "%s.pixel_aspect cannot be zero or a negative value" %
                 self.__class__.__name__
             )
 
@@ -182,7 +180,7 @@ class ImageFormat(Entity):
 
         if print_resolution <= 0:
             raise ValueError(
-                "%s.print_resolution should not be zero or negative" %
+                "%s.print_resolution cannot be zero or negative" %
                 self.__class__.__name__
             )
 
@@ -209,4 +207,4 @@ class ImageFormat(Entity):
     def __hash__(self):
         """the overridden __hash__ method
         """
-        return hash(self.id) + 2 * hash(self.name) + 3 * hash(self.entity_type)
+        return super(ImageFormat, self).__hash__()
