@@ -1,37 +1,34 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
-import datetime
 import logging
 
+import pytz
 from sqlalchemy import (Table, Column, String, Integer, ForeignKey, Interval,
-                        DateTime, PickleType, Float, Enum)
-from sqlalchemy.exc import UnboundExecutionError
+                        Float, Enum)
+from sqlalchemy.exc import UnboundExecutionError, OperationalError
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import synonym, relationship, validates
 
-from stalker import defaults
-from stalker.db.session import DBSession
+
 from stalker.db.declarative import Base
+from stalker.db.types import GenericDateTime
 from stalker.log import logging_level
-from stalker.models import make_plural, check_circular_dependency
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging_level)
@@ -45,6 +42,7 @@ def create_secondary_table(
         secondary_table_name=None):
     """creates any secondary table
     """
+    from stalker.models import make_plural
     plural_secondary_cls_name = make_plural(secondary_cls_name)
 
     # use the given class_name and the class_table
@@ -230,7 +228,7 @@ class StatusMixin(object):
     def status(cls):
         return relationship(
             'Status',
-            primaryjoin= "%s.status_id==Status.status_id" % cls.__name__,
+            primaryjoin="%s.status_id==Status.status_id" % cls.__name__,
             doc="""The current status of the object.
 
             It is a :class:`.Status` instance which
@@ -268,6 +266,7 @@ class StatusMixin(object):
             # StatusList from the database
 
             # disable autoflush to prevent premature class initialization
+            from stalker.db.session import DBSession
             with DBSession.no_autoflush:
                 try:
                     # try to get a StatusList with the target_entity_type is
@@ -275,7 +274,7 @@ class StatusMixin(object):
                     status_list = StatusList.query\
                         .filter_by(target_entity_type=self.__class__.__name__)\
                         .first()
-                except UnboundExecutionError:
+                except (UnboundExecutionError, OperationalError):
                     # it is not mapped just skip it
                     pass
 
@@ -303,7 +302,7 @@ class StatusMixin(object):
             # StatusList.target_entity_type
             if self.__class__.__name__ != status_list.target_entity_type:
                 raise TypeError(
-                    "the given StatusLists' target_entity_type is %s, "
+                    "The given StatusLists' target_entity_type is %s, "
                     "whereas the entity_type of this object is %s" %
                     (status_list.target_entity_type, self.__class__.__name__)
                 )
@@ -314,14 +313,8 @@ class StatusMixin(object):
     def _validate_status(self, key, status):
         """validates the given status value
         """
-        from stalker.models.status import Status, StatusList
-
-        with DBSession.no_autoflush:
-            if not isinstance(self.status_list, StatusList):
-                raise TypeError(
-                    "Please set the %s.status_list attribute first" %
-                    self.__class__.__name__
-                )
+        from stalker.models.status import Status
+        from stalker.db.session import DBSession
 
         # it is set to None
         if status is None:
@@ -392,7 +385,7 @@ class DateRangeMixin(object):
     +-------+-----+----------+----------------------------------------+
     | start | end | duration | DEFAULTS                               |
     +=======+=====+==========+========================================+
-    |       |     |          | start = datetime.datetime.now()        |
+    |       |     |          | start = datetime.datetime.now(pytz.utc)|
     |       |     |          |                                        |
     |       |     |          | duration = datetime.timedelta(days=10) |
     |       |     |          |                                        |
@@ -414,7 +407,7 @@ class DateRangeMixin(object):
     |       |     |          |                                        |
     |       |     |          | start = end - duration                 |
     +-------+-----+----------+----------------------------------------+
-    |       |     |    X     | start = datetime.datetime.now()        |
+    |       |     |    X     | start = datetime.datetime.now(pytz.utc)|
     |       |     |          |                                        |
     |       |     |          | end = start + duration                 |
     +-------+-----+----------+----------------------------------------+
@@ -439,7 +432,7 @@ class DateRangeMixin(object):
       any condition if the start is available then the value will be
       preserved. If start passes the end the end is also changed
       to a date to keep the timedelta between dates. The default value is
-      datetime.datetime.now()
+      datetime.datetime.now(pytz.utc)
 
     :type start: :class:`datetime.datetime`
 
@@ -469,7 +462,7 @@ class DateRangeMixin(object):
 
     @declared_attr
     def _end(cls):
-        return Column("end", DateTime)
+        return Column("end", GenericDateTime)
 
     def _end_getter(self):
         """The date that the entity should be delivered.
@@ -498,7 +491,7 @@ class DateRangeMixin(object):
 
     @declared_attr
     def _start(cls):
-        return Column("start", DateTime)
+        return Column("start", GenericDateTime)
 
     def _start_getter(self):
         """The date that this entity should start.
@@ -510,7 +503,7 @@ class DateRangeMixin(object):
         :attr:`.DateRangeMixin.duration` value fixed.
         :attr:`.DateRangeMixin.start` should be an instance of
         class:`datetime.datetime` and the default value is
-        :func:`datetime.datetime.now()`
+        :func:`datetime.datetime.now(pytz.utc)`
         """
         return self._start
 
@@ -537,6 +530,7 @@ class DateRangeMixin(object):
 
     def _duration_setter(self, duration_in):
         if duration_in is not None:
+            import datetime
             if isinstance(duration_in, datetime.timedelta):
                 # set the end to None
                 # to make it recalculated
@@ -572,6 +566,8 @@ class DateRangeMixin(object):
         # logger.debug('end      : %s' % end)
         # logger.debug('duration : %s' % duration)
 
+        from stalker import defaults
+        import datetime
         if not isinstance(start, datetime.datetime):
             start = None
 
@@ -586,7 +582,7 @@ class DateRangeMixin(object):
             # try to calculate the start from end and duration
             if end is None:
                 # set the defaults
-                start = datetime.datetime.now()
+                start = datetime.datetime.now(pytz.utc)
 
                 if duration is None:
                     # set the defaults
@@ -632,12 +628,11 @@ class DateRangeMixin(object):
 
     @declared_attr
     def computed_start(cls):
-        return Column('computed_start', DateTime)
+        return Column('computed_start', GenericDateTime)
 
     @declared_attr
     def computed_end(cls):
-        return Column('computed_end', DateTime)
-
+        return Column('computed_end', GenericDateTime)
 
     @property
     def computed_duration(self):
@@ -665,11 +660,15 @@ class DateRangeMixin(object):
         """
         # to be compatible with python 2.6 use the following instead of
         # total_seconds()
+        from stalker import defaults
         timing_resolution = defaults.timing_resolution
         trs = timing_resolution.days * 86400 + timing_resolution.seconds
 
         # convert to seconds
-        epoch = datetime.datetime(1970, 1, 1)
+        import datetime
+        import pytz
+        epoch = datetime.datetime(1970, 1, 1, tzinfo=pytz.utc)
+
         diff = dt - epoch
         diff_in_seconds = diff.days * 86400 + diff.seconds
         return epoch + datetime.timedelta(
@@ -693,13 +692,18 @@ class DateRangeMixin(object):
 class ProjectMixin(object):
     """Allows connecting a :class:`.Project` to the mixed in object.
 
+    This also forces a ``all, delete-orphan`` cascade, so when a
+    :class:``.Project`` instance is deleted then all the class instances that
+    are inherited from ``ProjectMixin`` will also be deleted. Meaning that, a
+    class which also derives from ``ProjectMixin`` will not be able to exists
+    without a project (``delete-orphan`` case).
+
     :param project: A :class:`.Project` instance holding the project which this
       object is related to. It can not be None, or anything other than a
       :class:`.Project` instance.
 
     :type project: :class:`.Project`
     """
-
     #    # add this lines for Sphinx
     #    __tablename__ = "ProjectMixins"
 
@@ -708,28 +712,29 @@ class ProjectMixin(object):
         return Column(
             "project_id",
             Integer,
-            #ForeignKey("Projects.id", use_alter=True, name="project_x_id"),
             ForeignKey("Projects.id"),
             # cannot use nullable cause a Project object needs
             # insert itself as the project and it needs post_update
             # thus nullable should be True
-            #nullable=False,
         )
 
     @declared_attr
     def project(cls):
-        backref = cls.__tablename__.lower()
+        from sqlalchemy.orm import backref
+        backref_table_name = cls.__tablename__.lower()
         doc = """The :class:`.Project` instance that
         this object belongs to.
         """
 
         return relationship(
             "Project",
-            primaryjoin=
-            cls.__tablename__ + ".c.project_id==Projects.c.id",
+            primaryjoin="%s.c.project_id==Projects.c.id" % cls.__tablename__,
             post_update=True,  # for project itself
             uselist=False,
-            backref=backref,
+            backref=backref(
+                backref_table_name,
+                cascade='all, delete-orphan'
+            ),
             doc=doc
         )
 
@@ -789,7 +794,7 @@ class ReferenceMixin(object):
             'Link',
             cls.__tablename__,
             'Links',
-            cls.__name__ + "_References"
+            "%s_References" % cls.__name__
         )
         # return the relationship
         return relationship(
@@ -809,8 +814,8 @@ class ReferenceMixin(object):
         # all the elements should be instance of stalker.models.entity.Entity
         if not isinstance(reference, Link):
             raise TypeError(
-                "%s.references should be all instances of "
-                "stalker.models.link.Link not %s" %
+                'All the elements in the %s.references should be '
+                'stalker.models.link.Link instances not %s' %
                 (self.__class__.__name__, reference.__class__.__name__)
             )
         return reference
@@ -869,7 +874,7 @@ class ACLMixin(object):
         """
         return [(perm.access,
                  '%s:%s' % (self.__class__.__name__, self.name),
-                 perm.action + '_' + perm.class_name)
+                 '%s_%s' % (perm.action, perm.class_name))
                 for perm in self.permissions]
 
 
@@ -936,25 +941,45 @@ class WorkingHoursMixin(object):
     Generally is meaningful for users, departments and studio.
 
     :param working_hours: A :class:`.WorkingHours` instance showing the working
-      hours settings for that project. This data is stored as a PickleType in
-      the database.
+      hours settings.
     """
 
     def __init__(self, working_hours=None, **kwargs):
         self.working_hours = working_hours
 
+    @declared_attr
+    def working_hours_id(cls):
+        """the id of the related working hours
+        """
+        return Column(
+            'working_hours_id',
+            Integer,
+            ForeignKey('WorkingHours.id')
+        )
+
     @declared_attr
     def working_hours(cls):
-        return Column(PickleType)
+        return relationship(
+            'WorkingHours',
+            primaryjoin='%s.working_hours_id==WorkingHours.working_hours_id' %
+                        cls.__name__
+        )
 
     @validates('working_hours')
     def _validate_working_hours(self, key, wh):
         """validates the given working hours value
         """
+        from stalker import WorkingHours
         if wh is None:
-            # use the default one
-            from stalker import WorkingHours
-            wh = WorkingHours()
+            wh = WorkingHours()  # without any argument this will use the
+                                 # default.working_hours settings
+        elif not isinstance(wh, WorkingHours):
+            raise TypeError(
+                '%s.working_hours should be a '
+                'stalker.models.studio.WorkingHours instance, not %s' % (
+                    self.__class__.__name__, wh.__class__.__name__
+                )
+            )
 
         return wh
 
@@ -970,6 +995,7 @@ class ScheduleMixin(object):
     """
 
     # some default values that can be overridden in Mixed in classes
+    from stalker import defaults
     __default_schedule_attr_name__ = 'schedule'
     __default_schedule_timing__ = defaults.timing_resolution.seconds / 60
     __default_schedule_unit__ = 'h'
@@ -1006,10 +1032,11 @@ class ScheduleMixin(object):
 
     @declared_attr
     def schedule_unit(cls):
+        from stalker import defaults
         return Column(
             '%s_unit' % cls.__default_schedule_attr_name__,
             Enum(*defaults.datetime_units, name='TimeUnit'),
-            nullable=False, default='h',
+            nullable=True, default='h',
             doc="""It is the unit of the %(attr)s timing. It is a string
             value. And should be one of 'min', 'h', 'd', 'w', 'm', 'y'.""" %
                 {'attr': cls.__default_schedule_attr_name__}
@@ -1153,6 +1180,7 @@ class ScheduleMixin(object):
     def _validate_schedule_unit(self, key, schedule_unit):
         """validates the given schedule_unit
         """
+        from stalker import defaults
         if schedule_unit is None:
             schedule_unit = self.__default_schedule_unit__
 
@@ -1233,6 +1261,7 @@ class ScheduleMixin(object):
         :returns int, string: Returns one integer and one string, showing the
           timing value and the unit.
         """
+        from stalker import defaults
         minutes = 60
         hour = 3600
         day = 86400
@@ -1296,6 +1325,7 @@ class ScheduleMixin(object):
         }
 
         if model in ['effort', 'length']:
+            from stalker import defaults
             day_wt = defaults.daily_working_hours * 3600
             week_wt = defaults.weekly_working_days * day_wt
             month_wt = 4 * week_wt
@@ -1338,8 +1368,12 @@ class DAGMixin(object):
 
           id = Column('id', Integer, primary_key=True)
           __id_column__ = id
+
+    Use the :attr:``.__dag_cascade__`` to control the cascade behaviour.
     """
 
+    __dag_cascade__ = 'all, delete'
+
     @declared_attr
     def parent_id(cls):
         return Column(
@@ -1374,7 +1408,7 @@ class DAGMixin(object):
             },
             back_populates='parent',
             post_update=True,
-            cascade='all, delete',
+            cascade=cls.__dag_cascade__,
             doc="""Other :class:`Budget` instances which are the children of this
             one. This attribute along with the :attr:`.parent` attribute is used in
             creating a DAG hierarchy of tasks.
@@ -1397,6 +1431,8 @@ class DAGMixin(object):
                         'parent_cls': parent.__class__.__name__
                     }
                 )
+
+            from stalker.models import check_circular_dependency
             check_circular_dependency(self, parent, 'children')
 
         return parent
@@ -1426,6 +1462,7 @@ class DAGMixin(object):
     def is_container(self):
         """Returns True if the Task has children Tasks
         """
+        from stalker.db.session import DBSession
         with DBSession.no_autoflush:
             return bool(len(self.children))
 
@@ -1442,13 +1479,14 @@ class DAGMixin(object):
         """
         parents = []
         entity = self.parent
+        # TODO: make this a generator
         while entity:
             parents.append(entity)
             entity = entity.parent
         parents.reverse()
         return parents
 
-    def walk_hierarchy(self, method=1):
+    def walk_hierarchy(self, method=0):
         """Walks the hierarchy of this task.
 
         :param method: The walk method, 0: Depth First, 1: Breadth First
@@ -1457,3 +1495,63 @@ class DAGMixin(object):
         for c in walk_hierarchy(self, 'children', method=method):
             yield c
 
+
+class AmountMixin(object):
+    """Adds ``amount`` attribute to the mixed in class.
+
+    :param int float amount:
+    """
+
+    def __init__(self, amount=0, **kwargs):
+        self.amount = amount
+
+    @declared_attr
+    def amount(cls):
+        return Column(Float, default=0.0)
+
+    @validates('amount')
+    def _validate_amount(self, key, amount):
+        """validates the given amount value
+        """
+        if amount is None:
+            amount = 0.0
+
+        if not isinstance(amount, (int, float)):
+            raise TypeError(
+                '%s.amount should be a number, not %s' % (
+                    self.__class__.__name__, amount.__class__.__name__
+                )
+            )
+
+        return float(amount)
+
+
+class UnitMixin(object):
+    """Adds ``unit`` attribute to the mixed in class.
+
+    :param str unit:
+    """
+
+    def __init__(self, unit='', **kwargs):
+        self.unit = unit
+
+    @declared_attr
+    def unit(cls):
+        return Column(String(64))
+
+    @validates('unit')
+    def _validate_unit(self, key, unit):
+        """validates the given unit value
+        """
+        if unit is None:
+            unit = ''
+
+        from stalker import __string_types__
+        if not isinstance(unit, __string_types__):
+            raise TypeError(
+                '%s.unit should be a string, not %s' % (
+                    self.__class__.__name__, unit.__class__.__name__
+                )
+            )
+
+        return unit
