@@ -1,30 +1,28 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import logging
 
-from sqlalchemy import (Column, Integer, ForeignKey, Float, Boolean, Table)
+from sqlalchemy import Column, Integer, ForeignKey, Float, Boolean
 from sqlalchemy.ext.associationproxy import association_proxy
+from sqlalchemy.ext.orderinglist import ordering_list
 from sqlalchemy.orm import relationship, validates
 
-from stalker import defaults
 from stalker.db.declarative import Base
 from stalker.models.entity import Entity
 from stalker.models.mixins import (StatusMixin, DateRangeMixin, ReferenceMixin,
@@ -35,6 +33,65 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging_level)
 
 
+class ProjectRepository(Base):
+    """The association object for Project to Repository instances
+    """
+    __tablename__ = 'Project_Repositories'
+
+    project_id = Column(
+        Integer,
+        ForeignKey('Projects.id'),
+        primary_key=True
+    )
+
+    project = relationship(
+        'Project',
+        back_populates='repositories_proxy',
+        primaryjoin='Project.project_id==ProjectRepository.project_id',
+    )
+
+    repository_id = Column(
+        Integer,
+        ForeignKey('Repositories.id'),
+        primary_key=True
+    )
+
+    repository = relationship(
+        'Repository',
+        primaryjoin=
+        'ProjectRepository.repository_id==Repository.repository_id',
+    )
+
+    position = Column(Integer)
+
+    def __init__(self, project=None, repository=None, position=None):
+        self.project = project
+        self.repository = repository
+        self.position = position
+
+    @validates('project')
+    def _validate_project(self, key, project):
+        """validates the given project value
+        """
+        return project
+
+    @validates("repository")
+    def _validate_repository(self, key, repository):
+        """validates the given repository value
+        """
+        if repository is not None:
+            from stalker.models.repository import Repository
+            if not isinstance(repository, Repository):
+                raise TypeError(
+                    "%s.repositories should be a list of "
+                    "stalker.models.repository.Repository instances or "
+                    "derivatives, not %s" %
+                    (self.__class__.__name__, repository.__class__.__name__)
+                )
+
+        return repository
+
+
 class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
     """All the information about a Project in Stalker is hold in this class.
 
@@ -67,10 +124,43 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
     To manage all the studio projects at once (schedule them at once please use
     :class:`.Studio`).
 
-    :param client: The client which the project is affiliated with. Default
-      value is None.
+    **Repositories**
 
-    :type client: :class:`.Client`
+    .. versionadded:: 0.2.13
+       Multiple Repositories per Project
+
+       Starting with v0.2.13 Project instances can have multiple Repositories,
+       which allows the project files to be placed in more than one repository
+       according to the need of the studio pipeline. One great advantage of
+       having multiple repositories is to be able to place Published versions
+       in to another repository which is placed on to a faster server.
+
+       Also the :attr:`.repositories` attribute is not a read-only attribute
+       anymore.
+
+    **Clients**
+
+    .. versionadded:: 0.2.15
+       Multiple Clients per Project
+
+       It is now possible to attach multiple :class:`.Client` instances to one
+       :class:`.Project` allowing to hold complex Projects to Client relations
+       by using the :attr:`.ProjectClient.role` attribute of the
+       :class:`.ProjectClient` class.
+
+    **Deleting a Project**
+
+    Deleting a :class:`.Project` instance will cascade the delete operation to
+    all the :class:`.Task` s related to that particular Project and it will
+    cascade the delete operation to :class:`.TimeLog` s, :class:`.Version` s,
+    :class:`.Link` s and :class:`.Review` s etc.. So one can delete a
+    :class:`.Project` instance without worrying about the non-project related
+    data like :class:`.User` s or :class:`.Department` s to be deleted.
+
+    :param clients: The clients which the project is affiliated with. Default
+      value is an empty list.
+
+    :type client: [:class:`.Client`]
 
     :param image_format: The output image format of the project. Default
       value is None.
@@ -89,10 +179,11 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
 
     :type structure: :class:`.Structure`
 
-    :param repository: The repository that the project files are going to be
-      stored in. You can not create a project without specifying the repository
-      argument and passing a :class:`.Repository` to it. Default value is None
-      which raises a TypeError.
+    :param repositories: A list of :class:`.Repository` instances that the
+      project files are going to be stored in. You can not create a project
+      without specifying the repositories argument and passing a
+      :class:`.Repository` to it. Default value is None which raises a
+      TypeError.
 
     :type repository: :class:`.Repository`.
 
@@ -100,7 +191,7 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
       to be a stereo 3D project, anything given as the argument will be
       converted to True or False. Default value is False.
 
-    :param users: A list of :class:`.User`\ s holding the users in this
+    :param users: A list of :class:`.User` s holding the users in this
       project. This will create a reduced or grouped list of studio workers and
       will make it easier to define the resources for a Task related to this
       project. The default value is an empty list.
@@ -118,18 +209,18 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
 
     active = Column(Boolean, default=True)
 
-    client_id = Column(Integer, ForeignKey("Clients.id"))
-    client = relationship(
-        "Client",
-        primaryjoin="Projects.c.client_id==Clients.c.id",
-        back_populates="projects",
-        uselist=False,
-        doc="""The client company assigning the studio
-        with the project.
+    clients = association_proxy(
+        'client_role',
+        'client',
+        creator=lambda n: ProjectClient(client=n)
+    )
 
-        Should be an instance of :class:`.Client`,
-        can also be set to None.
-        """
+    client_role = relationship(
+        'ProjectClient',
+        back_populates='project',
+        cascade='all, delete-orphan',
+        cascade_backrefs=False,
+        primaryjoin='Projects.c.id==Project_Clients.c.project_id'
     )
 
     tasks = relationship(
@@ -149,21 +240,30 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
         'ProjectUser',
         back_populates='project',
         cascade='all, delete-orphan',
+        cascade_backrefs=False,
         primaryjoin='Projects.c.id==Project_Users.c.project_id'
     )
 
-    repository_id = Column(Integer, ForeignKey("Repositories.id"))
-    repository = relationship(
-        "Repository",
-        primaryjoin="Project.repository_id==Repository.repository_id",
-        doc="""The :class:`.Repository` that this project should reside.
+    repositories_proxy = relationship(
+        'ProjectRepository',
+        back_populates='project',
+        cascade='all, delete-orphan',
+        cascade_backrefs=False,
+        order_by='ProjectRepository.position',
+        primaryjoin='Projects.c.id==Project_Repositories.c.project_id',
+        collection_class=ordering_list('position'),
+        doc="""The :class:`.Repository` that this project files should reside.
 
-        Should be an instance of :class:`.Repository`\ . It is a read-only
-        attribute. So it is not possible to change the repository of one
-        project.
+        Should be a list of :class:`.Repository`\ instances.
         """
     )
 
+    repositories = association_proxy(
+        'repositories_proxy',
+        'repository',
+        creator=lambda n: ProjectRepository(repository=n)
+    )
+
     structure_id = Column(Integer, ForeignKey("Structures.id"))
     structure = relationship(
         "Structure",
@@ -207,8 +307,8 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
     def __init__(self,
                  name=None,
                  code=None,
-                 client=None,
-                 repository=None,
+                 clients=None,
+                 repositories=None,
                  structure=None,
                  image_format=None,
                  fps=25.0,
@@ -218,7 +318,6 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
         # a projects project should be self
         # initialize the project argument to self
         kwargs['project'] = self
-
         kwargs['name'] = name
 
         super(Project, self).__init__(**kwargs)
@@ -226,14 +325,22 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
         ReferenceMixin.__init__(self, **kwargs)
         StatusMixin.__init__(self, **kwargs)
         DateRangeMixin.__init__(self, **kwargs)
-        #CodeMixin.__init__(self, **kwargs)
+
+        self.code = code
 
         if users is None:
             users = []
         self.users = users
-        self.repository = repository
+
+        if repositories is None:
+            repositories = []
+        self.repositories = repositories
+
         self.structure = structure
-        self.client = client
+
+        if clients is None:
+            clients = []
+        self.clients = clients
 
         self._sequences = []
         self._assets = []
@@ -241,7 +348,6 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
         self.image_format = image_format
         self.fps = fps
         self.is_stereoscopic = bool(is_stereoscopic)
-        self.code = code
 
         self.active = True
 
@@ -260,11 +366,19 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
     def _validate_fps(self, key, fps):
         """validates the given fps_in value
         """
+        if not isinstance(fps, (int, float)):
+            raise TypeError(
+                '%s.fps should be a positive float or int, not %s' % (
+                    self.__class__.__name__,
+                    fps.__class__.__name__
+                )
+            )
+
         fps = float(fps)
         if fps <= 0:
             raise ValueError(
-                '%s.fps can not be 0 or a negative value' %
-                self.__class__.__name__
+                '%s.fps should be a positive float or int, not %s' %
+                (self.__class__.__name__, fps)
             )
         return float(fps)
 
@@ -283,34 +397,6 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
             )
         return image_format
 
-    @validates("client")
-    def _validate_client(self, key, client):
-        """validates the given client value
-        """
-        if client is not None:
-            from stalker.models.client import Client
-            if not isinstance(client, Client):
-                raise TypeError(
-                    "%s.client should be an instance of "
-                    "stalker.models.auth.Client not %s" %
-                    (self.__class__.__name__, client.__class__.__name__)
-                )
-        return client
-
-    @validates("repository")
-    def _validate_repository(self, key, repository):
-        """validates the given repository_in value
-        """
-        from stalker.models.repository import Repository
-
-        if not isinstance(repository, Repository):
-            raise TypeError(
-                "%s.repository should be an instance of "
-                "stalker.models.repository.Repository, not %s" %
-                (self.__class__.__name__, repository.__class__.__name__)
-            )
-        return repository
-
     @validates("structure")
     def _validate_structure(self, key, structure_in):
         """validates the given structure_in value
@@ -330,41 +416,32 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
     def _validate_is_stereoscopic(self, key, is_stereoscopic_in):
         return bool(is_stereoscopic_in)
 
-    @validates('users')
-    def _validate_users(self, key, user_in):
-        """validates the given users_in value
-        """
-        from stalker.models.auth import User
-
-        if not isinstance(user_in, User):
-            raise TypeError(
-                '%s.users should be all stalker.models.auth.User instances, '
-                'not %s' %
-                (self.__class__.__name__, user_in.__class__.__name__)
-            )
-        return user_in
-
     @property
     def root_tasks(self):
         """returns a list of Tasks which have no parent
         """
-        from stalker.models.task import Task
+        from stalker import Task
+        from stalker.db.session import DBSession
 
-        return Task.query \
-            .filter(Task.project == self) \
-            .filter(Task.parent == None) \
-            .all()
+        # TODO: add a fallback method
+        with DBSession.no_autoflush:
+            return Task.query \
+                .filter(Task.project == self) \
+                .filter(Task.parent == None) \
+                .all()
 
     @property
     def assets(self):
         """returns the assets related to this project
         """
-        # use joins over the session.query
         from stalker.models.asset import Asset
+        from stalker.db.session import DBSession
 
-        return Asset.query \
-            .filter(Asset.project == self) \
-            .all()
+        # TODO: add a fallback method
+        with DBSession.no_autoflush:
+            return Asset.query \
+                .filter(Asset.project == self) \
+                .all()
 
     @property
     def sequences(self):
@@ -393,6 +470,7 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
         """returns a TaskJuggler compatible string representing this project
         """
         from jinja2 import Template
+        from stalker import defaults
         temp = Template(defaults.tjp_project_template, trim_blocks=True,
                         lstrip_blocks=True)
         return temp.render({'project': self})
@@ -447,7 +525,7 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
 
     @property
     def open_tickets(self):
-        """The list of open :class:`.Ticket`\ s in this project.
+        """The list of open :class:`.Ticket` s in this project.
 
         returns a list of :class:`.Ticket` instances which has a status of
         `Open` and created in this project.
@@ -459,8 +537,18 @@ class Project(Entity, ReferenceMixin, StatusMixin, DateRangeMixin, CodeMixin):
             .filter(Status.code != 'CLS') \
             .all()
 
+    @property
+    def repository(self):
+        """compatibility attribute for pre v0.2.13 systems. Returns the first
+        repository instance in the project.repositories attribute if there is
+        any or None
+        """
+        if self.repositories:
+            return self.repositories[0]
+        else:
+            return None
+
 
-# PROJECT_USERS
 class ProjectUser(Base):
     """The association object used in User-to-Project relation
     """
@@ -477,6 +565,7 @@ class ProjectUser(Base):
     user = relationship(
         'User',
         back_populates='project_role',
+        cascade_backrefs=False,
         primaryjoin='ProjectUser.user_id==User.user_id'
     )
 
@@ -490,6 +579,7 @@ class ProjectUser(Base):
     project = relationship(
         'Project',
         back_populates='user_role',
+        cascade_backrefs=False,
         primaryjoin='ProjectUser.project_id==Project.project_id'
     )
 
@@ -502,13 +592,18 @@ class ProjectUser(Base):
 
     role = relationship(
         'Role',
+        cascade_backrefs=False,
         primaryjoin='ProjectUser.role_id==Role.role_id'
     )
 
+    rate = Column(Float, default=0.0)
+
     def __init__(self, project=None, user=None, role=None):
         self.user = user
         self.project = project
         self.role = role
+        if self.user:
+            self.rate = user.rate
 
     @validates("user")
     def _validate_user(self, key, user):
@@ -522,6 +617,10 @@ class ProjectUser(Base):
                     "not %s" %
                     (self.__class__.__name__, user.__class__.__name__)
                 )
+
+            # also update rate attribute
+            self.rate = user.rate
+
         return user
 
     @validates("project")
@@ -546,7 +645,129 @@ class ProjectUser(Base):
             from stalker import Role
             if not isinstance(role, Role):
                 raise TypeError(
-                    '%s.role should be a'
+                    '%s.role should be a '
+                    'stalker.models.auth.Role instance, not %s' %
+                    (self.__class__.__name__, role.__class__.__name__)
+                )
+        return role
+
+    @validates('rate')
+    def _validate_rate(self, key, rate):
+        """validates the given rate value
+        """
+        if rate is None:
+            rate = 0.0
+
+        if not isinstance(rate, (int, float)):
+            raise TypeError(
+                '%(class)s.rate should be a float number greater or '
+                'equal to 0.0, not %(rate_class)s' % {
+                    'class': self.__class__.__name__,
+                    'rate_class': rate.__class__.__name__
+                }
+            )
+
+        if rate < 0:
+            raise ValueError(
+                '%(class)s.rate should be a float number greater or '
+                'equal to 0.0, not %(rate)s' % {
+                    'class': self.__class__.__name__,
+                    'rate': rate
+                }
+            )
+
+        return rate
+
+
+class ProjectClient(Base):
+    """The association object used in Client-to-Project relation
+    """
+
+    __tablename__ = 'Project_Clients'
+
+    client_id = Column(
+        'client_id',
+        Integer,
+        ForeignKey('Clients.id'),
+        primary_key=True
+    )
+
+    client = relationship(
+        'Client',
+        back_populates='project_role',
+        cascade_backrefs=False,
+        primaryjoin='Project_Clients.c.client_id==Clients.c.id'
+    )
+
+    project_id = Column(
+        'project_id',
+        Integer,
+        ForeignKey('Projects.id'),
+        primary_key=True
+    )
+
+    project = relationship(
+        'Project',
+        back_populates='client_role',
+        cascade_backrefs=False,
+        primaryjoin='ProjectClient.project_id==Project.project_id'
+    )
+
+    role_id = Column(
+        'rid',
+        Integer,
+        ForeignKey('Roles.id'),
+        nullable=True
+    )
+
+    role = relationship(
+        'Role',
+        cascade_backrefs=False,
+        primaryjoin='ProjectClient.role_id==Role.role_id'
+    )
+
+    def __init__(self, project=None, client=None, role=None):
+        self.client = client
+        self.project = project
+        self.role = role
+
+    @validates("client")
+    def _validate_client(self, key, client):
+        """validates the given client value
+        """
+        if client is not None:
+            from stalker.models.client import Client
+            if not isinstance(client, Client):
+                raise TypeError(
+                    "%s.client should be an instance of "
+                    "stalker.models.auth.Client not %s" %
+                    (self.__class__.__name__, client.__class__.__name__)
+                )
+        return client
+
+    @validates("project")
+    def _validate_project(self, key, project):
+        """validates the given project value
+        """
+        if project is not None:
+            # check if it is instance of Project object
+            if not isinstance(project, Project):
+                raise TypeError(
+                    "%s.project should be a "
+                    "stalker.models.project.Project instance, not %s" %
+                    (self.__class__.__name__, project.__class__.__name__)
+                )
+        return project
+
+    @validates('role')
+    def _validate_role(self, key, role):
+        """validates the given role instance
+        """
+        if role is not None:
+            from stalker import Role
+            if not isinstance(role, Role):
+                raise TypeError(
+                    '%s.role should be a '
                     'stalker.models.auth.Role instance, not %s' %
                     (self.__class__.__name__, role.__class__.__name__)
                 )
