@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
-# stalker_pyramid
-# Copyright (C) 2013 Erkan Ozgur Yilmaz
+# Stalker a Production Asset Management System
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
-# This file is part of stalker_pyramid.
+# This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import logging
 
@@ -24,9 +22,10 @@ from sqlalchemy import Column, Integer, ForeignKey
 from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.orm import relationship, validates, synonym
 
-from stalker.db import Base
+from stalker.db.declarative import Base
 from stalker.db.session import DBSession
 from stalker.log import logging_level
+from stalker.models import walk_hierarchy
 from stalker.models.entity import Entity, SimpleEntity
 from stalker.models.link import Link
 from stalker.models.status import Status
@@ -262,14 +261,14 @@ class Review(SimpleEntity, ScheduleMixin, StatusMixin):
                     total_seconds += review.schedule_seconds
                     revise_task = True
 
+            timing, unit = self.least_meaningful_time_unit(total_seconds)
             self.task._review_number += 1
             if revise_task:
                 # revise the task timing if the task needs more time
                 if total_seconds > self.task.schedule_seconds:
-                    logger.debug('total_seconds including reviews: %s' %
-                                 total_seconds)
-                    timing, unit = \
-                        self.least_meaningful_time_unit(total_seconds)
+                    logger.debug(
+                        'total_seconds including reviews: %s' % total_seconds
+                    )
 
                     self.task.schedule_timing = timing
                     self.task.schedule_unit = unit
@@ -277,20 +276,31 @@ class Review(SimpleEntity, ScheduleMixin, StatusMixin):
             else:
                 # approve the task
                 self.task.status = cmpl
-                # update task parent statuses
 
+                # also clamp the schedule timing
+                self.task.schedule_timing = timing
+                self.task.schedule_unit = unit
+
+            # update task parent statuses
             self.task.update_parent_statuses()
 
+            from stalker import TaskDependency
             # update dependent task statuses
-            for tdep in self.task.task_dependent_of:
-                dep = tdep.task
+
+            for dep in walk_hierarchy(self.task, 'dependent_of', method=1):
+                logger.debug('current TaskDependency object: %s' % dep)
                 dep.update_status_with_dependent_statuses()
                 if dep.status.code in ['HREV', 'PREV', 'DREV', 'OH', 'STOP']:
                     # for tasks that are still be able to continue to work,
                     # change the dependency_target to "onstart" to allow
                     # the two of the tasks to work together and still let the
                     # TJ to be able to schedule the tasks correctly
-                    tdep.dependency_target = 'onstart'
+                    with DBSession.no_autoflush:
+                        tdeps = TaskDependency.query\
+                            .filter_by(depends_to=dep).all()
+                    for tdep in tdeps:
+                        tdep.dependency_target = 'onstart'
+
                 # also update the status of parents of dependencies
                 dep.update_parent_statuses()
 
@@ -307,7 +317,7 @@ class Daily(Entity, StatusMixin, ProjectMixin):
     The main purpose of a ``Daily`` is to gather a group of :class:`.Link`
     instances and introduce a simple way of presenting them as a group.
 
-    :class:`.Note`\ s created during a Daily session can be directly stored
+    :class:`.Note` s created during a Daily session can be directly stored
     both in the :class:`.Link` and the :class:`.Daily` instances and a *join*
     will reveal which :class:`.Note` is created in which :class:`.Daily`.
     """
@@ -394,8 +404,8 @@ class DailyLink(Base):
         doc="""stalker.models.link.Link instances related to the Daily
         instance.
 
-        Attach the same :class:`.Link`\ s that are linked as an output to a
-        certain :class:`.Version`\ s instance to this attribute.
+        Attach the same :class:`.Link` s that are linked as an output to a
+        certain :class:`.Version` s instance to this attribute.
 
         This attribute is an **association_proxy** so and the real attribute
         that the data is related to is the :attr:`.link_relations` attribute.
