@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import os
 import subprocess
@@ -25,8 +23,8 @@ import datetime
 import time
 import csv
 
-import stalker
-from stalker import defaults
+import pytz
+
 from stalker.log import logging_level
 
 import logging
@@ -201,6 +199,7 @@ class TaskJugglerScheduler(SchedulerBase):
         """creates the tjp file
         """
         self.temp_file_full_path = tempfile.mktemp(prefix='Stalker_')
+        self.temp_file_path = os.path.dirname(self.temp_file_full_path)
         self.temp_file_name = os.path.basename(self.temp_file_full_path)
         self.tjp_file_full_path = self.temp_file_full_path + ".tjp"
         self.csv_file_full_path = self.temp_file_full_path + ".csv"
@@ -214,22 +213,18 @@ class TaskJugglerScheduler(SchedulerBase):
 
         # use new way of doing it, it will just work with PostgreSQL
         import json
-        from stalker import db
+        from stalker import defaults
+        from stalker.db.session import DBSession
+        template = Template(defaults.tjp_main_template2)
 
-        conn = db.DBSession.connection()
-        engine = conn.engine
+        if not self.projects:
+            project_ids = DBSession.connection().execute(
+                'select id, code from "Projects"'
+            ).fetchall()
+        else:
+            project_ids = [[project.id] for project in self.projects]
 
-        if engine.dialect.name == 'postgresql':
-            template = Template(defaults.tjp_main_template2)
-
-            if not self.projects:
-                project_ids = db.DBSession.connection().execute(
-                    'select id, code from "Projects"'
-                ).fetchall()
-            else:
-                project_ids = [[project.id] for project in self.projects]
-
-            sql_query = """select
+        sql_query = """select
     "Tasks".id,
     tasks.path,
     coalesce("Tasks".parent_id, "Tasks".project_id) as parent_id,
@@ -261,7 +256,7 @@ join (
             array[project_id] as path,
             0
         from "Tasks"
-        where parent_id is NULL and project_id = %(id)s
+        where parent_id is NULL and project_id = :id
     union all
         select
             task.id,
@@ -288,7 +283,7 @@ join (
 left outer join (
     select
         task_id,
-        array_agg(resource_id) as resource_ids
+        array_agg(resource_id order by resource_id) as resource_ids
     from "Task_Resources"
     group by task_id
 ) as task_resources on "Tasks".id = task_resources.task_id
@@ -297,7 +292,7 @@ left outer join (
 left outer join (
     select
         task_id,
-        array_agg(resource_id) as resource_ids
+        array_agg(resource_id order by resource_id) as resource_ids
     from "Task_Alternative_Resources"
     group by task_id
 ) as task_alternative_resources on "Tasks".id = task_alternative_resources.task_id
@@ -306,7 +301,7 @@ left outer join (
 left outer join (
     select
         "TimeLogs".task_id,
-        array_agg(('User_' || "TimeLogs".resource_id, to_char("TimeLogs".start, 'YYYY-MM-DD-HH24:MI:00'), to_char("TimeLogs".end, 'YYYY-MM-DD-HH24:MI:00'))) as time_log_array
+        array_agg(('User_' || "TimeLogs".resource_id, to_char(cast("TimeLogs".start at time zone 'utc' as timestamp), 'YYYY-MM-DD-HH24:MI:00'), to_char(cast("TimeLogs".end at time zone 'utc' as timestamp), 'YYYY-MM-DD-HH24:MI:00'))) as time_log_array
     from "TimeLogs"
     group by task_id
 ) as time_logs on "Tasks".id = time_logs.task_id
@@ -345,190 +340,177 @@ left outer join (
 --order by "Tasks".id
 order by path_as_text"""
 
-            result_buffer = []
-            num_of_records = 0
+        result_buffer = []
+        num_of_records = 0
 
-            # run it per project
-            for pr in project_ids:
-                p_id = pr[0]
-                #p_code = pr[1]
+        # run it per project
+        from sqlalchemy import text
+        for pr in project_ids:
+            p_id = pr[0]
+            #p_code = pr[1]
 
-                sql_query_pp = sql_query % {'id': p_id}
-                result = db.DBSession.connection().execute(sql_query_pp)
+            result = DBSession.connection().execute(text(sql_query), id=p_id)
 
-                # start by adding the project first
-                result_buffer.append('task Project_%s "Project_%s" {' % (p_id, p_id))
+            # start by adding the project first
+            result_buffer.append(
+                'task Project_%s "Project_%s" {' % (p_id, p_id)
+            )
 
-                # now start jumping around
-                previous_level = 0
-                for r in result.fetchall():
-                    # start by appending task tjp id first
-                    task_id = r[0]
-                    # path = r[1]
-                    # parent_id = r[2]
-                    # entity_type = r[3]
-                    #name = r[4]
-                    priority = r[5]
-                    schedule_timing = r[6]
-                    schedule_unit = r[7]
-                    schedule_model = r[8]
-                    allocation_strategy = r[9]
-                    persistent_allocation = r[10]
-                    depth = r[11] + 1
-                    resource_ids = r[12]
-                    alternative_resource_ids = r[13]
-                    time_log_array = r[14]
-                    dependency_info = r[15]
-                    is_leaf = r[16]
+            # now start jumping around
+            previous_level = 0
+            for r in result.fetchall():
+                # start by appending task tjp id first
+                task_id = r[0]
+                # path = r[1]
+                # parent_id = r[2]
+                # entity_type = r[3]
+                #name = r[4]
+                priority = r[5]
+                schedule_timing = r[6]
+                schedule_unit = r[7]
+                schedule_model = r[8]
+                allocation_strategy = r[9]
+                persistent_allocation = r[10]
+                depth = r[11] + 1
+                resource_ids = r[12]
+                alternative_resource_ids = r[13]
+                time_log_array = r[14]
+                dependency_info = r[15]
+                is_leaf = r[16]
 
-                    tab = '  ' * depth
+                tab = '  ' * depth
 
-                    # close the previous level if necessary
-                    for i in range(previous_level - depth + 1):
-                        i_tab = '  ' * (previous_level - i)
-                        result_buffer.append('%s}' % i_tab)
+                # close the previous level if necessary
+                for i in range(previous_level - depth + 1):
+                    i_tab = '  ' * (previous_level - i)
+                    result_buffer.append('%s}' % i_tab)
 
+                result_buffer.append(
+                    """%(tab)stask Task_%(id)s "Task_%(id)s" {""" % {
+                        'tab': tab,
+                        'id': task_id
+                    }
+                )
+
+                # append priority if it is different then 500
+                if priority != 500:
                     result_buffer.append(
-                        """%(tab)stask Task_%(id)s "Task_%(id)s" {""" % {
-                            'tab': tab,
-                            'id': task_id
-                        }
+                        '%s  priority %s' % (tab, priority))
+
+                # append dependency information
+                if dependency_info:
+                    dep_buffer = ['%s  depends ' % tab]
+
+                    json_data = json.loads(
+                        dependency_info.replace('{', '[')
+                        .replace('}', ']')
+                        .replace('(', '')
+                        .replace(')', '')
+                    )  # it is an array of string
+
+                    for i, dep in enumerate(json_data):
+                        if i > 0:
+                            dep_buffer.append(', ')
+
+                        dep_full_ids, \
+                            dependency_target, \
+                            gap_timing, \
+                            gap_unit, \
+                            gap_model = dep.split(',')
+
+                        dep_full_path = '.'.join(
+                            map(lambda x: 'Task_%s' % x,
+                                dep_full_ids.split('-'))
+                        )
+                        # fix for Project id
+                        dep_full_path = 'Project_%s' % dep_full_path[5:]
+
+                        dep_string = '%s {%s}' % (
+                            dep_full_path, dependency_target)
+
+                        dep_buffer.append(dep_string)
+
+                    result_buffer.append(''.join(dep_buffer))
+
+                # append schedule model and timing information
+                # if this is a leaf task and has resources
+                if is_leaf and resource_ids:
+                    result_buffer.append(
+                        '%s  %s %s%s' % (
+                            tab, schedule_model, schedule_timing,
+                            schedule_unit
+                        )
                     )
 
-                    # append priority if it is different then 500
-                    if priority != 500:
-                        result_buffer.append(
-                            '%s  priority %s' % (tab, priority))
+                    resource_buffer = ['%s  allocate ' % tab]
+                    for i, resource_id in enumerate(resource_ids):
+                        if i > 0:
+                            resource_buffer.append(', ')
+                        resource_buffer.append('User_%s' % resource_id)
 
-                    # append dependency information
-                    if dependency_info:
-                        dep_buffer = ['%s  depends ' % tab]
+                        # now go through alternatives
+                        if alternative_resource_ids:
+                            resource_buffer.append(' { alternative ')
+                            for j, alt_resource_id in \
+                                    enumerate(alternative_resource_ids):
+                                if j > 0:
+                                    resource_buffer.append(', ')
+                                resource_buffer.append(
+                                    'User_%s' % alt_resource_id)
 
+                            # set the allocation strategy
+                            resource_buffer.append(
+                                ' select %s' % allocation_strategy)
+
+                            # is is persistent
+                            if persistent_allocation:
+                                resource_buffer.append(' persistent')
+                            resource_buffer.append(' }')
+
+                    result_buffer.append(''.join(resource_buffer))
+
+                    # append any time log information
+                    if time_log_array:
                         json_data = json.loads(
-                            dependency_info.replace('{', '[')
+                            time_log_array.replace('{', '[')
                             .replace('}', ']')
                             .replace('(', '')
                             .replace(')', '')
                         )  # it is an array of string
 
-                        for i, dep in enumerate(json_data):
-                            if i > 0:
-                                dep_buffer.append(', ')
-
-                            dep_full_ids, \
-                                dependency_target, \
-                                gap_timing, \
-                                gap_unit, \
-                                gap_model = dep.split(',')
-
-                            dep_full_path = '.'.join(
-                                map(lambda x: 'Task_%s' % x,
-                                    dep_full_ids.split('-'))
-                            )
-
-                            dep_string = '%s {%s}' % (
-                                dep_full_path, dependency_target)
-
-                            dep_buffer.append(dep_string)
-
-                        result_buffer.append(''.join(dep_buffer))
-
-                    # append schedule model and timing information
-                    # if this is a leaf task and has resources
-                    if is_leaf and resource_ids:
-                        result_buffer.append(
-                            '%s  %s %s%s' % (
-                                tab, schedule_model, schedule_timing,
-                                schedule_unit
-                            )
-                        )
-
-                        resource_buffer = ['%s  allocate ' % tab]
-                        for i, resource_id in enumerate(resource_ids):
-                            if i > 0:
-                                resource_buffer.append(', ')
-                            resource_buffer.append('User_%s' % resource_id)
-
-                            # now go through alternatives
-                            if alternative_resource_ids:
-                                resource_buffer.append(' { alternative ')
-                                for j, alt_resource_id in \
-                                        enumerate(alternative_resource_ids):
-                                    if j > 0:
-                                        resource_buffer.append(', ')
-                                    resource_buffer.append(
-                                        'User_%s' % alt_resource_id)
-
-                                # set the allocation strategy
-                                resource_buffer.append(
-                                    ' select %s' % allocation_strategy)
-
-                                # is is persistent
-                                if persistent_allocation:
-                                    resource_buffer.append(' persistent')
-                                resource_buffer.append(' }')
-
-                        result_buffer.append(''.join(resource_buffer))
-
-                        # append any time log information
-                        if time_log_array:
-                            json_data = json.loads(
-                                time_log_array.replace('{', '[')
-                                .replace('}', ']')
-                                .replace('(', '')
-                                .replace(')', '')
-                            )  # it is an array of string
-
-                            for tlog in json_data:
-                                user_id, t_start, t_end = tlog.split(',')
-                                result_buffer.append(
-                                    '%s  booking %s %s - %s { overtime 2 }' % (
-                                        tab, user_id, t_start, t_end
-                                    )
+                        for tlog in json_data:
+                            user_id, t_start, t_end = tlog.split(',')
+                            result_buffer.append(
+                                '%s  booking %s %s - %s { overtime 2 }' % (
+                                    tab, user_id, t_start, t_end
                                 )
+                            )
 
-                    previous_level = depth
-                    num_of_records += 1
+                previous_level = depth
+                num_of_records += 1
 
-                # and close the brackets per project
-                depth = 0  # current depth is 0 (Project)
-                # previous_level is the last task
-                for i in range(previous_level - depth + 1):
-                    i_tab = '  ' * (previous_level - i)
-                    result_buffer.append('%s}' % i_tab)
+            # and close the brackets per project
+            depth = 0  # current depth is 0 (Project)
+            # previous_level is the last task
+            for i in range(previous_level - depth + 1):
+                i_tab = '  ' * (previous_level - i)
+                result_buffer.append('%s}' % i_tab)
 
-            tasks_buffer = '\n'.join(result_buffer)
+        tasks_buffer = '\n'.join(result_buffer)
 
-            self.tjp_content = template.render({
-                'stalker': stalker,
-                'studio': self.studio,
-                'csv_file_name': self.temp_file_name,
-                'csv_file_full_path': self.temp_file_full_path,
-                'compute_resources': self.compute_resources,
-                'tasks_buffer': tasks_buffer
-            })
+        import stalker
+        self.tjp_content = template.render({
+            'stalker': stalker,
+            'studio': self.studio,
+            'csv_file_name': self.temp_file_name,
+            'csv_file_full_path': self.temp_file_full_path,
+            'compute_resources': self.compute_resources,
+            'tasks_buffer': tasks_buffer
+        })
 
-            logger.debug(
-                'total number of records: %s' % num_of_records
-            )
-
-        else:
-            # fallback to the previous implementation
-            template = Template(defaults.tjp_main_template)
-
-            if self.projects:
-                projects = self.projects
-            else:
-                projects = self.studio.active_projects
-
-            self.tjp_content = template.render({
-                'stalker': stalker,
-                'studio': self.studio,
-                'projects': projects,
-                'csv_file_name': self.temp_file_name,
-                'csv_file_full_path': self.temp_file_full_path,
-                'compute_resources': self.compute_resources
-            })
+        logger.debug(
+            'total number of records: %s' % num_of_records
+        )
 
         end = time.time()
         logger.debug(
@@ -565,7 +547,7 @@ order by path_as_text"""
 
     def _parse_csv_file(self):
         """parses back the csv file and fills the tasks with computes_start and
-        computed end values
+        computed_end values
         """
         parsing_start = time.time()
 
@@ -575,7 +557,7 @@ order by path_as_text"""
                          'returning without updating db!')
             return
 
-        from stalker import db, Task, Project
+        from stalker import Task, Project
         from stalker.models.task import Task_Computed_Resources
 
         entity_ids = []
@@ -603,6 +585,10 @@ order by path_as_text"""
                     "%Y-%m-%d-%H:%M"
                 )
 
+                # implement time zone info
+                start_date = start_date.replace(tzinfo=pytz.utc)
+                end_date = end_date.replace(tzinfo=pytz.utc)
+
                 # computed_resources
                 if self.compute_resources:
                     if data[3] != '':
@@ -635,7 +621,8 @@ order by path_as_text"""
                 computed_start=bindparam('computed_start'),
                 computed_end=bindparam('computed_end')
             )
-        db.DBSession.connection().execute(
+        from stalker.db.session import DBSession
+        DBSession.connection().execute(
             update_statement,
             update_data
         )
@@ -649,24 +636,24 @@ order by path_as_text"""
                 computed_start=bindparam('computed_start'),
                 computed_end=bindparam('computed_end')
             )
-        db.DBSession.connection().execute(
+        DBSession.connection().execute(
             update_project_statement,
             update_data
         )
 
         # update computed resources data
         # first delete everything
-        delete_resources_statement = Task_Computed_Resources.delete()
-
-        update_resources_statement = Task_Computed_Resources.insert()\
-            .values(
-                task_id=bindparam('task_id'),
-                resource_id=bindparam('resource_id')
-            )
-
         if self.compute_resources:
-            db.DBSession.connection().execute(delete_resources_statement)
-            db.DBSession.connection().execute(
+            delete_resources_statement = Task_Computed_Resources.delete()
+
+            update_resources_statement = Task_Computed_Resources.insert()\
+                .values(
+                    task_id=bindparam('task_id'),
+                    resource_id=bindparam('resource_id')
+                )
+
+            DBSession.connection().execute(delete_resources_statement)
+            DBSession.connection().execute(
                 update_resources_statement,
                 update_user_data
             )
@@ -702,36 +689,52 @@ order by path_as_text"""
         logger.debug('tjp_file_full_path: %s' % self.tjp_file_full_path)
 
         # pass it to tj3
-        process = subprocess.Popen(
-            [defaults.tj_command,
-             self.tjp_file_full_path],
-            stderr=subprocess.PIPE
-        )
+        from stalker import defaults
+        if os.name == 'nt':
+            logger.debug('tj3 using fallback mode for Windows!')
+            command = '%s %s -o %s' % (
+                defaults.tj_command,
+                self.tjp_file_full_path,
+                self.temp_file_path,
+            )
+            logger.debug('tj3 command: %s' % command)
+            return_code = os.system(command)
+            stderr_buffer = ''
+        else:
+            process = subprocess.Popen(
+                [defaults.tj_command,
+                 self.tjp_file_full_path,
+                 '-o',
+                 self.temp_file_path],
+                stderr=subprocess.PIPE
+            )
 
-        # loop until process finishes and capture stderr output
-        stderr_buffer = []
-        while True:
-            stderr = process.stderr.readline()
+            # loop until process finishes and capture stderr output
+            stderr_buffer = []
+            while True:
+                stderr = process.stderr.readline()
 
-            if stderr == b'' and process.poll() is not None:
-                break
+                if stderr == b'' and process.poll() is not None:
+                    break
 
-            if stderr != b'':
-                stderr = stderr.strip()
-                stderr_buffer.append(stderr)
-                logger.debug(stderr.strip())
+                if stderr != b'':
+                    stderr = stderr.decode('utf-8').strip()
+                    stderr_buffer.append(stderr)
+                    logger.debug(stderr)
 
-        # flatten the buffer
-        stderr_buffer = '\n'.join(stderr_buffer)
+            # flatten the buffer
+            stderr_buffer = '\n'.join(stderr_buffer)
 
-        if process.returncode:
+            return_code = process.returncode
+
+        if return_code:
             # there is an error
             raise RuntimeError(stderr_buffer)
 
         # read back the csv file
         self._parse_csv_file()
 
-        logger.debug('tj3 return code: %s' % process.returncode)
+        logger.debug('tj3 return code: %s' % return_code)
 
         # remove the tjp file
         self._clean_up()
