@@ -1,25 +1,23 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
-from sqlalchemy import Column, Integer, ForeignKey, Table
-from sqlalchemy.orm import relationship, synonym, validates, reconstructor
+from sqlalchemy import Column, Integer, ForeignKey, Table, Float
+from sqlalchemy.orm import relationship, validates, reconstructor, synonym
 
 from stalker import ImageFormat
 from stalker.db.declarative import Base
@@ -115,12 +113,21 @@ class Shot(Task, CodeMixin):
 
        :attr:`.handles_at_start` and :attr:`.handles_at_end` attributes.
 
+    .. note::
+
+       .. versionadded:: 0.2.17.2
+
+       Per shot FPS values. It is now possible to change the shot fps by
+       setting its :attr:`.fps` attribute. The default values is same with the
+       :class:`.Project`\ .
+
+
     :param project: This is the :class:`.Project` instance that this shot
       belongs to. A Shot can not be created without a Project instance.
 
     :type project: :class:`.Project`
 
-    :param sequences: This is a list of :class:`.Sequence`\ s that this shot is
+    :param sequences: This is a list of :class:`.Sequence` s that this shot is
       assigned to. A Shot can be created without having a Sequence instance.
 
     :type sequences: list of :class:`.Sequence`
@@ -144,6 +151,9 @@ class Shot(Task, CodeMixin):
       the same with the Project that this Shot belongs to.
 
     :type image_format: :class:`.ImageFormat`
+
+    :param float fps: The FPS of this shot. Default value is the same with the
+      :class:`.Project`\ .
     """
     __auto_name__ = True
     __tablename__ = 'Shots'
@@ -169,7 +179,7 @@ class Shot(Task, CodeMixin):
     )
 
     image_format_id = Column(Integer, ForeignKey("ImageFormats.id"))
-    image_format = relationship(
+    _image_format = relationship(
         "ImageFormat",
         primaryjoin="Shots.c.image_format_id==ImageFormats.c.id",
         doc="""The :class:`.ImageFormat` of this shot.
@@ -211,6 +221,16 @@ class Shot(Task, CodeMixin):
     )
     #record_out = Column(Integer)
 
+    _fps = Column(
+        'fps',
+        Float(precision=3),
+        doc="""The fps of the project.
+
+        It is a float value, any other types will be converted to float. The
+        default value is equal to :attr:`stalker.modesl.project..Project.fps`.
+        """
+    )
+
     def __init__(self,
                  code=None,
                  project=None,
@@ -222,6 +242,7 @@ class Shot(Task, CodeMixin):
                  source_out=None,
                  record_in=None,
                  image_format=None,
+                 fps=None,
                  **kwargs):
 
         # initialize TaskableMixin
@@ -271,6 +292,8 @@ class Shot(Task, CodeMixin):
         self.source_out = source_out
         self.record_in = record_in
 
+        self.fps = fps
+
     @reconstructor
     def __init_on_load__(self):
         """initialize on DB load
@@ -304,14 +327,65 @@ class Shot(Task, CodeMixin):
         :return: bool
         """
         if project and code:
-            from stalker import db
-            with db.DBSession.no_autoflush:
-                return Shot.query\
-                           .filter(Shot.project == project)\
-                           .filter(Shot.code == code)\
-                           .first() is None
+            from stalker.db.session import DBSession
+            from sqlalchemy.exc import UnboundExecutionError, OperationalError
+            try:
+                with DBSession.no_autoflush:
+                    return Shot.query\
+                               .filter(Shot.project == project)\
+                               .filter(Shot.code == code)\
+                               .first() is None
+            except (UnboundExecutionError, OperationalError) as e:
+                # Fallback to Python
+                for t in project.tasks:
+                    if isinstance(t, Shot):
+                        if t.code == code:
+                            return False
         return True
 
+    def _fps_getter(self):
+        """returns the fps value either from the Project or from the _fps
+        attribute
+        """
+        if self._fps is None:
+            return self.project.fps
+        else:
+            return self._fps
+
+    def _fps_setter(self, fps_in):
+        """sets the fps value
+        """
+        self._fps = self._validate_fps(fps_in)
+
+    fps = synonym(
+        '_fps',
+        descriptor=property(_fps_getter, _fps_setter),
+        doc='The fps of this shot.'
+    )
+
+    def _validate_fps(self, fps):
+        """validates the given fps_in value
+        """
+        if fps is None:
+            # fps = self.project.fps
+            return None
+
+        if not isinstance(fps, (int, float)):
+            raise TypeError(
+                '%s.fps should be a positive float or int, not %s' % (
+                    self.__class__.__name__,
+                    fps.__class__.__name__
+                )
+            )
+
+        fps = float(fps)
+        if fps <= 0:
+            raise ValueError(
+                '%s.fps should be a positive float or int, not %s' %
+                (self.__class__.__name__, fps)
+            )
+        return float(fps)
+
     @validates('cut_in')
     def _validate_cut_in(self, key, cut_in):
         """validates the cut_in value
@@ -518,21 +592,41 @@ class Shot(Task, CodeMixin):
             )
         return scene
 
-    @validates('image_format')
-    def _validate_image_format(self, key, imf):
+    def _image_format_getter(self):
+        """returns the image_format value from the Project or from the
+        _image_format attribute
+        """
+        if self._image_format is None:
+            return self.project.image_format
+        else:
+            return self._image_format
+
+    def _image_format_setter(self, imf_in):
+        """sets the image_format value
+        """
+        self._image_format = self._validate_image_format(imf_in)
+
+    image_format = synonym(
+        '_image_format',
+        descriptor=property(_image_format_getter, _image_format_setter),
+        doc='The image_format of this shot. Set it to None to re-sync with '
+            'Project.image_format.'
+    )
+
+    def _validate_image_format(self, imf):
         """validates the given imf value
         """
         if imf is None:
-            # use the projects image format
-            imf = self.project.image_format
+            # do not set it to anything it will automatically use the proejct
+            # image format
+            return None
 
-        if imf is not None:
-            if not isinstance(imf, ImageFormat):
-                raise TypeError(
-                    '%s.image_format should be an instance of '
-                    'stalker.models.format.ImageFormat, not %s' %
-                    (self.__class__.__name__, imf.__class__.__name__)
-                )
+        if not isinstance(imf, ImageFormat):
+            raise TypeError(
+                '%s.image_format should be an instance of '
+                'stalker.models.format.ImageFormat, not %s' %
+                (self.__class__.__name__, imf.__class__.__name__)
+            )
 
         return imf
 
