@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import copy
 import logging
@@ -24,11 +22,11 @@ import time
 import datetime
 from math import ceil
 
-from sqlalchemy import (Column, Integer, ForeignKey, Interval, Boolean,
-                        DateTime, PickleType)
+from sqlalchemy import (Column, Integer, ForeignKey, Interval, Boolean, Text)
 from sqlalchemy.orm import validates, relationship, synonym, reconstructor
 
-from stalker import db, defaults, log
+from stalker import log
+from stalker.db.types import GenericJSON, GenericDateTime
 from stalker.models.entity import SimpleEntity, Entity
 from stalker.models.mixins import DateRangeMixin, WorkingHoursMixin
 from stalker.models.schedulers import SchedulerBase
@@ -93,7 +91,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
     :param now: The now attribute overrides the TaskJugglers ``now`` attribute
       allowing the user to schedule the projects as if the scheduling is done
       on that date. The default value is the rounded value of
-      datetime.datetime.now().
+      datetime.datetime.now(pytz.utc).
 
     :type now: datetime.datetime
 
@@ -132,13 +130,13 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         doc='The User who is scheduling the Studio projects right now'
     )
     scheduling_started_at = Column(
-        DateTime,
+        GenericDateTime,
         doc='Stores when the current scheduling is started at, it is a good '
             'measure for measuring if the last schedule is not correctly '
             'finished'
     )
     last_scheduled_at = Column(
-        DateTime,
+        GenericDateTime,
         doc='Stores the last schedule date'
     )
     last_scheduled_by_id = Column(
@@ -152,7 +150,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         doc='The User who has last scheduled the Studio projects'
     )
     last_schedule_message = Column(
-        PickleType,
+        Text,
         doc='Holds the last schedule message, generally coming generated by '
         'TaskJuggler'
     )
@@ -207,7 +205,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         except AttributeError:
             # The Studio and WorkingHours classes has changed from
             # v0.2.3 to v0.2.5 and with this change it is simply
-            # no possible to initialize the db if we insist to update the
+            # not possible to initialize the db if we insist to update the
             # defaults for daily_working_hours, because there is no
             # WorkingHours._daily_working_hours in versions before than
             # v0.2.5, so just skip it for at least the studio instance
@@ -277,7 +275,8 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         """validates the given now_in value
         """
         if now_in is None:
-            now_in = datetime.datetime.now()
+            import pytz
+            now_in = datetime.datetime.now(pytz.utc)
 
         if not isinstance(now_in, datetime.datetime):
             raise TypeError(
@@ -292,11 +291,14 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
     def now(self):
         """now getter
         """
+        import pytz
         try:
             if self._now is None:
-                self._now = self.round_time(datetime.datetime.now())
+                self._now = self.round_time(datetime.datetime.now(pytz.utc))
         except AttributeError:
-            setattr(self, '_now', self.round_time(datetime.datetime.now()))
+            setattr(
+                self, '_now', self.round_time(datetime.datetime.now(pytz.utc))
+            )
         return self._now
 
     @now.setter
@@ -334,6 +336,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         """converts the studio to a tjp representation
         """
         from jinja2 import Template
+        from stalker import defaults
 
         temp = Template(
             defaults.tjp_studio_template,
@@ -341,10 +344,13 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
             lstrip_blocks=True
         )
         start = time.time()
+        import pytz
         rendered_template = temp.render({
             'studio': self,
             'datetime': datetime,
-            'now': self.round_time(self.now).strftime('%Y-%m-%d-%H:%M')
+            'utc': pytz.utc,
+            'now': self.round_time(self.now).astimezone(pytz.utc)
+                .strftime('%Y-%m-%d-%H:%M')
         })
         end = time.time()
         logger.debug('render studio to tjp took: %s seconds' % (end - start))
@@ -410,8 +416,10 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
                 }
             )
 
-        with db.DBSession.no_autoflush:
-            self.scheduling_started_at = datetime.datetime.now()
+        import pytz
+        from stalker.db.session import DBSession
+        with DBSession.no_autoflush:
+            self.scheduling_started_at = datetime.datetime.now(pytz.utc)
 
             # run the scheduler
             self.scheduler.studio = self
@@ -425,7 +433,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
             result = self.scheduler.schedule()
         finally:
             # in any case set is_scheduling to False
-            with db.DBSession.no_autoflush:
+            with DBSession.no_autoflush:
                 self.is_scheduling = False
                 self.is_scheduling_by = None
 
@@ -434,8 +442,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
                 self.last_schedule_message = result
 
                 # And the date the schedule is completed
-                # TODO: convert to UTC time
-                self.last_scheduled_at = datetime.datetime.now()
+                self.last_scheduled_at = datetime.datetime.now(pytz.utc)
 
                 # and who has done the scheduling
                 if scheduled_by:
@@ -510,6 +517,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         """validates the given timing_resolution value
         """
         if timing_resolution is None:
+            from stalker import defaults
             timing_resolution = defaults.timing_resolution
 
         if not isinstance(timing_resolution, datetime.timedelta):
@@ -522,7 +530,7 @@ class Studio(Entity, DateRangeMixin, WorkingHoursMixin):
         return timing_resolution
 
 
-class WorkingHours(object):
+class WorkingHours(Entity):
     """A helper class to manage Studio working hours.
 
     Working hours is a data class to store the weekly working hours pattern of
@@ -578,15 +586,28 @@ class WorkingHours(object):
       value is going to be used.
     """
 
+    __auto_name__ = True
+    __tablename__ = 'WorkingHours'
+    __mapper_args__ = {'polymorphic_identity': 'WorkingHours'}
+
+    working_hours_id = \
+        Column('id', Integer, ForeignKey('Entities.id'), primary_key=True)
+
+    working_hours = Column(GenericJSON)
+
+    # I know it is not a very common place to import modules
+    from stalker import defaults
+    daily_working_hours = Column(Integer, default=defaults.daily_working_hours)
+
     def __init__(self,
                  working_hours=None,
                  daily_working_hours=None,
                  **kwargs):
+        super(WorkingHours, self).__init__(**kwargs)
         if working_hours is None:
+            from stalker import defaults
             working_hours = defaults.working_hours
-        self._wh = None
-        self.working_hours = self._validate_working_hours(working_hours)
-        self._daily_working_hours = None
+        self.working_hours = working_hours
         self.daily_working_hours = daily_working_hours
 
     def __eq__(self, other):
@@ -595,31 +616,30 @@ class WorkingHours(object):
         return isinstance(other, WorkingHours) and \
             other.working_hours == self.working_hours
 
-    def __hash__(self):
-        """the overridden __hash__ method
-        """
-        return hash(self.working_hours)
-
     def __getitem__(self, item):
+        from stalker import __string_types__
         if isinstance(item, int):
-            return self._wh[defaults.day_order[item]]
-        elif isinstance(item, str):
-            return self._wh[item]
+            from stalker import defaults
+            return self.working_hours[defaults.day_order[item]]
+        elif isinstance(item, __string_types__):
+            return self.working_hours[item]
 
     def __setitem__(self, key, value):
         self._validate_wh_value(value)
+        from stalker import __string_types__, defaults
         if isinstance(key, int):
-            self._wh[defaults.day_order[key]] = value
-        elif isinstance(key, str):
+            self.working_hours[defaults.day_order[key]] = value
+        elif isinstance(key, __string_types__):
             # check if key is in
             if key not in defaults.day_order:
                 raise KeyError(
-                    '%s accepts only %s as key, not %s' %
+                    "%s accepts only %s as key, not '%s'" %
                     (self.__class__.__name__, defaults.day_order, key)
                 )
-            self._wh[key] = value
+            self.working_hours[key] = value
 
-    def _validate_working_hours(self, wh_in):
+    @validates('working_hours')
+    def _validate_working_hours(self, key, wh_in):
         """validates the given working hours
         """
         if not isinstance(wh_in, dict):
@@ -643,24 +663,13 @@ class WorkingHours(object):
 
         # update the default values with the supplied working_hour dictionary
         # copy the defaults
+        from stalker import defaults
         wh_def = copy.copy(defaults.working_hours)
         # update them
         wh_def.update(wh_in)
 
         return wh_def
 
-    @property
-    def working_hours(self):
-        """the getter of _wh
-        """
-        return self._wh
-
-    @working_hours.setter
-    def working_hours(self, wh_in):
-        """the setter of _wh
-        """
-        self._wh = self._validate_working_hours(wh_in)
-
     def is_working_hour(self, check_for_date):
         """checks if the given datetime is in working hours
 
@@ -722,6 +731,7 @@ class WorkingHours(object):
         """
         # render the template
         from jinja2 import Template
+        from stalker import defaults
 
         template = Template(defaults.tjp_working_hours_template)
         return template.render({'workinghours': self})
@@ -753,10 +763,12 @@ class WorkingHours(object):
         """
         return int(ceil(self.weekly_working_days * 52.1428))
 
-    def _validate_daily_working_hours(self, dwh):
+    @validates('daily_working_hours')
+    def _validate_daily_working_hours(self, key, dwh):
         """validates the given daily working hours value
         """
         if dwh is None:
+            from stalker import defaults
             dwh = defaults.daily_working_hours
 
         if not isinstance(dwh, int):
@@ -768,22 +780,11 @@ class WorkingHours(object):
         if dwh <= 0 or dwh > 24:
             raise ValueError(
                 '%s.daily_working_hours should be a positive integer value '
-                'greater than 0 and smaller than or equal to 24'
+                'greater than 0 and smaller than or equal to 24' %
+                self.__class__.__name__
             )
         return dwh
 
-    @property
-    def daily_working_hours(self):
-        """getter for daily_working_hours attribute
-        """
-        return self._daily_working_hours
-
-    @daily_working_hours.setter
-    def daily_working_hours(self, dwh):
-        """setter for daily_working_hours attribute
-        """
-        self._daily_working_hours = self._validate_daily_working_hours(dwh)
-
     def split_in_to_working_hours(self, start, end):
         """splits the given start and end datetime objects in to working hours
         """
@@ -852,6 +853,10 @@ class Vacation(SimpleEntity, DateRangeMixin):
         """overridden to_tjp method
         """
         from jinja2 import Template
-
+        from stalker import defaults
+        import pytz
         template = Template(defaults.tjp_vacation_template)
-        return template.render({'vacation': self})
+        return template.render({
+            'vacation': self,
+            'utc': pytz.utc
+        })
