@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import uuid
 
@@ -31,8 +29,6 @@ from stalker.db.declarative import Base
 from stalker.models.entity import Entity, SimpleEntity
 from stalker.models.mixins import StatusMixin
 
-from stalker import defaults
-
 from stalker.log import logging_level
 import logging
 
@@ -102,7 +98,7 @@ class Ticket(Entity, StatusMixin):
     automatically filled accordingly. For newly created Tickets the status of
     the ticket is ``NEW`` and can be changed to other statuses as follows:
 
-        Status -> Action -> New Status
+        Status   -> Action   -> New Status
 
         NEW      -> resolve  -> CLOSED
         NEW      -> accept   -> ACCEPTED
@@ -154,6 +150,7 @@ class Ticket(Entity, StatusMixin):
         "id", Integer, ForeignKey("Entities.id"), primary_key=True
     )
 
+    # TODO: use ProjectMixin
     project_id = Column('project_id', Integer, ForeignKey('Projects.id'),
                         nullable=False)
 
@@ -244,7 +241,8 @@ class Ticket(Entity, StatusMixin):
                  summary=None, **kwargs):
         # just force auto name generation
         self._number = self._generate_ticket_number()
-        kwargs['name'] = defaults.ticket_label + ' #%i' % self.number
+        from stalker import defaults
+        kwargs['name'] = '%s #%i' % (defaults.ticket_label, self.number)
 
         super(Ticket, self).__init__(**kwargs)
         StatusMixin.__init__(self, **kwargs)
@@ -287,9 +285,11 @@ class Ticket(Entity, StatusMixin):
         """
         try:
             # do your query
-            max_ticket = Ticket.query \
-                .order_by(Ticket.number.desc()) \
-                .first()
+            from stalker.db.session import DBSession
+            with DBSession.no_autoflush:
+                max_ticket = Ticket.query \
+                    .order_by(Ticket.number.desc()) \
+                    .first()
         except UnboundExecutionError:
             max_ticket = None
 
@@ -360,6 +360,7 @@ class Ticket(Entity, StatusMixin):
         :param stalker.models.auth.User created_by: The User creating this
             action
         """
+        from stalker import defaults
         statuses = defaults.ticket_workflow[action].keys()
         status = self.status.name
         return_value = None
@@ -466,6 +467,7 @@ class TicketLog(SimpleEntity):
 
     .. _Track Workflow: http://trac.edgewall.org/wiki/TracWorkflow
     """
+    from stalker import defaults  # need to limit it with a scope
 
     # TODO: there are no tests for the TicketLog class
 
