@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 from sqlalchemy import Column, Integer, ForeignKey, String, Boolean
 
@@ -62,10 +60,10 @@ class Type(Entity, TargetEntityTypeMixin, CodeMixin):
       Type(name="Commercial", target_entity_type=Project)
       Type(name="Image", target_entity_type=Link)
 
-    By using :class:`.Type`\ s, one can able to sort and group same type of
+    By using :class:`.Type` s, one can able to sort and group same type of
     entities.
 
-    :class:`.Type`\ s are generally used in :class:`.Structure`\ s.
+    :class:`.Type` s are generally used in :class:`.Structure` s.
 
     :param string target_entity_type: The string defining the target type of
       this :class:`.Type`.
@@ -94,7 +92,7 @@ class Type(Entity, TargetEntityTypeMixin, CodeMixin):
     def __hash__(self):
         """the overridden __hash__ method
         """
-        return hash(self.id) + 2 * hash(self.name) + 3 * hash(self.entity_type)
+        return super(Type, self).__hash__()
 
 
 class EntityType(Base):
