@@ -1,22 +1,20 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import os
 
@@ -24,14 +22,13 @@ import re
 import jinja2
 
 from sqlalchemy import Table, Column, Integer, ForeignKey, String, Boolean
-from sqlalchemy.exc import UnboundExecutionError
+from sqlalchemy.exc import UnboundExecutionError, OperationalError
 from sqlalchemy.orm import relationship, validates
 
 from stalker.db.declarative import Base
-from stalker.models import check_circular_dependency
 from stalker.models.link import Link
 
-from stalker import defaults
+from stalker import DAGMixin
 
 from stalker.log import logging_level
 import logging
@@ -40,7 +37,7 @@ logger = logging.getLogger(__name__)
 logger.setLevel(logging_level)
 
 
-class Version(Link):
+class Version(Link, DAGMixin):
     """Holds information about the created versions (files) for a class:`.Task`
 
     A :class:`.Version` holds information about the
@@ -49,6 +46,33 @@ class Version(Link):
     :class:`.Task` then the information is hold in the
     :class:`.Version` instance.
 
+    .. versionadded: 0.2.13
+
+       After Stalker 0.2.13 the :attr:`.path` become an absolute path which is
+       not anymore merged with the project repository in anyway.
+
+    .. warning:
+
+       For projects those are created prior to Stalker version 0.2.13 and that
+       has a :class:`.Structure` with :class:`.FilenameTemplate` that doesn't
+       include the repository info, it is suggested to update the related
+       ``FilenameTemplate`` s to include a the repository info manually.
+
+       Example:
+         pre 0.2.13 setup:
+
+         FilenameTemplate with path attribute is set to:
+
+           {{project.code}}/{%- for parent_task in parent_tasks -%}{{parent_task.nice_name}}/{%- endfor -%}
+
+         Update to:
+
+           {{project.repository.path}}/{{project.code}}/{%- for parent_task in parent_tasks -%}{{parent_task.nice_name}}/{%- endfor -%}
+
+         Or, lets have a setup with environment variables:
+
+           $REPO{{project.repository.id}}/{{project.code}}/{%- for parent_task in parent_tasks -%}{{parent_task.nice_name}}/{%- endfor -%}
+
     :param str take_name: A short string holding the current take name. Takes
       in Stalker are used solely for grouping individual versions together.
       Versions with the same ``take_name`` (of the same Task) are numbered
@@ -81,12 +105,18 @@ class Version(Link):
 
     :type parent: :class:`.Version`
     """
+    from stalker import defaults
     __auto_name__ = True
     __tablename__ = "Versions"
     __mapper_args__ = {"polymorphic_identity": "Version"}
 
+    __dag_cascade__ = "save-update, merge"
+
     version_id = Column("id", Integer, ForeignKey("Links.id"),
                         primary_key=True)
+
+    __id_column__ = 'version_id'
+
     task_id = Column(Integer, ForeignKey("Tasks.id"), nullable=False)
     task = relationship(
         "Task",
@@ -113,25 +143,6 @@ class Version(Link):
         """
     )
 
-    parent_id = Column('parent_id', Integer, ForeignKey('Versions.id'))
-    parent = relationship(
-        'Version',
-        remote_side=[version_id],
-        primaryjoin='Versions.c.parent_id==Versions.c.id',
-        back_populates='children',
-        post_update=True
-    )
-
-    children = relationship(
-        'Version',
-        primaryjoin='Versions.c.parent_id==Versions.c.id',
-        back_populates='parent',
-        post_update=True,
-        doc="""The children :class:`.Version` instances which are derived from
-        this particular Version instance.
-        """
-    )
-
     inputs = relationship(
         "Link",
         secondary="Version_Inputs",
@@ -171,6 +182,8 @@ class Version(Link):
         kwargs['full_path'] = full_path
         super(Version, self).__init__(**kwargs)
 
+        DAGMixin.__init__(self, parent=parent)
+
         self.take_name = take_name
         self.task = task
         self.version_number = None
@@ -185,7 +198,6 @@ class Version(Link):
 
         self.is_published = False
 
-        self.parent = parent
         self.created_with = created_with
 
     def __repr__(self):
@@ -254,13 +266,27 @@ class Version(Link):
         :returns: :class:`.Version` instance
         """
         try:
-            last_version = Version.query\
-                .filter(Version.task == self.task)\
-                .filter(Version.take_name == self.take_name)\
-                .order_by(Version.version_number.desc())\
-                .first()
-        except UnboundExecutionError:
-            last_version = None
+            from stalker.db.session import DBSession
+            with DBSession.no_autoflush:
+                last_version = Version.query\
+                    .filter(Version.task == self.task)\
+                    .filter(Version.take_name == self.take_name)\
+                    .order_by(Version.version_number.desc())\
+                    .first()
+        except (UnboundExecutionError, OperationalError):
+            all_versions = \
+                sorted(
+                    self.task.versions,
+                    key=lambda x: x.version_number if x.version_number else -1
+                )
+            if all_versions:
+                last_version = all_versions[-1]
+                if last_version != self:
+                    return last_version
+                else:
+                    return 0
+            else:
+                last_version = None
         return last_version
 
     @property
@@ -280,11 +306,7 @@ class Version(Link):
         """validates the given version_number value
         """
         # get the latest version
-        # and do it with auto flush turned off,
-        # or it will find itself and increase the version number unnecessarily
-        from stalker.db.session import DBSession
-        with DBSession.no_autoflush:
-            latest_version = self.latest_version
+        latest_version = self.latest_version
 
         max_version_number = 0
         if latest_version:
@@ -360,35 +382,6 @@ class Version(Link):
 
         return output
 
-    @validates('parent')
-    def _validate_parent(self, key, parent):
-        """validates the given parent value
-        """
-        if parent is not None:
-            if not isinstance(parent, Version):
-                raise TypeError(
-                    '%s.parent should be a stalker.models.version.Version '
-                    'instance, not %s' %
-                    (self.__class__.__name__, parent.__class__.__name__)
-                )
-
-        # check for CircularDependency
-        check_circular_dependency(self, parent, 'children')
-
-        return parent
-
-    @validates('children')
-    def _validate_children(self, key, child):
-        """validates the given child value
-        """
-        if not isinstance(child, Version):
-            raise TypeError(
-                'All elements in %s.children should be a '
-                'stalker.models.version.Version instance, not %s' %
-                (self.__class__.__name__, child.__class__.__name__)
-            )
-        return child
-
     def _template_variables(self):
         """variables used in rendering the filename template
         """
@@ -428,12 +421,9 @@ class Version(Link):
         # get a suitable FilenameTemplate
         structure = self.task.project.structure
 
-        from stalker import FilenameTemplate
-
         vers_template = None
         if structure:
             for template in structure.templates:
-                assert isinstance(template, FilenameTemplate)
                 if template.target_entity_type == self.task.entity_type:
                     vers_template = template
                     break
@@ -482,27 +472,24 @@ class Version(Link):
 
         :return: str
         """
-        project = self.task.project
-        repo = project.repository
-
-        return os.path.join(
-            repo.path,
-            self.full_path
+        return os.path.normpath(
+            os.path.expandvars(
+                self.full_path
+            )
         ).replace('\\', '/')
 
     @property
     def absolute_path(self):
-        """Returns the absolute path of this version including the repository
-        path of the related project
+        """Returns the absolute path.
+
+        Due to the changes in the project.repository
 
         :return: str
         """
-        project = self.task.project
-        repo = project.repository
-
-        return os.path.join(
-            repo.path,
-            self.path
+        return os.path.normpath(
+            os.path.expandvars(
+                self.path
+            )
         ).replace('\\', '/')
 
     def is_latest_published_version(self):
@@ -591,15 +578,6 @@ class Version(Link):
             '_' + self.take_name
         )
 
-    def walk_hierarchy(self, method=0):
-        """Walks the hierarchy of this version
-
-        :param method: The walk method, 0: Depth First, 1: Breadth First
-        """
-        from stalker.models import walk_hierarchy
-        for v in walk_hierarchy(self, 'children', method=method):
-            yield v
-
     def walk_inputs(self, method=0):
         """Walks the inputs of this version
 
@@ -613,7 +591,12 @@ class Version(Link):
 Version_Inputs = Table(
     "Version_Inputs", Base.metadata,
     Column("version_id", Integer, ForeignKey("Versions.id"), primary_key=True),
-    Column("link_id", Integer, ForeignKey("Links.id"), primary_key=True)
+    Column(
+        "link_id",
+        Integer,
+        ForeignKey("Links.id", onupdate="CASCADE", ondelete="CASCADE"),
+        primary_key=True
+    )
 )
 
 # VERSION_OUTPUTS
