@@ -1,31 +1,27 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
-import os
-import shutil
-import tempfile
-import datetime
 import unittest
-import logging
-import stalker
+import pytest
 
+from stalker.testing import UnitTestDBBase
+
+import logging
 logger = logging.getLogger("stalker")
 logger.setLevel(logging.DEBUG)
 
@@ -37,27 +33,19 @@ class ConfigTester(unittest.TestCase):
     def setUp(self):
         """setup the test
         """
+        super(ConfigTester, self).setUp()
+        import tempfile
         # so we need a temp directory to be specified as our config folder
         self.temp_config_folder = tempfile.mkdtemp()
 
         # we should set the environment variable
+        import os
         os.environ["STALKER_PATH"] = self.temp_config_folder
 
         self.config_full_path = os.path.join(
             self.temp_config_folder, "config.py"
         )
 
-    def tearDown(self):
-        """clean up the test
-        """
-        from stalker.db import DBSession
-        DBSession.remove()
-
-        # and remove the temp directory
-        shutil.rmtree(self.temp_config_folder)
-        # restore defaults.timing_resolution
-        stalker.defaults.timing_resolution = datetime.timedelta(hours=1)
-
     def test_config_variable_updates_with_user_config(self):
         """testing if the database_file_name will be updated by the user
         config
@@ -77,7 +65,7 @@ class ConfigTester(unittest.TestCase):
         from stalker import config
         conf = config.Config()
 
-        self.assertEqual(test_value, conf.database_engine_settings)
+        assert test_value == conf.database_engine_settings
 
     def test_config_variable_does_create_new_variables_with_user_config(self):
         """testing if the config will be updated by the user config by adding
@@ -96,13 +84,13 @@ class ConfigTester(unittest.TestCase):
         from stalker import config
         conf = config.Config()
 
-        #self.assertRaises(KeyError, getattr, conf, "test_value")
-        self.assertEqual(conf.test_value, test_value)
+        assert conf.test_value == test_value
 
     def test_env_variable_with_vars_module_import_with_shortcuts(self):
         """testing if the module path has shortcuts like ~ and other env
         variables
         """
+        import os
         splits = os.path.split(self.temp_config_folder)
         var1 = splits[0]
         var2 = os.path.sep.join(splits[1:])
@@ -122,12 +110,13 @@ class ConfigTester(unittest.TestCase):
         from stalker import config
         conf = config.Config()
 
-        self.assertEqual(test_value, conf.database_url)
+        assert test_value == conf.database_url
 
     def test_env_variable_with_deep_vars_module_import_with_shortcuts(self):
         """testing if the module path has multiple shortcuts like ~ and other
         env variables
         """
+        import os
         splits = os.path.split(self.temp_config_folder)
         var1 = splits[0]
         var2 = os.path.sep.join(splits[1:])
@@ -149,14 +138,15 @@ class ConfigTester(unittest.TestCase):
         from stalker import config
         conf = config.Config()
 
-        self.assertEqual(test_value, conf.database_url)
+        assert test_value == conf.database_url
 
     def test_non_existing_path_in_environment_variable(self):
         """testing if the non existing path situation will be handled
         gracefully by warning the user
         """
-        os.environ["STALKER_PATH"] = "/tmp/non_existing_path"
+        import os
         from stalker import config
+        os.environ["STALKER_PATH"] = "/tmp/non_existing_path"
         config.Config()
 
     def test_syntax_error_in_settings_file(self):
@@ -165,7 +155,7 @@ class ConfigTester(unittest.TestCase):
         """
         # now create a config.py file and fill it with the desired values
         # like database_file_name = "test_value.db"
-        # but do a syntax error on purpose, like forgetting the last quato sign
+        # but do a syntax error on purpose, like forgetting the last quote sign
         test_value = ".test_value.db"
         config_file = open(self.config_full_path, "w")
         config_file.writelines(["#-*- coding: utf-8 -*-\n",
@@ -175,25 +165,69 @@ class ConfigTester(unittest.TestCase):
         # now import the config.py and see if it updates the
         # database_file_name variable
         from stalker import config
-        self.assertRaises(RuntimeError, config.Config)
+        with pytest.raises(RuntimeError) as cm:
+            config.Config()
+
+        assert str(cm.value) == \
+               'There is a syntax error in your configuration file: EOL ' \
+               'while scanning string literal (<string>, line 2)'
+
+    def test___getattr___is_working_properly(self):
+        """testing if config.Config.__getattr__() method is working properly
+        """
+        from stalker import config
+        c = config.Config()
+        assert c.admin_name == 'admin'
+
+    def test___getitem___is_working_properly(self):
+        """testing if config.Config.__getitem__() method is working properly
+        """
+        from stalker import config
+        c = config.Config()
+        assert c['admin_name'] == 'admin'
+
+    def test___setitem__is_working_properly(self):
+        """testing if config.Config.__setitem__() method is working properly
+        """
+        from stalker import config
+        c = config.Config()
+        test_value = 'administrator'
+        assert c['admin_name'] != test_value
+        c['admin_name'] = test_value
+        assert c['admin_name'] == test_value
+
+    def test___delitem__is_working_properly(self):
+        """testing if config.Config.__delitem__() method is working properly
+        """
+        from stalker import config
+        c = config.Config()
+        assert c['admin_name'] is not None
+        del c['admin_name']
+        assert 'admin_name' not in c
+
+    def test___contains___is_working_properly(self):
+        """testing if config.Config.__contains__() method is working properly
+        """
+        from stalker import config
+        c = config.Config()
+        assert 'admin_name' in c
+
+
+class ConfigTesterDB(UnitTestDBBase):
+    """tests config module with database
+    """
 
     def test_update_with_studio_is_working_properly(self):
         """testing if the default values are updated with the Studio instance
         if there is a database connection and there is a Studio in there
         """
         import datetime
-        from stalker import db, defaults
+        from stalker import defaults
         from stalker.db.session import DBSession
         from stalker.models.studio import Studio
 
-        db.setup()
-        db.init()
-
         # check the defaults are still using them self
-        self.assertEqual(
-            defaults.timing_resolution,
-            datetime.timedelta(hours=1)
-        )
+        assert defaults.timing_resolution == datetime.timedelta(hours=1)
 
         studio = Studio(
             name='Test Studio',
@@ -203,7 +237,4 @@ class ConfigTester(unittest.TestCase):
         DBSession.commit()
 
         # now check it again
-        self.assertEqual(
-            defaults.timing_resolution,
-            studio.timing_resolution
-        )
+        assert defaults.timing_resolution ==studio.timing_resolution
