@@ -1,26 +1,23 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import unittest
 from sqlalchemy import Column, Integer, ForeignKey
-from stalker.db.session import DBSession
 from stalker.models.project import Project
 from stalker.models.type import Type
 from stalker.models.status import Status, StatusList
@@ -60,6 +57,7 @@ class ProjectMixinTester(unittest.TestCase):
     def setUp(self):
         """setup the test
         """
+        super(ProjectMixinTester, self).setUp()
         self.test_stat1 = Status(name="On Hold", code="OH")
         self.test_stat2 = Status(name="Work In Progress", code="WIP")
         self.test_stat3 = Status(name="Approved", code="APP")
@@ -79,17 +77,18 @@ class ProjectMixinTester(unittest.TestCase):
         self.test_project_statuses = StatusList(
             name="Project Statuses",
             statuses=[self.test_stat2, self.test_stat3],
-            target_entity_type=Project
+            target_entity_type='Project'
         )
 
         self.test_project_type = Type(
             name='Test Project Type',
             code='testproj',
-            target_entity_type=Project,
+            target_entity_type='Project',
         )
 
         self.test_repository = Repository(
             name="Test Repo",
+            code='TR',
         )
 
         self.test_project = Project(
@@ -108,13 +107,8 @@ class ProjectMixinTester(unittest.TestCase):
 
         self.test_a_obj = DeclProjMixA(**self.kwargs)
 
-    def tearDown(self):
-        """clean up the test
-        """
-        DBSession.remove()
-
     def test_project_attribute_is_working_properly(self):
         """testing if the project attribute is working properly
         """
 
-        self.assertEqual(self.test_a_obj.project, self.test_project)
+        assert self.test_a_obj.project == self.test_project
