@@ -1,29 +1,25 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 import unittest
 
+import pytest
 from sqlalchemy import Column, Integer, ForeignKey
-from stalker.db.session import DBSession
-from stalker.models.mixins import StatusMixin
-from stalker.models.status import StatusList, Status
+from stalker import StatusMixin, StatusList, Status
 
 # create a new mixed in SimpleEntity
 from stalker.models.entity import SimpleEntity
@@ -58,6 +54,8 @@ class StatusMixinTester(unittest.TestCase):
     def setUp(self):
         """setup the test
         """
+        super(StatusMixinTester, self).setUp()
+
         self.test_stat1 = Status(name="On Hold", code="OH")
         self.test_stat2 = Status(name="Work In Progress", code="WIP")
         self.test_stat3 = Status(name="Approved", code="APP")
@@ -79,24 +77,32 @@ class StatusMixinTester(unittest.TestCase):
             "status_list": self.test_a_statusList
         }
 
-    def tearDown(self):
-        """clean up the test
-        """
-        DBSession.remove()
-
     def test_status_list_argument_not_set(self):
         """testing if a TypeError will be raised when the status_list argument
         is not set
         """
         self.kwargs.pop("status_list")
-        self.assertRaises(TypeError, DeclStatMixA, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            DeclStatMixA(**self.kwargs)
+
+        assert str(cm.value) == \
+            "DeclStatMixA instances can not be initialized without a " \
+            "stalker.models.status.StatusList instance, please pass a " \
+            "suitable StatusList " \
+            "(StatusList.target_entity_type=DeclStatMixA) with the " \
+            "'status_list' argument"
 
     def test_status_list_argument_is_not_correct(self):
         """testing if a TypeError will be raised when the given StatusList
         instance with the status_list argument is not suitable for this class
         """
         self.kwargs["status_list"] = self.test_b_statusList
-        self.assertRaises(TypeError, DeclStatMixA, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            DeclStatMixA(**self.kwargs)
+
+        assert str(cm.value) == \
+            "The given StatusLists' target_entity_type is DeclStatMixB, " \
+            "whereas the entity_type of this object is DeclStatMixA"
 
     def test_status_list_working_properly(self):
         """testing if the status_list attribute is working properly
@@ -106,6 +112,6 @@ class StatusMixinTester(unittest.TestCase):
             status_list=self.test_a_statusList
         )
 
-        self.assertTrue(self.test_stat1 in new_a_ins.status_list)
-        self.assertFalse(self.test_stat2 in new_a_ins.status_list)
-        self.assertTrue(self.test_stat3 in new_a_ins.status_list)
+        assert self.test_stat1 in new_a_ins.status_list
+        assert self.test_stat2 not in new_a_ins.status_list
+        assert self.test_stat3 in new_a_ins.status_list
