@@ -1,30 +1,28 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import unittest
 
+import pytest
 from sqlalchemy import Column, Integer, ForeignKey
 from stalker.models.entity import Entity
 from stalker.models.link import Link
 from stalker.models.mixins import ReferenceMixin
-from stalker.db.session import DBSession
 from stalker.models.type import Type
 from stalker.models.entity import SimpleEntity
 
@@ -46,6 +44,7 @@ class ReferenceMixinTester(unittest.TestCase):
     def setUp(self):
         """setup the test
         """
+        super(ReferenceMixinTester, self).setUp()
         # link type
         self.test_link_type = Type(
             name='Test Link Type',
@@ -99,77 +98,58 @@ class ReferenceMixinTester(unittest.TestCase):
 
         self.test_foo_obj = RefMixFooClass(name="Ref Mixin Test")
 
-    def tearDown(self):
-        """clean up the test
-        """
-        DBSession.remove()
-
     def test_references_attribute_accepting_empty_list(self):
         """testing if references attribute accepting empty lists
         """
         self.test_foo_obj.references = []
 
-    def test_references_attribute_only_accepts_listlike_objects(self):
+    def test_references_attribute_only_accepts_list_like_objects(self):
         """testing if references attribute accepts only list-like objects,
         (objects with __setitem__, __getitem__ methods
         """
-        test_values = [1, 1.2, "a string"]
+        with pytest.raises(TypeError) as cm:
+            self.test_foo_obj.references = "a string"
 
-        for test_value in test_values:
-            self.assertRaises(
-                TypeError,
-                setattr,
-                self.test_foo_obj,
-                "references",
-                test_value
-            )
+        assert str(cm.value) == \
+            'Incompatible collection type: str is not list-like'
 
     def test_references_attribute_accepting_only_lists_of_Link_instances(self):
         """testing if references attribute accepting only lists with Link
         instances and derivatives
         """
-        test_value = [1, 2.2, ["a reference as list"], "some references"]
+        test_value = [1, 2.2, "some references"]
 
-        self.assertRaises(
-            TypeError,
-            setattr,
-            self.test_foo_obj,
-            "references",
-            test_value
-        )
+        with pytest.raises(TypeError) as cm:
+            self.test_foo_obj.references = test_value
 
-        # and test if it is accepting a proper list
-        test_value = [self.test_link1, self.test_link2, self.test_link3]
-        self.test_foo_obj.references = test_value
-        self.assertEqual(
-            sorted(test_value, key=lambda x: x.name),
-            sorted(self.test_foo_obj.references, key=lambda x: x.name)
-        )
+        assert str(cm.value) == \
+            'All the elements in the RefMixFooClass.references should be ' \
+            'stalker.models.link.Link instances not int'
 
     def test_references_attribute_elements_accepts_Links_only(self):
         """testing if a TypeError will be raised when trying to assign
         something other than an instance of Link or its derived classes to
         the references list
         """
-        self.assertRaises(
-            TypeError,
-            setattr,
-            self.test_foo_obj, 'references',
-            [self.test_entity1, self.test_entity2]
-        )
+        with pytest.raises(TypeError) as cm:
+            self.test_foo_obj.references = \
+                [self.test_entity1, self.test_entity2]
 
-    def test_references_attribute_working_properly(self):
+        assert str(cm.value) == \
+            'All the elements in the RefMixFooClass.references should be ' \
+            'stalker.models.link.Link instances not Entity'
+
+    def test_references_attribute_is_working_properly(self):
         """testing if references attribute working properly
         """
         self.test_foo_obj.references = self.test_links
-        self.assertEqual(self.test_foo_obj.references, self.test_links)
+        assert self.test_foo_obj.references == self.test_links
 
         test_value = [self.test_link1, self.test_link2]
         self.test_foo_obj.references = test_value
-        self.assertEqual(
-            sorted(self.test_foo_obj.references, key=lambda x: x.name),
+        assert \
+            sorted(self.test_foo_obj.references, key=lambda x: x.name) == \
             sorted(test_value, key=lambda x: x.name)
-        )
 
     def test_references_application_test(self):
         """testing an example of ReferenceMixin usage
@@ -198,4 +178,4 @@ class ReferenceMixinTester(unittest.TestCase):
 
         my_ge.references = test_value
 
-        self.assertEqual(my_ge.references, test_value)
+        assert my_ge.references == test_value
