@@ -1,26 +1,26 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import unittest
-import datetime
-from stalker import db, Department, Entity, User
+import pytest
+
+from stalker.testing import UnitTestDBBase
+from stalker import Department
 
 
 class DepartmentTester(unittest.TestCase):
@@ -30,10 +30,15 @@ class DepartmentTester(unittest.TestCase):
     def setUp(self):
         """lets setup the tests
         """
-        db.setup()  # uses in memory sqlite db
-        db.init()
+        super(DepartmentTester, self).setUp()
 
-        self.test_admin = User.query.filter_by(login="admin").first()
+        from stalker import User
+        self.test_admin = User(
+            name='admin',
+            login='admin',
+            email='admin@admins.com',
+            password='12345',
+        )
 
         # create a couple of test users
         self.test_user1 = User(
@@ -42,7 +47,6 @@ class DepartmentTester(unittest.TestCase):
             email="user1@test.com",
             password="123456",
         )
-        db.DBSession.add(self.test_user1)
 
         self.test_user2 = User(
             name="User2",
@@ -50,7 +54,6 @@ class DepartmentTester(unittest.TestCase):
             email="user2@test.com",
             password="123456",
         )
-        db.DBSession.add(self.test_user2)
 
         self.test_user3 = User(
             name="User3",
@@ -58,7 +61,6 @@ class DepartmentTester(unittest.TestCase):
             email="user3@test.com",
             password="123456",
         )
-        db.DBSession.add(self.test_user3)
 
         self.test_user4 = User(
             name="User4",
@@ -66,7 +68,6 @@ class DepartmentTester(unittest.TestCase):
             email="user4@test.com",
             password="123456",
         )
-        db.DBSession.add(self.test_user4)
 
         self.test_user5 = User(
             name="User5",
@@ -74,7 +75,6 @@ class DepartmentTester(unittest.TestCase):
             email="user5@test.com",
             password="123456",
         )
-        db.DBSession.add(self.test_user5)
 
         self.users_list = [
             self.test_user1,
@@ -83,7 +83,9 @@ class DepartmentTester(unittest.TestCase):
             self.test_user4
         ]
 
-        self.date_created = self.date_updated = datetime.datetime.now()
+        import datetime
+        import pytz
+        self.date_created = self.date_updated = datetime.datetime.now(pytz.utc)
 
         self.kwargs = {
             "name": "Test Department",
@@ -97,14 +99,20 @@ class DepartmentTester(unittest.TestCase):
 
         # create a default department object
         self.test_department = Department(**self.kwargs)
-        db.DBSession.add(self.test_department)
-        db.DBSession.commit()
 
     def test___auto_name__class_attribute_is_set_to_false(self):
         """testing if the __auto_name__ class attribute is set to False for
         Department class
         """
-        self.assertFalse(Department.__auto_name__)
+        assert Department.__auto_name__ is False
+
+    def test___hash___value_is_correctly_calculated(self):
+        """testing if the __hash__ value is correctly calculated
+        """
+        assert self.test_department.__hash__() == \
+            hash(self.test_department.id) + \
+            2 * hash(self.test_department.name) + \
+            3 * hash(self.test_department.entity_type)
 
     def test_users_argument_accepts_an_empty_list(self):
         """testing if users argument accepts an empty list
@@ -112,7 +120,7 @@ class DepartmentTester(unittest.TestCase):
         # this should work without raising any error
         self.kwargs["users"] = []
         new_dep = Department(**self.kwargs)
-        self.assertTrue(isinstance(new_dep, Department))
+        assert isinstance(new_dep, Department)
 
     def test_users_attribute_accepts_an_empty_list(self):
         """testing if users attribute accepts an empty list
@@ -126,61 +134,71 @@ class DepartmentTester(unittest.TestCase):
         test_value = [1, 2.3, [], {}]
         self.kwargs["users"] = test_value
         # this should raise a TypeError
-        self.assertRaises(
-            TypeError,
-            Department,
-            **self.kwargs
-        )
+        with pytest.raises(TypeError) as cm:
+            Department(**self.kwargs)
+
+        assert str(cm.value) == \
+            'DepartmentUser.user should be a stalker.models.auth.User ' \
+            'instance, not int'
 
     def test_users_attribute_accepts_only_a_list_of_user_objects(self):
         """testing if users attribute accepts only a list of user objects
         """
         test_value = [1, 2.3, [], {}]
         # this should raise a TypeError
-        self.assertRaises(
-            TypeError,
-            setattr,
-            self.test_department,
-            "users",
-            test_value
-        )
+        with pytest.raises(TypeError) as cm:
+            self.test_department.users = test_value
 
-    def test_users_attribute_elements_accepts_User_only(self):
+        assert str(cm.value) == \
+            'DepartmentUser.user should be a stalker.models.auth.User ' \
+            'instance, not int'
+
+    def test_users_attribute_elements_accepts_User_only_1(self):
         """testing if a TypeError will be raised when trying to assign
         something other than a User object to the users list
         """
         # append
-        self.assertRaises(
-            TypeError,
-            self.test_department.users.append,
-            0
-        )
+        with pytest.raises(TypeError) as cm:
+            self.test_department.users.append(0)
 
+        assert str(cm.value) == \
+            'DepartmentUser.user should be a stalker.models.auth.User ' \
+            'instance, not int'
+
+    def test_users_attribute_elements_accepts_User_only_2(self):
+        """testing if a TypeError will be raised when trying to assign
+        something other than a User object to the users list
+        """
         # __setitem__
-        self.assertRaises(
-            TypeError,
-            self.test_department.users.__setitem__,
-            0,
-            0
-        )
+        with pytest.raises(TypeError) as cm:
+            self.test_department.users[0] = 0
+
+        assert str(cm.value) == \
+            'DepartmentUser.user should be a stalker.models.auth.User ' \
+            'instance, not int'
 
     def test_users_argument_is_not_iterable(self):
         """testing if a TypeError will be raised when the given users
         argument is not an instance of list
         """
-        test_values = [1, 1.2, "a user"]
-        for test_value in test_values:
-            self.kwargs["users"] = test_value
-            self.assertRaises(TypeError, Department, **self.kwargs)
+        self.kwargs["users"] = "a user"
+        with pytest.raises(TypeError) as cm:
+            Department(**self.kwargs)
+
+        assert str(cm.value) == \
+            'DepartmentUser.user should be a stalker.models.auth.User ' \
+            'instance, not str'
 
     def test_users_attribute_is_not_iterable(self):
         """testing if a TypeError will be raised when the users attribute
         is tried to be set to a non-iterable value
         """
-        test_values = [1, 1.2, "a user"]
-        for test_value in test_values:
-            self.assertRaises(TypeError, setattr, self.test_department,
-                              "users", test_value)
+        with pytest.raises(TypeError) as cm:
+            self.test_department.users = "a user"
+
+        assert str(cm.value) == \
+            'DepartmentUser.user should be a stalker.models.auth.User ' \
+            'instance, not str'
 
     def test_users_attribute_defaults_to_empty_list(self):
         """testing if the users attribute defaults to an empty list if the
@@ -188,14 +206,133 @@ class DepartmentTester(unittest.TestCase):
         """
         self.kwargs.pop("users")
         new_department = Department(**self.kwargs)
-        self.assertEqual(new_department.users, [])
+        assert new_department.users == []
 
     def test_users_attribute_set_to_None(self):
         """testing if a TypeError will be raised when the users attribute is
         set to None
         """
-        self.assertRaises(TypeError, setattr, self.test_department, "users",
-                          None)
+        with pytest.raises(TypeError) as cm:
+            self.test_department.users = None
+
+        assert str(cm.value) == "'NoneType' object is not iterable"
+
+    def test_equality(self):
+        """testing equality of two Department objects
+        """
+        from stalker import Entity
+        dep1 = Department(**self.kwargs)
+        dep2 = Department(**self.kwargs)
+
+        entity_kwargs = self.kwargs.copy()
+        entity_kwargs.pop("users")
+        entity1 = Entity(**entity_kwargs)
+
+        self.kwargs["name"] = "Animation"
+        dep3 = Department(**self.kwargs)
+
+        assert dep1 == dep2
+        assert not dep1 == dep3
+        assert not dep1 == entity1
+
+    def test_inequality(self):
+        """testing inequality of two Department objects
+        """
+        from stalker import Entity
+        dep1 = Department(**self.kwargs)
+        dep2 = Department(**self.kwargs)
+
+        entity_kwargs = self.kwargs.copy()
+        entity_kwargs.pop("users")
+        entity1 = Entity(**entity_kwargs)
+
+        self.kwargs["name"] = "Animation"
+        dep3 = Department(**self.kwargs)
+
+        assert not dep1 != dep2
+        assert dep1 != dep3
+        assert dep1 != entity1
+
+
+class DepartmentDBTester(UnitTestDBBase):
+    """tests that needs a Database
+    """
+
+    def setUp(self):
+        """lets setup the tests
+        """
+        super(DepartmentDBTester, self).setUp()
+
+        from stalker import User
+        self.test_admin = User.query.filter_by(login="admin").first()
+
+        # create a couple of test users
+        self.test_user1 = User(
+            name="User1",
+            login="user1",
+            email="user1@test.com",
+            password="123456",
+        )
+        from stalker.db.session import DBSession
+        DBSession.add(self.test_user1)
+
+        self.test_user2 = User(
+            name="User2",
+            login="user2",
+            email="user2@test.com",
+            password="123456",
+        )
+        DBSession.add(self.test_user2)
+
+        self.test_user3 = User(
+            name="User3",
+            login="user3",
+            email="user3@test.com",
+            password="123456",
+        )
+        DBSession.add(self.test_user3)
+
+        self.test_user4 = User(
+            name="User4",
+            login="user4",
+            email="user4@test.com",
+            password="123456",
+        )
+        DBSession.add(self.test_user4)
+
+        self.test_user5 = User(
+            name="User5",
+            login="user5",
+            email="user5@test.com",
+            password="123456",
+        )
+        DBSession.add(self.test_user5)
+
+        self.users_list = [
+            self.test_user1,
+            self.test_user2,
+            self.test_user3,
+            self.test_user4
+        ]
+
+        import datetime
+        import pytz
+        self.date_created = self.date_updated = datetime.datetime.now(pytz.utc)
+
+        self.kwargs = {
+            "name": "Test Department",
+            "description": "This is a department for testing purposes",
+            "created_by": self.test_admin,
+            "updated_by": self.test_admin,
+            "date_created": self.date_created,
+            "date_updated": self.date_updated,
+            "users": self.users_list
+        }
+
+        # create a default department object
+        self.test_department = Department(**self.kwargs)
+        DBSession.add(self.test_department)
+        DBSession.commit()
 
     def test_user_role_attribute(self):
         """testing the automatic generation of the DepartmentUser class
@@ -203,8 +340,9 @@ class DepartmentTester(unittest.TestCase):
         # assign a user to a department and search for a DepartmentUser
         # representing that relation
         from stalker import DepartmentUser
-        db.DBSession.commit()
-        with db.DBSession.no_autoflush:
+        from stalker.db.session import DBSession
+        DBSession.commit()
+        with DBSession.no_autoflush:
             self.test_department.users.append(self.test_user5)
 
         dus = DepartmentUser.query\
@@ -212,60 +350,23 @@ class DepartmentTester(unittest.TestCase):
             .filter(DepartmentUser.department == self.test_department)\
             .all()
 
-        self.assertTrue(len(dus) > 0)
+        assert len(dus) > 0
         du = dus[0]
-        self.assertTrue(isinstance(du, DepartmentUser))
-        self.assertEqual(du.department, self.test_department)
-        self.assertEqual(du.user, self.test_user5)
-        self.assertEqual(du.role, None)
-
-    def test_equality(self):
-        """testing equality of two Department objects
-        """
-        dep1 = Department(**self.kwargs)
-        dep2 = Department(**self.kwargs)
-
-        entity_kwargs = self.kwargs.copy()
-        entity_kwargs.pop("users")
-        entity1 = Entity(**entity_kwargs)
-
-        self.kwargs["name"] = "Animation"
-        dep3 = Department(**self.kwargs)
-
-        self.assertTrue(dep1 == dep2)
-        self.assertFalse(dep1 == dep3)
-        self.assertFalse(dep1 == entity1)
-
-    def test_inequality(self):
-        """testing inequality of two Department objects
-        """
-        dep1 = Department(**self.kwargs)
-        dep2 = Department(**self.kwargs)
-
-        entity_kwargs = self.kwargs.copy()
-        entity_kwargs.pop("users")
-        entity1 = Entity(**entity_kwargs)
-
-        self.kwargs["name"] = "Animation"
-        dep3 = Department(**self.kwargs)
-
-        self.assertFalse(dep1 != dep2)
-        self.assertTrue(dep1 != dep3)
-        self.assertTrue(dep1 != entity1)
+        assert isinstance(du, DepartmentUser)
+        assert du.department == self.test_department
+        assert du.user == self.test_user5
+        assert du.role == None
 
     def test_tjp_id_is_working_properly(self):
         """testing if the tjp_is working properly
         """
-        self.assertEqual(self.test_department.tjp_id, 'Department_35')
+        assert self.test_department.tjp_id == 'Department_36'
 
     def test_to_tjp_is_working_properly(self):
         """testing if the to_tjp property is working properly
         """
         expected_tjp = """
-resource Department_35 "Department_35" {
-    resource User_30 "User_30" {
-    efficiency 1.0
-}
+resource Department_36 "Department_36" {
     resource User_31 "User_31" {
     efficiency 1.0
 }
@@ -274,10 +375,13 @@ resource Department_35 "Department_35" {
 }
     resource User_33 "User_33" {
     efficiency 1.0
+}
+    resource User_34 "User_34" {
+    efficiency 1.0
 }
 }"""
         # print expected_tjp
         # print "-----------------"
         # print self.test_department.to_tjp
         # self.maxDiff = None
-        self.assertEqual(self.test_department.to_tjp, expected_tjp)
+        assert self.test_department.to_tjp == expected_tjp
