@@ -1,25 +1,25 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
+
 
 import unittest
-from stalker.db import DBSession
+import pytest
+
 from stalker.models.auth import User, Group
 
 
@@ -27,21 +27,11 @@ class GroupTester(unittest.TestCase):
     """tests the stalker.models.auth.Group class
     """
 
-    @classmethod
-    def setUpClass(cls):
-        """sets the test in class level
-        """
-        DBSession.remove()
-
-    @classmethod
-    def tearDownClass(cls):
-        """clear the test in class level
-        """
-        DBSession.remove()
-
     def setUp(self):
         """set up the test in method level
         """
+        super(GroupTester, self).setUp()
+
         # create a couple of Users
         self.test_user1 = User(
             name='User1',
@@ -80,7 +70,7 @@ class GroupTester(unittest.TestCase):
         """testing if the __auto_name__ class attribute is set to False for
         Group class
         """
-        self.assertFalse(Group.__auto_name__)
+        assert Group.__auto_name__ is False
 
     def test_users_argument_is_skipped(self):
         """testing if the users argument is skipped the users attribute will be
@@ -88,26 +78,30 @@ class GroupTester(unittest.TestCase):
         """
         self.kwargs.pop('users')
         new_group = Group(**self.kwargs)
-        self.assertEqual(new_group.users, [])
+        assert new_group.users == []
 
     def test_users_argument_is_not_a_list_of_User_instances(self):
         """testing if a TypeError will be raised when the users argument is not
         a list of User instances
         """
         self.kwargs['users'] = [12, 'not a user']
-        self.assertRaises(TypeError, Group, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            Group(**self.kwargs)
+
+        assert str(cm.value) == \
+            'Group.users attribute must all be stalker.models.auth.User ' \
+            'instances not int'
 
     def test_users_attribute_is_not_a_list_of_User_instances(self):
         """testing if a TypeError will be raised when the users attribute is
         not a list of User instances
         """
-        self.assertRaises(
-            TypeError,
-            setattr,
-            self.test_group,
-            'users',
-            [12, 'not a user']
-        )
+        with pytest.raises(TypeError) as cm:
+            self.test_group.users = [12, 'not a user']
+
+        assert str(cm.value) == \
+            'Group.users attribute must all be stalker.models.auth.User ' \
+            'instances not int'
 
     def test_users_argument_updates_the_groups_attribute_in_the_given_User_instances(self):
         """testing if the Users given with the users argument will have the
@@ -117,7 +111,7 @@ class GroupTester(unittest.TestCase):
         new_group = Group(**self.kwargs)
 
         for user in self.kwargs['users']:
-            self.assertTrue(new_group in user.groups)
+            assert new_group in user.groups
 
     def test_users_attribute_updates_the_groups_attribute_in_the_given_User_instances(self):
         """testing if the Users given with the users attribute will have the
@@ -127,4 +121,21 @@ class GroupTester(unittest.TestCase):
         new_group = Group(**self.kwargs)
         new_group.users = test_users
         for user in test_users:
-            self.assertTrue(new_group in user.groups)
+            assert new_group in user.groups
+
+    def test_permissions_argument_is_working_properly(self):
+        """testing if permissions can be added to the Group on __init__()
+        """
+        # create a couple of permissions
+        from stalker import Permission
+        perm1 = Permission('Allow', 'Create', 'User')
+        perm2 = Permission('Allow', 'Read', 'User')
+        perm3 = Permission('Deny',  'Delete', 'User')
+
+        new_group = Group(
+            name='Test Group',
+            users=[self.test_user1, self.test_user2],
+            permissions=[perm1, perm2, perm3]
+        )
+
+        assert new_group.permissions == [perm1, perm2, perm3]
