@@ -1,62 +1,43 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-import json
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
-import os
-import pickle
-import shutil
-import tempfile
-import datetime
 import unittest
-
-from stalker import defaults, db, User
-from stalker.db import DBSession
-from stalker.models.auth import LocalSession
+from stalker.testing import UnitTestDBBase
 
 
 class LocalSessionTester(unittest.TestCase):
     """tests LocalSession class
     """
 
-    @classmethod
-    def setUpClass(cls):
-        """setting up the test in class level
-        """
-        DBSession.remove()
-
-    @classmethod
-    def tearDownClass(cls):
-        """clean up the test in class level
-        """
-        DBSession.remove()
-
     def setUp(self):
         """setup the test
         """
-        DBSession.remove()
-        db.setup()
+        super(LocalSessionTester, self).setUp()
+        import tempfile
+        from stalker import defaults
         defaults.local_storage_path = tempfile.mktemp()
 
     def tearDown(self):
         """cleans the test environment
         """
+        super(LocalSessionTester, self).tearDown()
+        import shutil
+        from stalker import defaults
         shutil.rmtree(
             defaults.local_storage_path,
             True
@@ -65,16 +46,17 @@ class LocalSessionTester(unittest.TestCase):
     def test_save_serializes_the_class_itself(self):
         """testing if the save function serializes the class to the filesystem
         """
+        from stalker import LocalSession
         new_local_session = LocalSession()
         new_local_session.save()
 
         # check if a file is created in the users local storage
-        self.assertTrue(
-            os.path.exists(
-                os.path.join(
-                    defaults.local_storage_path,
-                    defaults.local_session_data_file_name
-                )
+        import os
+        from stalker import defaults
+        assert os.path.exists(
+            os.path.join(
+                defaults.local_storage_path,
+                defaults.local_session_data_file_name
             )
         )
 
@@ -86,6 +68,7 @@ class LocalSessionTester(unittest.TestCase):
         logged_in_user_id = -10
 
         # create a local_session
+        from stalker import LocalSession
         local_session = LocalSession()
 
         # store some data
@@ -96,15 +79,14 @@ class LocalSessionTester(unittest.TestCase):
         local_session2 = LocalSession()
 
         # now try to get the data back
-        self.assertEqual(
-            local_session2.logged_in_user_id,
-            logged_in_user_id
-        )
+        assert local_session2.logged_in_user_id == logged_in_user_id
 
-    def test_logged_in_user_returns_the_stored_User_instance_from_last_time(self):
-        """testing if logged_in_user returns the logged in user
+    def test_delete_will_delete_the_session_cache(self):
+        """testing if the LocalSession.delete() will delete the current cache
+        file
         """
         # create a new user
+        from stalker import User
         new_user = User(
             name='Test User',
             login='test_user',
@@ -113,36 +95,55 @@ class LocalSessionTester(unittest.TestCase):
         )
 
         # save it to the Database
-        DBSession.add(new_user)
-        DBSession.commit()
-
-        self.assertTrue(new_user.id is not None)
+        new_user.id = 1023
+        assert new_user.id is not None
 
         # save it to the local storage
+        from stalker import LocalSession
         local_session = LocalSession()
         local_session.store_user(new_user)
 
         # save the session
         local_session.save()
 
-        # now get it back with a new local_session
-        local_session2 = LocalSession()
-
-        self.assertEqual(
-            local_session2.logged_in_user_id,
-            new_user.id
+        # check if the file is created
+        # check if a file is created in the users local storage
+        import os
+        from stalker import defaults
+        assert os.path.exists(
+            os.path.join(
+                defaults.local_storage_path,
+                defaults.local_session_data_file_name
+            )
         )
 
-        self.assertEqual(
-            local_session2.logged_in_user,
-            new_user
+        # now delete the session by calling delete()
+        local_session.delete()
+
+        # check if the file is gone
+        # check if a file is created in the users local storage
+        assert not os.path.exists(
+            os.path.join(
+                defaults.local_storage_path,
+                defaults.local_session_data_file_name
+            )
         )
 
+        # delete a second time
+        # this should not raise an OSError
+        local_session.delete()
+
+
+class LocalSessionDBTester(UnitTestDBBase):
+    """tests that needs a database
+    """
+
     def test_LocalSession_will_not_use_the_stored_data_if_it_is_invalid(self):
         """testing if the LocalSession will not use the stored session if it is
         not valid anymore
         """
         # create a new user
+        from stalker import User, LocalSession
         new_user = User(
             name='Test User',
             login='test_user',
@@ -151,10 +152,10 @@ class LocalSessionTester(unittest.TestCase):
         )
 
         # save it to the Database
+        from stalker.db.session import DBSession
         DBSession.add(new_user)
         DBSession.commit()
-
-        self.assertTrue(new_user.id is not None)
+        assert new_user.id is not None
 
         # save it to the local storage
         local_session = LocalSession()
@@ -164,10 +165,13 @@ class LocalSessionTester(unittest.TestCase):
         local_session.save()
 
         # set the valid time to an early date
+        import pytz
+        import datetime
         local_session.valid_to = \
-            datetime.datetime.now() - datetime.timedelta(10)
+            datetime.datetime.now(pytz.utc) - datetime.timedelta(10)
 
         # pickle the data
+        import json
         data = json.dumps(
             {
                 'valid_to': local_session.valid_to,
@@ -175,23 +179,19 @@ class LocalSessionTester(unittest.TestCase):
             },
             default=local_session.default_json_serializer
         )
-        print('data: %s' % data)
         local_session._write_data(data)
 
         # now get it back with a new local_session
         local_session2 = LocalSession()
 
-        self.assertEqual(
-            local_session2.logged_in_user_id, None
-        )
+        assert local_session2.logged_in_user_id is None
+        assert local_session2.logged_in_user is None
 
-        self.assertTrue(local_session2.logged_in_user is None)
-
-    def test_delete_will_delete_the_session_cache(self):
-        """testing if the LocalSession.delete() will delete the current cache
-        file
+    def test_logged_in_user_returns_the_stored_User_instance_from_last_time(self):
+        """testing if logged_in_user returns the logged in user
         """
         # create a new user
+        from stalker import User
         new_user = User(
             name='Test User',
             login='test_user',
@@ -200,43 +200,21 @@ class LocalSessionTester(unittest.TestCase):
         )
 
         # save it to the Database
+        from stalker.db.session import DBSession
         DBSession.add(new_user)
         DBSession.commit()
-
-        self.assertTrue(new_user.id is not None)
+        assert new_user.id is not None
 
         # save it to the local storage
+        from stalker import LocalSession
         local_session = LocalSession()
         local_session.store_user(new_user)
 
         # save the session
         local_session.save()
 
-        # check if the file is created
-        # check if a file is created in the users local storage
-        self.assertTrue(
-            os.path.exists(
-                os.path.join(
-                    defaults.local_storage_path,
-                    defaults.local_session_data_file_name
-                )
-            )
-        )
+        # now get it back with a new local_session
+        local_session2 = LocalSession()
 
-        # now delete the session by calling delete()
-        local_session.delete()
-
-        # check if the file is gone
-        # check if a file is created in the users local storage
-        self.assertFalse(
-            os.path.exists(
-                os.path.join(
-                    defaults.local_storage_path,
-                    defaults.local_session_data_file_name
-                )
-            )
-        )
-
-        # delete a second time
-        # this should not raise an OSError
-        local_session.delete()
+        assert local_session2.logged_in_user_id == new_user.id
+        assert local_session2.logged_in_user == new_user
