@@ -1,34 +1,34 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
-import unittest2
+import unittest
+import pytest
 from stalker import Note
 
 
-class NoteTester(unittest2.TestCase):
+class NoteTester(unittest.TestCase):
     """tests  the Note class
     """
 
     def setUp(self):
         """setup the test
         """
+        super(NoteTester, self).setUp()
         self.kwargs = {
             "name": "Note to something",
             "description": "this is a simple note",
@@ -42,7 +42,7 @@ class NoteTester(unittest2.TestCase):
         """testing if the __auto_name__ class attribute is set to True for
         Note class
         """
-        self.assertTrue(Note.__auto_name__)
+        assert Note.__auto_name__ is True
 
     def test_content_argument_is_missing(self):
         """testing if nothing is going to happen when no content argument is
@@ -50,7 +50,7 @@ class NoteTester(unittest2.TestCase):
         """
         self.kwargs.pop("content")
         new_note = Note(**self.kwargs)
-        self.assertIsInstance(new_note, Note)
+        assert isinstance(new_note, Note)
 
     def test_content_argument_is_set_to_None(self):
         """testing if nothing is going to happen when content argument is given
@@ -58,7 +58,7 @@ class NoteTester(unittest2.TestCase):
         """
         self.kwargs["content"] = None
         new_note = Note(**self.kwargs)
-        self.assertIsInstance(new_note, Note)
+        assert isinstance(new_note, Note)
 
     def test_content_attribute_is_set_to_None(self):
         """testing if nothing is going to happen when content attribute is set
@@ -72,7 +72,7 @@ class NoteTester(unittest2.TestCase):
         as an empty string
         """
         self.kwargs["content"] = ""
-        new_note = Note(**self.kwargs)
+        Note(**self.kwargs)
 
     def test_content_attribute_is_set_to_empty_string(self):
         """testing if nothing is going to happen when content argument is set
@@ -82,39 +82,37 @@ class NoteTester(unittest2.TestCase):
         self.test_note.content = ""
 
     def test_content_argument_is_set_to_something_other_than_a_string(self):
-        """testing if a TypeEror will be raised when trying to set the content
-        argument to something other than a string or unicode
+        """testing if a TypeError will be raised when trying to set the content
+        argument to something other than a string
         """
-        test_values = [1, 1.24, ["content"], {"a": "Content"}]
+        test_value = 1.24
 
-        for test_value in test_values:
-            self.kwargs["content"] = test_value
-            self.assertRaises(TypeError, Note, **self.kwargs)
+        self.kwargs["content"] = test_value
+        with pytest.raises(TypeError) as cm:
+            Note(**self.kwargs)
+
+        assert str(cm.value) == \
+            'Note.description should be a string, not float'
 
     def test_content_attribute_is_set_to_something_other_than_a_string(self):
         """testing if a TypeError will be raised when trying to set the
-        content attribute to something other than a string or unicode
+        content attribute to something other than a string
         """
-        test_values = [1, 1.24, ["content"], {"a": "Content"}]
+        test_value = 1
 
-        for test_value in test_values:
-            self.assertRaises(
-                TypeError,
-                setattr,
-                self.test_note,
-                "content",
-                test_value
-            )
+        with pytest.raises(TypeError) as cm:
+            self.test_note.content = test_value
+
+        assert str(cm.value) == \
+            'Note.description should be a string, not int'
 
     def test_content_attribute_is_working_properly(self):
         """testing if the content attribute is working properly
         """
         new_content = "This is my new content for the note, and I expect it to\
         work fine when I assign it to a Note object"
-
         self.test_note.content = new_content
-
-        self.assertEqual(self.test_note.content, new_content)
+        assert self.test_note.content == new_content
 
     def test_equality_operator(self):
         """testing equality operator
@@ -125,8 +123,8 @@ class NoteTester(unittest2.TestCase):
         self.kwargs["content"] = "this is a different content"
         note3 = Note(**self.kwargs)
 
-        self.assertTrue(note1 == note2)
-        self.assertFalse(note1 == note3)
+        assert note1 == note2
+        assert not note1 == note3
 
     def test_inequality_operator(self):
         """testing inequality operator
@@ -137,10 +135,10 @@ class NoteTester(unittest2.TestCase):
         self.kwargs["content"] = "this is a different content"
         note3 = Note(**self.kwargs)
 
-        self.assertFalse(note1 != note2)
-        self.assertTrue(note1 != note3)
+        assert not note1 != note2
+        assert note1 != note3
 
     def test_plural_class_name(self):
         """testing the plural name of Note class
         """
-        self.assertTrue(self.test_note.plural_class_name, "Notes")
+        assert self.test_note.plural_class_name == "Notes"
