@@ -1,26 +1,24 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import unittest
-
-from stalker import Studio, SchedulerBase
+import pytest
+from stalker import SchedulerBase
 
 
 class SchedulerBaseTester(unittest.TestCase):
@@ -30,6 +28,9 @@ class SchedulerBaseTester(unittest.TestCase):
     def setUp(self):
         """set up the test
         """
+        super(SchedulerBaseTester, self).setUp()
+
+        from stalker import Studio
         self.test_studio = Studio(name='Test Studio')
         self.kwargs = {
             'studio': self.test_studio
@@ -42,7 +43,7 @@ class SchedulerBaseTester(unittest.TestCase):
         """
         self.kwargs.pop('studio')
         new_scheduler_base = SchedulerBase(**self.kwargs)
-        self.assertIsNone(new_scheduler_base.studio)
+        assert new_scheduler_base.studio is None
 
     def test_studio_argument_is_None(self):
         """testing if the studio attribute will be None if the studio argument
@@ -50,38 +51,56 @@ class SchedulerBaseTester(unittest.TestCase):
         """
         self.kwargs['studio'] = None
         new_scheduler_base = SchedulerBase(**self.kwargs)
-        self.assertIsNone(new_scheduler_base.studio)
+        assert new_scheduler_base.studio is None
 
     def test_studio_attribute_is_None(self):
         """testing if the studio argument can be set to None
         """
         self.test_scheduler_base.studio = None
-        self.assertIsNone(self.test_scheduler_base.studio)
+        assert self.test_scheduler_base.studio is None
 
     def test_studio_argument_is_not_a_Studio_instance(self):
         """testing if a TypeError will be raised when the studio argument is
         not stalker.models.studio.Studio instance
         """
         self.kwargs['studio'] = 'not a studio instance'
-        self.assertRaises(TypeError, SchedulerBase, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            SchedulerBase(**self.kwargs)
+
+        assert str(cm.value) == \
+            'SchedulerBase.studio should be an instance of ' \
+            'stalker.models.studio.Studio, not str'
 
     def test_studio_attribute_is_not_a_Studio_instance(self):
         """testing if a TypeError will be raised when the studio attribute is
         set to a value which is not a stalker.models.studio.Studio instance
         """
-        self.assertRaises(TypeError, setattr, self.test_scheduler_base,
-                          'studio', 'not a studio instance')
+        with pytest.raises(TypeError) as cm:
+            self.test_scheduler_base.studio = 'not a studio instance'
+
+        assert str(cm.value) == \
+            'SchedulerBase.studio should be an instance of ' \
+            'stalker.models.studio.Studio, not str'
 
     def test_studio_argument_is_working_properly(self):
         """testing if the studio argument value is correctly passed to the
         studio attribute
         """
-        self.assertEqual(self.test_scheduler_base.studio,
-                         self.kwargs['studio'])
+        assert self.test_scheduler_base.studio == self.kwargs['studio']
 
     def test_studio_attribute_is_working_properly(self):
         """testing if the studio attribute is working properly
         """
+        from stalker import Studio
         new_studio = Studio(name='Test Studio 2')
         self.test_scheduler_base.studio = new_studio
-        self.assertEqual(self.test_scheduler_base.studio, new_studio)
+        assert self.test_scheduler_base.studio == new_studio
+
+    def test_schedule_method_will_raise_not_implemented_error(self):
+        """testing if the schedule() method will raise a NotImplementedError
+        """
+        base = SchedulerBase()
+        with pytest.raises(NotImplementedError) as cm:
+            base.schedule()
+
+        assert str(cm.value) == ''
