@@ -1,38 +1,26 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
 import unittest
-from stalker import Structure, FilenameTemplate, Type
 
+import pytest
 
-# mock classes
-class Asset(object):
-    pass
-
-
-class Shot(object):
-    pass
-
-
-class Link(object):
-    pass
+from stalker import Structure
 
 
 class StructureTester(unittest.TestCase):
@@ -42,7 +30,9 @@ class StructureTester(unittest.TestCase):
     def setUp(self):
         """setting up the tests
         """
+        super(StructureTester, self).setUp()
 
+        from stalker import Type
         vers_type = Type(
             name="Version",
             code='vers',
@@ -56,6 +46,7 @@ class StructureTester(unittest.TestCase):
         )
 
         # type templates
+        from stalker import FilenameTemplate
         self.asset_template = FilenameTemplate(
             name="Test Asset Template",
             target_entity_type="Asset",
@@ -85,7 +76,7 @@ class StructureTester(unittest.TestCase):
         self.test_type = Type(
             name="Commercial Structure",
             code='comm',
-            target_entity_type=Structure,
+            target_entity_type='Structure',
         )
 
         # keyword arguments
@@ -102,14 +93,14 @@ class StructureTester(unittest.TestCase):
         """testing if the __auto_name__ class attribute is set to False for
         Structure class
         """
-        self.assertFalse(Structure.__auto_name__)
+        assert Structure.__auto_name__ is False
 
     def test_custom_template_argument_can_be_skipped(self):
         """testing if the custom_template argument can be skipped
         """
         self.kwargs.pop("custom_template")
         new_structure = Structure(**self.kwargs)
-        self.assertEqual(new_structure.custom_template, "")
+        assert new_structure.custom_template == ""
 
     def test_custom_template_argument_is_None(self):
         """testing if no error will be raised when the custom_template argument
@@ -117,7 +108,7 @@ class StructureTester(unittest.TestCase):
         """
         self.kwargs["custom_template"] = None
         new_structure = Structure(**self.kwargs)
-        self.assertEqual(new_structure.custom_template, "")
+        assert new_structure.custom_template == ""
 
     def test_custom_template_argument_is_empty_string(self):
         """testing if no error will be raised when the custom_template argument
@@ -125,21 +116,28 @@ class StructureTester(unittest.TestCase):
         """
         self.kwargs["custom_template"] = ""
         new_structure = Structure(**self.kwargs)
-        self.assertEqual(new_structure.custom_template, "")
+        assert new_structure.custom_template == ""
 
     def test_custom_template_argument_is_not_a_string(self):
         """testing if a TypeError will be raised when the custom_template
         argument is not a string
         """
         self.kwargs['custom_template'] = ["this is not a string"]
-        self.assertRaises(TypeError, Structure, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            Structure(**self.kwargs)
+
+        assert str(cm.value) == \
+            'Structure.custom_template should be a string not list'
 
     def test_custom_template_attribute_is_not_a_string(self):
         """testing if a TypeError will be raised when the custom_template
         attribute is not a string
         """
-        self.assertRaises(TypeError, setattr, self.test_structure,
-                          'custom_template', ['this is not a string'])
+        with pytest.raises(TypeError) as cm:
+            self.test_structure.custom_template = ['this is not a string']
+
+        assert str(cm.value) == \
+            'Structure.custom_template should be a string not list'
 
     def test_templates_argument_can_be_skipped(self):
         """testing if no error will be raised when the templates argument is
@@ -147,7 +145,7 @@ class StructureTester(unittest.TestCase):
         """
         self.kwargs.pop("templates")
         new_structure = Structure(**self.kwargs)
-        self.assertTrue(isinstance(new_structure, Structure))
+        assert isinstance(new_structure, Structure)
 
     def test_templates_argument_can_be_None(self):
         """testing if no error will be raised when the templates argument is
@@ -155,30 +153,45 @@ class StructureTester(unittest.TestCase):
         """
         self.kwargs["templates"] = None
         new_structure = Structure(**self.kwargs)
-        self.assertTrue(isinstance(new_structure, Structure))
+        assert isinstance(new_structure, Structure)
 
     def test_templates_attribute_cannot_be_set_to_None(self):
         """testing if a TypeError will be raised when the templates attribute
         is set to None
         """
-        self.assertRaises(TypeError, setattr, self.test_structure, "templates",
-                          None)
+        with pytest.raises(TypeError) as cm:
+            self.test_structure.templates = None
+
+        assert str(cm.value) == \
+            'Incompatible collection type: None is not list-like'
 
     def test_templates_argument_only_accepts_list(self):
         """testing if a TypeError will be raised when the given templates
         argument is not a list
         """
         self.kwargs["templates"] = 1
-        self.assertRaises(TypeError, Structure, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            Structure(**self.kwargs)
 
-    def test_templates_attribute_only_accepts_list(self):
+        assert str(cm.value) == \
+            'Incompatible collection type: int is not list-like'
+
+    def test_templates_attribute_only_accepts_list_1(self):
         """testing if a TypeError will be raised when the templates attribute
         is tried to be set to an object which is not a list instance.
         """
-        self.assertRaises(TypeError, setattr, self.test_structure, "templates",
-                          1.121)
+        with pytest.raises(TypeError) as cm:
+            self.test_structure.templates = 1.121
+
+        assert str(cm.value) == \
+            'Incompatible collection type: float is not list-like'
+
+    def test_templates_attribute_is_working_properly(self):
+        """testing if templates attribute is working properly
+        """
         # test the correct value
         self.test_structure.templates = self.test_templates
+        assert self.test_structure.templates == self.test_templates
 
     def test_templates_argument_accepts_only_list_of_FilenameTemplate_instances(self):
         """testing if a TypeError will be raised when the templates argument is
@@ -187,12 +200,21 @@ class StructureTester(unittest.TestCase):
         """
         test_value = [1, 1.2, "a string"]
         self.kwargs["templates"] = test_value
-        self.assertRaises(TypeError, Structure, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            Structure(**self.kwargs)
 
+        assert str(cm.value) == \
+            'All the elements in the Structure.templates should be a ' \
+            'stalker.models.template.FilenameTemplate instance not int'
+
+    def test_templates_argument_is_working_properly(self):
+        """testing if the templates argument value is correctly passed to the
+        templates attribute
+        """
         # test the correct value
         self.kwargs["templates"] = self.test_templates
         new_structure = Structure(**self.kwargs)
-        self.assertTrue(isinstance(new_structure, Structure))
+        assert new_structure.templates ==self.test_templates
 
     def test_templates_attribute_accpets_only_list_of_FilenameTemplate_instances(self):
         """testing if a TypeError will be raised when the templates attribute
@@ -200,13 +222,17 @@ class StructureTester(unittest.TestCase):
         class.
         """
         test_value = [1, 1.2, "a string"]
-        self.assertRaises(TypeError, setattr, self.test_structure, "templates",
-                          test_value)
+        with pytest.raises(TypeError) as cm:
+            self.test_structure.templates = test_value
+
+        assert str(cm.value) == \
+            'All the elements in the Structure.templates should be a ' \
+            'stalker.models.template.FilenameTemplate instance not int'
 
     def test___strictly_typed___is_False(self):
         """testing if the __strictly_typed__ is False
         """
-        self.assertEqual(Structure.__strictly_typed__, False)
+        assert Structure.__strictly_typed__ is False
 
     def test_equality_operator(self):
         """testing equality of two Structure objects
@@ -220,9 +246,9 @@ class StructureTester(unittest.TestCase):
         self.kwargs["templates"] = self.test_templates2
         new_structure4 = Structure(**self.kwargs)
 
-        self.assertTrue(self.test_structure == new_structure2)
-        self.assertFalse(self.test_structure == new_structure3)
-        self.assertFalse(self.test_structure == new_structure4)
+        assert self.test_structure == new_structure2
+        assert not self.test_structure == new_structure3
+        assert not self.test_structure == new_structure4
 
     def test_inequality_operator(self):
         """testing inequality of two Structure objects
@@ -236,11 +262,20 @@ class StructureTester(unittest.TestCase):
         self.kwargs["templates"] = self.test_templates2
         new_structure4 = Structure(**self.kwargs)
 
-        self.assertFalse(self.test_structure != new_structure2)
-        self.assertTrue(self.test_structure != new_structure3)
-        self.assertTrue(self.test_structure != new_structure4)
+        assert not self.test_structure != new_structure2
+        assert self.test_structure != new_structure3
+        assert self.test_structure != new_structure4
 
     def test_plural_class_name(self):
         """testing the plural name of Structure class
         """
-        self.assertTrue(self.test_structure.plural_class_name, "Structures")
+        assert self.test_structure.plural_class_name == "Structures"
+
+    # def test_hash_value(self):
+    #     """testing if the hash value is correctly calculated
+    #     """
+    #     assert \
+    #         hash(self.test_structure) == \
+    #         hash(self.test_structure.id) +
+    #         2 * hash(self.test_structure.name) +
+    #         3 * hash(self.test_structure.entity_type)
