@@ -1,60 +1,45 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-import datetime
-import unittest
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 import os
 
-import stalker
-from stalker.db import DBSession
-from stalker import (db, Department, User, Repository, Status, StatusList,
-                     Project, Task, TaskJugglerScheduler, Studio)
+import pytest
+import pytz
+import datetime
+from stalker import TaskJugglerScheduler
+from stalker.testing import UnitTestDBBase
 
 
-class TaskJugglerSchedulerTester(unittest.TestCase):
+class TaskJugglerSchedulerDBTester(UnitTestDBBase):
     """tests the stalker.models.scheduler.TaskJugglerScheduler class
     """
 
-    @classmethod
-    def setUpClass(cls):
-        """setup tests in class level
-        """
-        cls.config = {
-            'sqlalchemy.url': 'sqlite:///:memory:',
-            'sqlalchemy.echo': False
-        }
-
     def setUp(self):
         """set up the test
         """
-        # we need a database
-        db.setup(self.config)
-        db.init()
-
-        # replace datetime now function
+        super(self.__class__, self).setUp()
 
         # create departments
+        from stalker import Department
         self.test_dep1 = Department(name='Dep1')
         self.test_dep2 = Department(name='Dep2')
 
         # create resources
+        from stalker import User
         self.test_user1 = User(
             login='user1',
             name='User1',
@@ -62,6 +47,7 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
             password='1234',
             departments=[self.test_dep1]
         )
+        from stalker.db.session import DBSession
         DBSession.add(self.test_user1)
 
         self.test_user2 = User(
@@ -111,8 +97,10 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
         DBSession.add(self.test_user6)
 
         # repository
+        from stalker import Repository
         self.test_repo = Repository(
             name='Test Repository',
+            code='TR',
             linux_path='/mnt/T/',
             windows_path='T:/',
             osx_path='/Volumes/T/'
@@ -120,43 +108,31 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
         DBSession.add(self.test_repo)
 
         # statuses
+        from stalker import Status
         self.test_status1 = Status(name='Status 1', code='STS1')
         self.test_status2 = Status(name='Status 2', code='STS2')
         self.test_status3 = Status(name='Status 3', code='STS3')
         self.test_status4 = Status(name='Status 4', code='STS4')
         self.test_status5 = Status(name='Status 5', code='STS5')
-        DBSession.add_all([self.test_status1,
-                           self.test_status2,
-                           self.test_status3,
-                           self.test_status4,
-                           self.test_status5])
-
-        # status lists
-        self.test_proj_status_list = StatusList(
-            name='Project Status List',
-            statuses=[self.test_status1, self.test_status2, self.test_status3],
-            target_entity_type='Project'
-        )
-        DBSession.add(self.test_proj_status_list)
+        DBSession.add_all([
+            self.test_status1, self.test_status2, self.test_status3,
+            self.test_status4, self.test_status5
+        ])
 
         # create one project
+        from stalker import Project
         self.test_proj1 = Project(
             name='Test Project 1',
             code='TP1',
             repository=self.test_repo,
-            status_list=self.test_proj_status_list,
-            start=datetime.datetime(2013, 4, 4),
-            end=datetime.datetime(2013, 5, 4)
+            start=datetime.datetime(2013, 4, 4, tzinfo=pytz.utc),
+            end=datetime.datetime(2013, 5, 4, tzinfo=pytz.utc)
         )
         DBSession.add(self.test_proj1)
-        self.test_proj1.now = datetime.datetime(2013, 4, 4)
-
-        # create task status list
-        with DBSession.no_autoflush:
-            self.test_task_status_list = StatusList.query\
-                .filter_by(target_entity_type='Task').first()
+        self.test_proj1.now = datetime.datetime(2013, 4, 4, tzinfo=pytz.utc)
 
         # create two tasks with the same resources
+        from stalker import Task
         self.test_task1 = Task(
             name='Task1',
             project=self.test_proj1,
@@ -167,7 +143,6 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
             schedule_model=0,
             schedule_timing=50,
             schedule_unit='h',
-            status_list=self.test_task_status_list
         )
         DBSession.add(self.test_task1)
 
@@ -178,18 +153,13 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
             alternative_resources=[
                 self.test_user3, self.test_user4, self.test_user5
             ],
+            depends=[self.test_task1],
             schedule_model=0,
             schedule_timing=60,
             schedule_unit='h',
-            status_list=self.test_task_status_list
+            priority=800
         )
-        DBSession.add(self.test_task2)
-        DBSession.commit()
-
-    def tearDown(self):
-        """clean up the test
-        """
-        DBSession.remove()
+        DBSession.save(self.test_task2)
 
     def test_tjp_file_is_created(self):
         """testing if the tjp file is correctly created
@@ -203,7 +173,7 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
         tjp_sched._fill_tjp_file()
 
         # check
-        self.assertTrue(os.path.exists(tjp_sched.tjp_file_full_path))
+        assert os.path.exists(tjp_sched.tjp_file_full_path)
 
         # clean up the test
         tjp_sched._clean_up()
@@ -211,7 +181,20 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
     def test_tjp_file_content_is_correct(self):
         """testing if the tjp file content is correct
         """
+        # enter a couple of time logs
+        from stalker import TimeLog
+        tlog1 = TimeLog(
+            resource=self.test_user1,
+            task=self.test_task1,
+            start=datetime.datetime(2013, 4, 16, 6, 0, tzinfo=pytz.utc),
+            end=datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc)
+        )
+        from stalker.db.session import DBSession
+        DBSession.add(tlog1)
+        DBSession.commit()
+
         tjp_sched = TaskJugglerScheduler()
+        from stalker import Studio
         test_studio = Studio(
             name='Test Studio',
             timing_resolution=datetime.timedelta(minutes=30)
@@ -219,16 +202,18 @@ class TaskJugglerSchedulerTester(unittest.TestCase):
         test_studio.daily_working_hours = 9
 
         test_studio.id = 564
-        test_studio.start = datetime.datetime(2013, 4, 16, 0, 7)
-        test_studio.end = datetime.datetime(2013, 6, 30, 0, 0)
-        test_studio.now = datetime.datetime(2013, 4, 16, 0, 0)
+        test_studio.start = \
+            datetime.datetime(2013, 4, 16, 0, 7, tzinfo=pytz.utc)
+        test_studio.end = datetime.datetime(2013, 6, 30, 0, 0, tzinfo=pytz.utc)
+        test_studio.now = datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
         tjp_sched.studio = test_studio
 
         tjp_sched._create_tjp_file()
         tjp_sched._create_tjp_file_content()
 
-        import jinja2
+        assert TimeLog.query.all() != []
 
+        import jinja2
         expected_tjp_template = jinja2.Template(
             """# Generated By Stalker v{{stalker.__version__}}
         
@@ -274,58 +259,34 @@ project Studio_564 "Studio_564" 2013-04-16 - 2013-06-30 {
 }
         }
 
-        # tasks
-        
-            
+# tasks
 task Project_{{proj.id}} "Project_{{proj.id}}" {
-        
-task Task_{{task1.id}} "Task_{{task1.id}}" {
+  task Task_{{task1.id}} "Task_{{task1.id}}" {
+    effort 50.0h
+    allocate User_{{user1.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }, User_{{user2.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }
+    booking User_{{user1.id}} 2013-04-16-06:00:00 - 2013-04-16-09:00:00 { overtime 2 }
+  }
+  task Task_{{task2.id}} "Task_{{task2.id}}" {
+    priority 800
+    depends Project_{{proj.id}}.Task_{{task1.id}} {onend}
+    effort 60.0h
+    allocate User_{{user1.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }, User_{{user2.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }
+  }
+}
 
-    
-            
-            
-            effort 50.0h
-            allocate User_{{user1.id}} {
-                    alternative
-                    User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated
-                    persistent
-                }, User_{{user2.id}} {
-                    alternative
-                    User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated
-                    persistent
-                }            
-}        
-task Task_{{task2.id}} "Task_{{task2.id}}" {
-
-    
-            
-            
-            effort 60.0h
-            allocate User_{{user1.id}} {
-                    alternative
-                    User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated
-                    persistent
-                }, User_{{user2.id}} {
-                    alternative
-                    User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated
-                    persistent
-                }            
-}}
-        
-
-        # reports
-        taskreport breakdown "{{csv_path}}"{
-            formats csv
-            timeformat "%Y-%m-%d-%H:%M"
-            columns id, start, end
-        }
-        
+# reports
+taskreport breakdown "{{csv_path}}"{
+    formats csv
+    timeformat "%Y-%m-%d-%H:%M"
+    columns id, start, end
+}
 """)
+        import stalker
         expected_tjp_content = expected_tjp_template.render(
             {
                 'stalker': stalker,
                 'studio': test_studio,
-                'csv_path': tjp_sched.temp_file_full_path,
+                'csv_path': tjp_sched.temp_file_name,
                 'user1': self.test_user1,
                 'user2': self.test_user2,
                 'user3': self.test_user3,
@@ -342,7 +303,10 @@ task Task_{{task2.id}} "Task_{{task2.id}}" {
         tjp_content = tjp_sched.tjp_content
         # print tjp_content
         tjp_sched._clean_up()
-        self.assertEqual(tjp_content, expected_tjp_content)
+        # print(expected_tjp_content)
+        # print('----------------')
+        # print(tjp_content)
+        assert tjp_content == expected_tjp_content
 
     def test_schedule_will_not_work_when_the_studio_attribute_is_None(self):
         """testing if a TypeError will be raised when the studio attribute is
@@ -350,174 +314,191 @@ task Task_{{task2.id}} "Task_{{task2.id}}" {
         """
         tjp_sched = TaskJugglerScheduler()
         tjp_sched.studio = None
-        self.assertRaises(TypeError, tjp_sched.schedule)
+        with pytest.raises(TypeError) as cm:
+            tjp_sched.schedule()
+
+        assert str(cm.value) == \
+            'TaskJugglerScheduler.studio should be an instance of ' \
+            'stalker.models.studio.Studio, not NoneType'
 
     def test_tasks_are_correctly_scheduled(self):
         """testing if the tasks are correctly scheduled
         """
         tjp_sched = TaskJugglerScheduler(compute_resources=True)
-        test_studio = Studio(name='Test Studio',
-                             now=datetime.datetime(2013, 4, 16, 0, 0))
-        test_studio.start = datetime.datetime(2013, 4, 16, 0, 0)
-        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0)
+        from stalker import Studio
+        test_studio = Studio(
+            name='Test Studio',
+            now=datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        )
+        test_studio.start = \
+            datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        test_studio.end = \
+            datetime.datetime(2013, 4, 30, 0, 0, tzinfo=pytz.utc)
         test_studio.daily_working_hours = 9
+        from stalker.db.session import DBSession
         DBSession.add(test_studio)
 
         tjp_sched.studio = test_studio
         tjp_sched.schedule()
-        db.DBSession.commit()
+        DBSession.commit()
 
         # check if the task and project timings are all adjusted
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc) == \
             self.test_proj1.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 19, 12, 0),
+        assert \
+            datetime.datetime(2013, 4, 24, 10, 0, tzinfo=pytz.utc) == \
             self.test_proj1.computed_end
-        )
 
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        possible_resources = [
+            self.test_user1, self.test_user2, self.test_user3, self.test_user4,
+            self.test_user5
+        ]
+        assert \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc) == \
             self.test_task1.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 18, 16, 0),
+        assert \
+            datetime.datetime(2013, 4, 18, 16, 0, tzinfo=pytz.utc) == \
             self.test_task1.computed_end
-        )
-        self.assertEqual(
-            sorted([self.test_user5, self.test_user4], key=lambda x: x.name),
-            sorted(self.test_task1.computed_resources, key=lambda x: x.name)
-        )
 
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert len(self.test_task1.computed_resources) == 2
+        assert self.test_task1.computed_resources[0] in possible_resources
+        assert self.test_task1.computed_resources[1] in possible_resources
+
+        assert \
+            datetime.datetime(2013, 4, 18, 16, 0, tzinfo=pytz.utc) == \
             self.test_task2.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 19, 12, 0),
+        assert \
+            datetime.datetime(2013, 4, 24, 10, 0, tzinfo=pytz.utc) == \
             self.test_task2.computed_end
-        )
-        self.assertEqual(
-            sorted([self.test_user1, self.test_user2], key=lambda x: x.name),
-            sorted(self.test_task2.computed_resources, key=lambda x: x.name)
-        )
+
+        assert len(self.test_task2.computed_resources) == 2
+        assert self.test_task2.computed_resources[0] in possible_resources
+        assert self.test_task2.computed_resources[1] in possible_resources
 
     def test_tasks_are_correctly_scheduled_when_compute_resources_is_False(self):
         """testing if the tasks are correctly scheduled when the compute
         resources is False
         """
         tjp_sched = TaskJugglerScheduler(compute_resources=False)
-        test_studio = Studio(name='Test Studio',
-                             now=datetime.datetime(2013, 4, 16, 0, 0))
-        test_studio.start = datetime.datetime(2013, 4, 16, 0, 0)
-        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0)
+        from stalker import Studio
+        test_studio = Studio(
+            name='Test Studio',
+            now=datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        )
+        test_studio.start = \
+            datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0, tzinfo=pytz.utc)
         test_studio.daily_working_hours = 9
+        from stalker.db.session import DBSession
         DBSession.add(test_studio)
 
         tjp_sched.studio = test_studio
         tjp_sched.schedule()
-        db.DBSession.commit()
+        DBSession.commit()
 
         # check if the task and project timings are all adjusted
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc) == \
             self.test_proj1.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 19, 12, 0),
+        assert \
+            datetime.datetime(2013, 4, 24, 10, 0, tzinfo=pytz.utc) == \
             self.test_proj1.computed_end
-        )
 
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc) == \
             self.test_task1.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 18, 16, 0),
+        assert \
+            datetime.datetime(2013, 4, 18, 16, 0, tzinfo=pytz.utc) == \
             self.test_task1.computed_end
-        )
-        self.assertEqual(
-            sorted(self.test_task1.resources, key=lambda x: x.name),
-            sorted(self.test_task1.computed_resources, key=lambda x: x.name)
-        )
+        assert len(self.test_task1.computed_resources) == 2
+        assert self.test_task1.computed_resources[0] in \
+            [self.test_user1, self.test_user2, self.test_user3,
+             self.test_user4, self.test_user5]
+        assert self.test_task1.computed_resources[1] in \
+            [self.test_user1, self.test_user2, self.test_user3,
+             self.test_user4, self.test_user5]
 
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 18, 16, 0, tzinfo=pytz.utc) == \
             self.test_task2.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 19, 12, 0),
+        assert \
+            datetime.datetime(2013, 4, 24, 10, 0, tzinfo=pytz.utc) == \
             self.test_task2.computed_end
-        )
-        self.assertEqual(
-            sorted(self.test_task2.resources, key=lambda x: x.name),
-            sorted(self.test_task2.computed_resources, key=lambda x: x.name)
-        )
+        assert len(self.test_task2.computed_resources) == 2
+        assert self.test_task2.computed_resources[0] in \
+            [self.test_user1, self.test_user2, self.test_user3,
+             self.test_user4, self.test_user5]
+        assert self.test_task2.computed_resources[1] in \
+            [self.test_user1, self.test_user2, self.test_user3,
+             self.test_user4, self.test_user5]
 
     def test_tasks_are_correctly_scheduled_when_compute_resources_is_True(self):
         """testing if the tasks are correctly scheduled when the compute
         resources is True
         """
         tjp_sched = TaskJugglerScheduler(compute_resources=True)
-        test_studio = Studio(name='Test Studio',
-                             now=datetime.datetime(2013, 4, 16, 0, 0))
-        test_studio.start = datetime.datetime(2013, 4, 16, 0, 0)
-        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0)
+        from stalker import Studio
+        test_studio = Studio(
+            name='Test Studio',
+            now=datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        )
+        test_studio.start = \
+            datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0, tzinfo=pytz.utc)
         test_studio.daily_working_hours = 9
+        from stalker.db.session import DBSession
         DBSession.add(test_studio)
 
         tjp_sched.studio = test_studio
         tjp_sched.schedule()
-        db.DBSession.commit()
+        DBSession.commit()
+
+        possible_resources = [
+            self.test_user1, self.test_user2, self.test_user3, self.test_user4,
+            self.test_user5
+        ]
 
         # check if the task and project timings are all adjusted
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc) == \
             self.test_proj1.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 19, 12, 0),
+        assert \
+            datetime.datetime(2013, 4, 24, 10, 0, tzinfo=pytz.utc) == \
             self.test_proj1.computed_end
-        )
 
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc) == \
             self.test_task1.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 18, 16, 0),
+        assert \
+            datetime.datetime(2013, 4, 18, 16, 0, tzinfo=pytz.utc) == \
             self.test_task1.computed_end
-        )
-        self.assertEqual(
-            sorted([self.test_user4, self.test_user5], key=lambda x: x.name),
-            sorted(self.test_task1.computed_resources, key=lambda x: x.name)
-        )
+        assert len(self.test_task1.computed_resources) == 2
+        assert self.test_task1.computed_resources[0] in possible_resources
+        assert self.test_task1.computed_resources[1] in possible_resources
 
-        self.assertEqual(
-            datetime.datetime(2013, 4, 16, 9, 0),
+        assert \
+            datetime.datetime(2013, 4, 18, 16, 0, tzinfo=pytz.utc) == \
             self.test_task2.computed_start
-        )
-        self.assertEqual(
-            datetime.datetime(2013, 4, 19, 12, 0),
+        assert \
+            datetime.datetime(2013, 4, 24, 10, 0, tzinfo=pytz.utc) == \
             self.test_task2.computed_end
-        )
-        self.assertEqual(
-            sorted([self.test_user1, self.test_user2], key=lambda x: x.name),
-            sorted(self.test_task2.computed_resources, key=lambda x: x.name)
-        )
+        assert self.test_task2.computed_resources[0] in possible_resources
+        assert self.test_task2.computed_resources[1] in possible_resources
 
     def test_tasks_of_given_projects_are_correctly_scheduled(self):
         """testing if the tasks of given projects are correctly scheduled
         """
         # create a dummy project
         # create a dummy Project to schedule
+        from stalker import Project
         dummy_project = Project(
             name='Dummy Project',
             code='DP',
             repository=self.test_repo
         )
 
+        from stalker import Task
         dt1 = Task(
             name='Dummy Task 1',
             project=dummy_project,
@@ -533,60 +514,69 @@ task Task_{{task2.id}} "Task_{{task2.id}}" {
             schedule_unit='h',
             resources=[self.test_user2]
         )
-        db.DBSession.add_all([dummy_project, dt1, dt2])
-        db.DBSession.commit()
+        from stalker.db.session import DBSession
+        DBSession.add_all([dummy_project, dt1, dt2])
+        DBSession.commit()
 
         tjp_sched = TaskJugglerScheduler(compute_resources=True,
                                          projects=[dummy_project])
-        test_studio = Studio(name='Test Studio',
-                             now=datetime.datetime(2013, 4, 16, 0, 0))
-        test_studio.start = datetime.datetime(2013, 4, 16, 0, 0)
-        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0)
+        from stalker import Studio
+        test_studio = Studio(
+            name='Test Studio',
+            now=datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        )
+        test_studio.start = \
+            datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
+        test_studio.end = datetime.datetime(2013, 4, 30, 0, 0, tzinfo=pytz.utc)
         test_studio.daily_working_hours = 9
         DBSession.add(test_studio)
-        db.DBSession.commit()
+        DBSession.commit()
 
         tjp_sched.studio = test_studio
         tjp_sched.schedule()
-        db.DBSession.commit()
+        DBSession.commit()
 
         # check if the task and project timings are all adjusted
-        self.assertEqual(self.test_proj1.computed_start, None)
-        self.assertEqual(self.test_proj1.computed_end, None)
+        assert self.test_proj1.computed_start is None
+        assert self.test_proj1.computed_end is None
 
-        self.assertEqual(self.test_task1.computed_start, None)
-        self.assertEqual(self.test_task1.computed_end, None)
-        self.assertEqual(self.test_task1.computed_resources,
-                         [self.test_user1, self.test_user2])
+        assert self.test_task1.computed_start is None
+        assert self.test_task1.computed_end is None
+        assert self.test_task1.computed_resources == \
+            [self.test_user1, self.test_user2]
 
-        self.assertEqual(self.test_task2.computed_start, None)
-        self.assertEqual(self.test_task2.computed_end, None)
-        self.assertEqual(self.test_task2.computed_resources,
-                         [self.test_user1, self.test_user2])
+        assert self.test_task2.computed_start is None
+        assert self.test_task2.computed_end is None
+        assert self.test_task2.computed_resources == \
+            [self.test_user1, self.test_user2]
 
-        self.assertEqual(dt1.computed_start,
-                         datetime.datetime(2013, 4, 16, 9, 0))
-        self.assertEqual(dt1.computed_end,
-                         datetime.datetime(2013, 4, 16, 13, 0))
+        assert \
+            dt1.computed_start == \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc)
+        assert \
+            dt1.computed_end == \
+            datetime.datetime(2013, 4, 16, 13, 0, tzinfo=pytz.utc)
 
-        self.assertEqual(dt2.computed_start,
-                         datetime.datetime(2013, 4, 16, 9, 0))
-        self.assertEqual(dt2.computed_end,
-                         datetime.datetime(2013, 4, 16, 13, 0))
+        assert \
+            dt2.computed_start == \
+            datetime.datetime(2013, 4, 16, 9, 0, tzinfo=pytz.utc)
+        assert \
+            dt2.computed_end == \
+            datetime.datetime(2013, 4, 16, 13, 0, tzinfo=pytz.utc)
 
     def test_projects_argument_is_skipped(self):
         """testing if the projects attribute will be an empty list if the
         projects argument is skipped
         """
         tjp_sched = TaskJugglerScheduler(compute_resources=True)
-        self.assertEqual(tjp_sched.projects, [])
+        assert tjp_sched.projects == []
 
     def test_projects_argument_is_None(self):
         """testing if the projects attribute will be an empty list if the
         projects argument is None
         """
         tjp_sched = TaskJugglerScheduler(compute_resources=True, projects=None)
-        self.assertEqual(tjp_sched.projects, [])
+        assert tjp_sched.projects == []
 
     def test_projects_attribute_is_set_to_None(self):
         """testing if the projects attribute will be an empty list if it is set
@@ -594,68 +584,61 @@ task Task_{{task2.id}} "Task_{{task2.id}}" {
         """
         tjp_sched = TaskJugglerScheduler(compute_resources=True)
         tjp_sched.projects = None
-        self.assertEqual(tjp_sched.projects, [])
+        assert tjp_sched.projects == []
 
     def test_projects_argument_is_not_a_list(self):
         """testing if a TypeError will be raised when the projects argument
         value is not a list
         """
-        with self.assertRaises(TypeError) as cm:
+        with pytest.raises(TypeError) as cm:
             TaskJugglerScheduler(compute_resources=True,
                                  projects='not a list of projects')
 
-        self.assertEqual(
-            str(cm.exception),
-            'TaskJugglerScheduler.projects should be a list of '
+        assert str(cm.value) == \
+            'TaskJugglerScheduler.projects should be a list of ' \
             'stalker.models.project.Project instances, not str'
-        )
 
     def test_projects_attribute_is_not_a_list(self):
         """testing if a TypeError will be raised when the projects attribute
         is set to something else than a list
         """
         tjp = TaskJugglerScheduler(compute_resources=True)
-        with self.assertRaises(TypeError) as cm:
+        with pytest.raises(TypeError) as cm:
             tjp.projects = 'not a list of projects'
 
-        self.assertEqual(
-            str(cm.exception),
-            'TaskJugglerScheduler.projects should be a list of '
+        assert str(cm.value) == \
+            'TaskJugglerScheduler.projects should be a list of ' \
             'stalker.models.project.Project instances, not str'
-        )
 
     def test_projects_argument_is_not_a_list_of_all_projects(self):
         """testing if a TypeError will be raised when the elements in the
         projects argument is not all Project instances
         """
-        with self.assertRaises(TypeError) as cm:
+        with pytest.raises(TypeError) as cm:
             TaskJugglerScheduler(compute_resources=True,
                                  projects=['not', 1, [], 'of', 'projects'])
 
-        self.assertEqual(
-            str(cm.exception),
-            'TaskJugglerScheduler.projects should be a list of '
+        assert str(cm.value) == \
+            'TaskJugglerScheduler.projects should be a list of ' \
             'stalker.models.project.Project instances, not str'
-        )
 
     def test_projects_attribute_is_not_list_of_all_projects(self):
         """testing if a TypeError will be raised when the elements in the
         projects attribute is not all Project instances
         """
         tjp = TaskJugglerScheduler(compute_resources=True)
-        with self.assertRaises(TypeError) as cm:
+        with pytest.raises(TypeError) as cm:
             tjp.projects = ['not', 1, [], 'of', 'projects']
 
-        self.assertEqual(
-            str(cm.exception),
-            'TaskJugglerScheduler.projects should be a list of '
+        assert str(cm.value) == \
+            'TaskJugglerScheduler.projects should be a list of ' \
             'stalker.models.project.Project instances, not str'
-        )
 
     def test_projects_argument_is_working_properly(self):
         """testing if the projects argument value is correctly passed to the
         projects attribute
         """
+        from stalker import Project
         dp1 = Project(
             name='Dummy Project',
             code='DP',
@@ -671,11 +654,12 @@ task Task_{{task2.id}} "Task_{{task2.id}}" {
         tjp = TaskJugglerScheduler(compute_resources=True,
                                    projects=[dp1, dp2])
 
-        self.assertEqual(tjp.projects, [dp1, dp2])
+        assert tjp.projects == [dp1, dp2]
 
     def test_projects_attribute_is_working_properly(self):
         """testing if the projects attribute is working properly
         """
+        from stalker import Project
         dp1 = Project(
             name='Dummy Project',
             code='DP',
@@ -691,54 +675,13 @@ task Task_{{task2.id}} "Task_{{task2.id}}" {
         tjp = TaskJugglerScheduler(compute_resources=True)
         tjp.projects = [dp1, dp2]
 
-        self.assertEqual(tjp.projects, [dp1, dp2])
+        assert tjp.projects == [dp1, dp2]
 
-
-class TaskJugglerScheduler_PostgreSQL_Tester(TaskJugglerSchedulerTester):
-    """tests the stalker.models.scheduler.TaskJugglerScheduler class with
-    PostgreSQL
-    """
-
-    @classmethod
-    def setUpClass(cls):
-        """setup tests in class level
-        """
-        cls.config = {
-            'sqlalchemy.url':
-                'postgresql://stalker_admin:stalker@localhost/stalker_test',
-            'sqlalchemy.echo': False
-        }
-        # clean up test database
-        db.setup(cls.config)
-        from stalker.db.declarative import Base
-        Base.metadata.drop_all(db.DBSession.connection())
-        DBSession.commit()
-
-    def setUp(self):
-        """set up
-        """
-        from stalker.db.declarative import Base
-        Base.metadata.drop_all(db.DBSession.connection())
-        DBSession.commit()
-
-        super(TaskJugglerScheduler_PostgreSQL_Tester, self).setUp()
-
-    def tearDown(self):
-        """clean up the test
-        """
-        # clean up test database
-        from stalker.db.declarative import Base
-        Base.metadata.drop_all(db.DBSession.connection())
-        from sqlalchemy.exc import ProgrammingError
-        try:
-            DBSession.commit()
-        except ProgrammingError:
-            DBSession.rollback()
-
-    def test_tjp_file_content_is_correct(self):
+    def test_tjp_file_content_is_correct_2(self):
         """testing if the tjp file content is correct
         """
         tjp_sched = TaskJugglerScheduler()
+        from stalker import Studio
         test_studio = Studio(
             name='Test Studio',
             timing_resolution=datetime.timedelta(minutes=30)
@@ -746,9 +689,12 @@ class TaskJugglerScheduler_PostgreSQL_Tester(TaskJugglerSchedulerTester):
         test_studio.daily_working_hours = 9
 
         test_studio.id = 564
-        test_studio.start = datetime.datetime(2013, 4, 16, 0, 7)
-        test_studio.end = datetime.datetime(2013, 6, 30, 0, 0)
-        test_studio.now = datetime.datetime(2013, 4, 16, 0, 0)
+        test_studio.start = \
+            datetime.datetime(2013, 4, 16, 0, 7, tzinfo=pytz.utc)
+        test_studio.end = \
+            datetime.datetime(2013, 6, 30, 0, 0, tzinfo=pytz.utc)
+        test_studio.now = \
+            datetime.datetime(2013, 4, 16, 0, 0, tzinfo=pytz.utc)
         tjp_sched.studio = test_studio
 
         tjp_sched._create_tjp_file()
@@ -808,6 +754,8 @@ task Project_{{proj1.id}} "Project_{{proj1.id}}" {
     allocate User_{{user1.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }, User_{{user2.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }
   }
   task Task_{{task2.id}} "Task_{{task2.id}}" {
+    priority 800
+    depends Project_{{proj1.id}}.Task_{{task1.id}} {onend}
     effort 60.0h
     allocate User_{{user1.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }, User_{{user2.id}} { alternative User_{{user3.id}}, User_{{user4.id}}, User_{{user5.id}} select minallocated persistent }
   }
@@ -819,11 +767,12 @@ taskreport breakdown "{{csv_path}}"{
     timeformat "%Y-%m-%d-%H:%M"
     columns id, start, end
 }""")
+        import stalker
         expected_tjp_content = expected_tjp_template.render(
             {
                 'stalker': stalker,
                 'studio': test_studio,
-                'csv_path': tjp_sched.temp_file_full_path,
+                'csv_path': tjp_sched.temp_file_name,
                 'user1': self.test_user1,
                 'user2': self.test_user2,
                 'user3': self.test_user3,
@@ -840,4 +789,4 @@ taskreport breakdown "{{csv_path}}"{
         tjp_content = tjp_sched.tjp_content
         # print tjp_content
         tjp_sched._clean_up()
-        self.assertEqual(tjp_content, expected_tjp_content)
+        assert tjp_content == expected_tjp_content
