@@ -1,27 +1,25 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2018 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation;
-# version 2.1 of the License.
+# Stalker is free software: you can redistribute it and/or modify
+# it under the terms of the Lesser GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License.
 #
-# This library is distributed in the hope that it will be useful,
+# Stalker is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# Lesser General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# Lesser GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+# You should have received a copy of the Lesser GNU General Public License
+# along with Stalker.  If not, see <http://www.gnu.org/licenses/>
 
-import datetime
 import unittest
 
-from stalker import User, Type
+import pytest
+
 from stalker.models.studio import Vacation
 
 
@@ -32,7 +30,10 @@ class VacationTestCase(unittest.TestCase):
     def setUp(self):
         """setup the test
         """
+        super(VacationTestCase, self).setUp()
+
         # create a user
+        from stalker import User
         self.test_user = User(
             name='Test User',
             login='testuser',
@@ -41,6 +42,7 @@ class VacationTestCase(unittest.TestCase):
         )
 
         # vacation type
+        from stalker import Type
         self.personal_vacation = Type(
             name='Personal',
             code='PERS',
@@ -53,11 +55,13 @@ class VacationTestCase(unittest.TestCase):
             target_entity_type='Vacation'
         )
 
+        import datetime
+        import pytz
         self.kwargs = {
             'user': self.test_user,
             'type': self.personal_vacation,
-            'start': datetime.datetime(2013, 6, 6, 10, 0),
-            'end': datetime.datetime(2013, 6, 10, 19, 0)
+            'start': datetime.datetime(2013, 6, 6, 10, 0, tzinfo=pytz.utc),
+            'end': datetime.datetime(2013, 6, 10, 19, 0, tzinfo=pytz.utc)
         }
 
         self.test_vacation = Vacation(**self.kwargs)
@@ -66,7 +70,7 @@ class VacationTestCase(unittest.TestCase):
         """testing if the __strictly_typed_ attribute is False for Vacation
         class
         """
-        self.assertEqual(Vacation.__strictly_typed__, False)
+        assert Vacation.__strictly_typed__ is False
 
     def test_user_argument_is_skipped(self):
         """testing if the user argument can be skipped skipped
@@ -90,24 +94,34 @@ class VacationTestCase(unittest.TestCase):
         a stalker.models.auth.User instance
         """
         self.kwargs['user'] = 'not a user instance'
-        self.assertRaises(TypeError, Vacation, **self.kwargs)
+        with pytest.raises(TypeError) as cm:
+            Vacation(**self.kwargs)
+
+        assert str(cm.value) == \
+            'Vacation.user should be an instance of ' \
+            'stalker.models.auth.User, not str'
 
     def test_user_attribute_is_not_a_User_instance(self):
         """testing if a TypeError will be raised when the user attribute is set
         to a value which is not a stalker.models.auth.User instance
         """
-        self.assertRaises(TypeError, setattr, self.test_vacation, 'user',
-                          'not a user instance')
+        with pytest.raises(TypeError) as cm:
+            self.test_vacation.user = 'not a user instance'
+
+        assert str(cm.value) == \
+            'Vacation.user should be an instance of ' \
+            'stalker.models.auth.User, not str'
 
     def test_user_argument_is_working_properly(self):
         """testing if the user argument value is correctly passed to the user
         attribute
         """
-        self.assertEqual(self.kwargs['user'], self.test_vacation.user)
+        assert self.test_vacation.user == self.kwargs['user']
 
     def test_user_attribute_is_working_properly(self):
         """testing if the user attribute is working properly
         """
+        from stalker import User
         new_user = User(
             name='test user 2',
             login='testuser2',
@@ -115,23 +129,21 @@ class VacationTestCase(unittest.TestCase):
             password='secret'
         )
 
-        self.assertNotEqual(self.test_vacation.user, new_user)
+        assert self.test_vacation.user != new_user
         self.test_vacation.user = new_user
-        self.assertEqual(self.test_vacation.user, new_user)
+        assert self.test_vacation.user == new_user
 
     def test_user_argument_back_populates_vacations_attribute(self):
         """testing if the user argument back populates vacations attribute of
         the User instance
         """
-        self.assertIn(
-            self.test_vacation,
-            self.kwargs['user'].vacations
-        )
+        assert self.test_vacation in self.kwargs['user'].vacations
 
     def test_user_attribute_back_populates_vacations_attribute(self):
         """testing if the user attribute back populates vacations attribute of
         the User instance
         """
+        from stalker import User
         new_user = User(
             name='test user 2',
             login='testuser2',
@@ -140,19 +152,19 @@ class VacationTestCase(unittest.TestCase):
         )
 
         self.test_vacation.user = new_user
-        self.assertIn(self.test_vacation, new_user.vacations)
+        assert self.test_vacation in new_user.vacations
 
     def test_to_tjp_attribute_is_a_read_only_property(self):
         """testing if the to_tjp is a read-only attribute
         """
-        self.assertRaises(AttributeError, setattr, self.test_vacation,
-                          'to_tjp', 'some value')
+        with pytest.raises(AttributeError) as cm:
+            self.test_vacation.to_tjp = 'some value'
+
+        assert str(cm.value) == "can't set attribute"
 
     def test_to_tjp_attribute_is_working_properly(self):
         """testing if the to_tjp attribute is working properly
         """
+        # TODO: Vacation should also use time zone info
         expected_tjp = "vacation 2013-06-06-10:00:00 - 2013-06-10-19:00:00"
-        self.assertEqual(
-            self.test_vacation.to_tjp,
-            expected_tjp
-        )
+        assert self.test_vacation.to_tjp == expected_tjp
